/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2012-2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef AST_CTRLFLOW_HPP
#define AST_CTRLFLOW_HPP

#include <list>

#include "../compilecontext.hpp"

class AstNodeStatementCtrlFlow : public AstNodeStatement
{
	public:
		AstNodeStatementCtrlFlow(AstNodeExpression *condition, AstNodeStatement *statement);
		virtual ~AstNodeStatementCtrlFlow();
		virtual AstNodeStatement *Normalize(CompileContext &context);
		
		virtual void Link(std::map<std::string, AstNode *> &node_map);
		
		AstNodeExpression *GetCondition();
		AstNodeStatement *GetStatement();
		
		virtual std::list<AstNodeStatementVariableDeclaration *> GetVariableDeclarations(AstNode *stop = 0);
		virtual void ReplaceVariableReferences(std::map<AstNodeStatementVariableDeclaration *, AstNodeStatementVariableDeclaration *> replacements);
		
	private:
		virtual std::list<AstNode **> GetChildrenPointers();
		
		AstNodeExpression *m_condition;
		AstNodeStatement *m_statement;
};


class AstNodeStatementFor : public AstNodeStatement
{
	public:
		AstNodeStatementFor(AstNodeType *type, std::string identifier, AstNodeExpression *expression, AstNodeStatement *statement);
		AstNodeStatementFor(AstNodeStatementVariableDeclaration *variable_declaration, AstNodeExpression *expression, AstNodeStatement *statement);
		virtual ~AstNodeStatementFor();
		virtual AstNodeStatementFor *Copy();
		virtual AstNodeStatement *Normalize(CompileContext &context);
		virtual void SetCodeGenerator(CodeGeneratorType type);
		
		virtual void Link(std::map<std::string, AstNode *> &node_map);
		
		virtual std::list<AstNodeStatementVariableDeclaration *> GetVariableDeclarations(AstNode *stop = 0);
		virtual void ReplaceVariableReferences(std::map<AstNodeStatementVariableDeclaration *, AstNodeStatementVariableDeclaration *> replacements);
		
		
		AstNodeStatementVariableDeclaration *GetVariableDeclaration();
		AstNodeExpression *GetExpression();
		AstNodeStatement *GetStatement();
		
		virtual uint64_t GetStaticComplexityEstimate(ComputeSpec &spec);
		virtual AstNodeExpression *GetDynamicComplexityEstimate(ComputeSpec &spec);
		
		
	private:
		virtual std::string GetDotName();
		virtual std::list<AstNode **> GetChildrenPointers();
		
		AstNodeStatementVariableDeclaration *m_variable_declaration;
		AstNodeExpression *m_expression;
		AstNodeStatement *m_statement;
};

class AstNodeStatementWhile : public AstNodeStatementCtrlFlow
{
	public:
		AstNodeStatementWhile(AstNodeExpression *condition, AstNodeStatement *statement);
		virtual ~AstNodeStatementWhile() = default;
		virtual AstNodeStatementWhile *Copy();
		virtual void SetCodeGenerator(CodeGeneratorType type);
		
		virtual uint64_t GetStaticComplexityEstimate(ComputeSpec &spec);
		virtual AstNodeExpression *GetDynamicComplexityEstimate(ComputeSpec &spec);
		
	private:
		virtual std::string GetDotName();
};

class AstNodeStatementIf : public AstNodeStatement
{
	public:
		AstNodeStatementIf(AstNodeExpression *condition, AstNodeStatement *if_statement, AstNodeStatement *else_statement);
		virtual ~AstNodeStatementIf();
		virtual AstNodeStatementIf *Copy();
		virtual AstNodeStatement *Normalize(CompileContext &context);
		virtual void SetCodeGenerator(CodeGeneratorType type);
		
		virtual void Link(std::map<std::string, AstNode *> &node_map);
		
		AstNodeExpression *GetCondition();
		AstNodeStatement *GetIfStatement();
		AstNodeStatement *GetElseStatement();
		
		virtual std::list<AstNodeStatementVariableDeclaration *> GetVariableDeclarations(AstNode *stop = 0);
		virtual void ReplaceVariableReferences(std::map<AstNodeStatementVariableDeclaration *, AstNodeStatementVariableDeclaration *> replacements);
		
		virtual uint64_t GetStaticComplexityEstimate(ComputeSpec &spec);
		virtual AstNodeExpression *GetDynamicComplexityEstimate(ComputeSpec &spec);
		
	private:
		virtual std::string GetDotName();
		virtual std::list<AstNode **> GetChildrenPointers();
		
		AstNodeExpression *m_condition;
		AstNodeStatement *m_if_statement;
		AstNodeStatement *m_else_statement;
};

#include "parallel.hpp"

#endif
