/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2012-2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef AST_DYNVECT_HPP
#define AST_DYNVECT_HPP

#include "../compilecontext.hpp"

class AstNodeTypeDynamicVector : public AstNodeTypeVector
{
	public:
		AstNodeTypeDynamicVector(AstNodeType *type);
		virtual AstNodeTypeDynamicVector *Copy();
		virtual void SetCodeGenerator(CodeGeneratorType type);
		virtual int Compare(AstNodeType *rhs, CompileContext &context);
		
		virtual std::string GetName();
		
	private:
		virtual std::string GetDotName();
};

class AstNodeExpressionDynamicVector : public AstNodeExpressionValue
{
	public:
		AstNodeExpressionDynamicVector(AstNodeType *type, AstNodeExpression *expression);
		virtual ~AstNodeExpressionDynamicVector();
		virtual AstNodeExpressionDynamicVector *Copy();
		virtual AstNodeExpression *Normalize(CompileContext &context);
		virtual void SetCodeGenerator(CodeGeneratorType type);
		
		AstNodeExpression *GetExpression();
		
		virtual void Link(std::map<std::string, AstNode *> &node_map);
		virtual void ReplaceVariableReferences(std::map<AstNodeStatementVariableDeclaration *, AstNodeStatementVariableDeclaration *> replacements);
		
	private:
		virtual std::string GetDotName();
		virtual std::list<AstNode **> GetChildrenPointers();
		
		AstNodeExpression *m_expression;
};

class AstNodeStatementDynamicVectorOperation : public AstNodeStatement
{
	public:
		AstNodeStatementDynamicVectorOperation(AstNodeExpression *dynvect, std::string dotname);
		virtual ~AstNodeStatementDynamicVectorOperation();
		virtual AstNodeStatement *Normalize(CompileContext &context);
		
		virtual void Link(std::map<std::string, AstNode *> &node_map);
		virtual void ReplaceVariableReferences(std::map<AstNodeStatementVariableDeclaration *, AstNodeStatementVariableDeclaration *> replacements);
		
		virtual uint64_t GetStaticComplexityEstimate(ComputeSpec &spec);
		virtual AstNodeExpression *GetDynamicComplexityEstimate(ComputeSpec &spec);
		
		AstNodeExpression *GetExpression();
		
	private:
		virtual std::string GetDotName();
		virtual std::list<AstNode **> GetChildrenPointers();
		
		AstNodeExpression *m_dynvect;
		std::string m_dotname;
};

class AstNodeDynamicVectorCommitBefore : public AstNodeStatementDynamicVectorOperation
{
	public:
		AstNodeDynamicVectorCommitBefore(AstNodeExpression *dynvect);
		virtual AstNodeDynamicVectorCommitBefore *Copy();
		virtual void SetCodeGenerator(CodeGeneratorType type);
};

class AstNodeDynamicVectorCommitAfter : public AstNodeStatementDynamicVectorOperation
{
	public:
		AstNodeDynamicVectorCommitAfter(AstNodeExpression *dynvect);
		virtual AstNodeDynamicVectorCommitAfter *Copy();
		virtual void SetCodeGenerator(CodeGeneratorType type);
};

//This has pointer type
class AstNodeDynamicVectorGetPointer : public AstNodeExpression
{
	public:
		AstNodeDynamicVectorGetPointer(AstNodeExpressionLValue *dynvect, bool read, bool write);
		virtual ~AstNodeDynamicVectorGetPointer();
		virtual AstNodeDynamicVectorGetPointer *Copy();
		virtual AstNodeExpression *Normalize(CompileContext &context);
		virtual void SetCodeGenerator(CodeGeneratorType type);
		
		virtual AstNodeType *GetType();
		
		virtual void Link(std::map<std::string, AstNode *> &node_map);
		virtual void ReplaceVariableReferences(std::map<AstNodeStatementVariableDeclaration *, AstNodeStatementVariableDeclaration *> replacements);
		
		virtual uint64_t GetStaticComplexityEstimate(ComputeSpec &spec);
		virtual AstNodeExpression *GetDynamicComplexityEstimate(ComputeSpec &spec);
		
		AstNodeExpressionLValue *GetLValue();
		bool GetRead();
		bool GetWrite();
		
	private:
		virtual std::string GetDotName();
		virtual std::list<AstNode **> GetChildrenPointers();
		
		AstNodeExpressionLValue *m_dynvect;
		AstNodeType *m_type;
		bool m_read;
		bool m_write;
};


#endif
