/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2012-2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef AST_FUNCTION_HPP
#define AST_FUNCTION_HPP

#include <list>

#include "../compilecontext.hpp"

class AstNodeFunctionDeclaration : public AstNode
{
	public:
		AstNodeFunctionDeclaration(AstNodeType *return_type, std::string name,
			std::list<AstNodeStatementVariableDeclaration *> args, AstNodeStatement *statement, bool is_export, bool is_ccall, bool is_lock);
		virtual AstNodeFunctionDeclaration *Copy();
		virtual AstNodeFunctionDeclaration *Normalize(CompileContext &context);
		virtual void SetCodeGenerator(CodeGeneratorType type);
		
		virtual ~AstNodeFunctionDeclaration();
		
		bool GetIsCompiler();
		void SetIsCompiler();
		bool GetCCall();
		bool GetExport();
		bool GetLock();
		std::string GetIdentifier();
		AstNodeTypeFunctionPointer *GetType();
		AstNodeType *GetReturnType();
		AstNodeStatement *GetStatement();
		std::list<AstNodeType *> GetArgumentTypes();
		std::list<AstNodeStatementVariableDeclaration *> GetArguments();
		void LinkToOthers(std::map<std::string, AstNode *> &node_map);
		void LinkToMap(std::map<std::string, AstNode *> &node_map);
		
		//Be careful with this
		unsigned int ReplaceArgument(AstNodeStatementVariableDeclaration *argument, std::list<AstNodeStatementVariableDeclaration *> replacements);
		
		//Gets a list of variable declarations up to and excluding stop. If stop = 0, returns the entire function's declarations
		std::list<AstNodeStatementVariableDeclaration *> GetVariableDeclarations(AstNode *stop = 0);
		void ReplaceVariableReferences(std::map<AstNodeStatementVariableDeclaration *, AstNodeStatementVariableDeclaration *> replacements);
		
		virtual uint64_t GetStaticComplexityEstimate(ComputeSpec &spec);
		virtual AstNodeExpression *GetDynamicComplexityEstimate(ComputeSpec &spec);
		
	private:
		virtual std::string GetDotName();
		virtual std::list<AstNode **> GetChildrenPointers();
		
		AstNodeTypeFunctionPointer *m_type;
		AstNodeType *m_return_type;
		std::string m_name;
		std::list<AstNodeStatementVariableDeclaration *> m_args;
		AstNodeStatement *m_statement;
		bool m_is_export;
		bool m_is_ccall;
		bool m_is_lock;
		bool m_is_compiler;
};

class AstNodeFunctionCall : public AstNodeExpression
{
	public:
		AstNodeFunctionCall(AstNodeExpressionLValue *lvalue, std::list<AstNodeExpression *> expressions);
		virtual AstNodeFunctionCall *Copy();
		virtual ~AstNodeFunctionCall();
		virtual AstNodeExpression *Normalize(CompileContext &context);
		virtual void SetCodeGenerator(CodeGeneratorType type);
		
		virtual AstNodeType *GetType();
		virtual void Link(std::map<std::string, AstNode *> &node_map);
		
		bool GetCCall();
		std::list<AstNodeType *> GetArgumentTypes();
		std::list<AstNodeExpression *> GetExpressions();
		AstNodeExpressionLValue *GetLValue();
		
		//Be careful with this
		void ReplaceArgument(AstNodeExpression *argument, std::list<AstNodeExpression *> replacements);
		
		virtual void ReplaceVariableReferences(std::map<AstNodeStatementVariableDeclaration *, AstNodeStatementVariableDeclaration *> replacements);
		
		virtual uint64_t GetStaticComplexityEstimate(ComputeSpec &spec);
		virtual AstNodeExpression *GetDynamicComplexityEstimate(ComputeSpec &spec);
		
	private:
		virtual std::string GetDotName();
		virtual std::list<AstNode **> GetChildrenPointers();
		
		AstNodeTypeFunctionPointer *GetFunctionPointerType();
		
		AstNodeExpressionLValue *m_lvalue;
		std::list<AstNodeExpression *> m_expressions;
};

#endif
