/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2012-2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef AST_PARALLEL_HPP
#define AST_PARALLEL_HPP

#include "../compilecontext.hpp"

class AstNodeStatementParallel : public AstNodeStatement
{
	public:
		AstNodeStatementParallel(AstNodeStatement *statement);
		virtual ~AstNodeStatementParallel();
		virtual AstNodeStatement *Normalize(CompileContext &context);
		virtual void SetCodeGenerator(CodeGeneratorType type);
		
		virtual void Link(std::map<std::string, AstNode *> &node_map);
		
		AstNodeStatement *GetStatement();
		
		virtual std::list<AstNodeStatementVariableDeclaration *> GetVariableDeclarations(AstNode *stop = 0);
		virtual void ReplaceVariableReferences(std::map<AstNodeStatementVariableDeclaration *, AstNodeStatementVariableDeclaration *> replacements);
		
		virtual uint64_t GetStaticComplexityEstimate(ComputeSpec &spec);
		virtual AstNodeExpression *GetDynamicComplexityEstimate(ComputeSpec &spec);
		
	protected:
		void SetStatement(AstNodeStatement *statement);
		
	private:
		virtual std::list<AstNode **> GetChildrenPointers();
		
		AstNodeStatement *m_statement;
};

class AstNodeStatementParallelSpawn : public AstNodeStatementParallel
{
	public:
		AstNodeStatementParallelSpawn(AstNodeStatement *statement);
		virtual AstNodeStatementParallelSpawn *Copy();
		virtual void SetCodeGenerator(CodeGeneratorType type);
		
		//Removes the statement from being a child, but does not delete the statement
		void RemoveStatement();
		
	private:
		virtual std::string GetDotName();
};

class AstNodeStatementParallelSync : public AstNodeStatementParallel
{
	public:
		AstNodeStatementParallelSync(AstNodeStatement *statement);
		virtual AstNodeStatementParallelSync *Copy();
		virtual void SetCodeGenerator(CodeGeneratorType type);
		
	private:
		virtual std::string GetDotName();
};

class AstNodeStatementExtracted : public AstNodeStatement
{
	public:
		AstNodeStatementExtracted(AstNodeStatement *statement, std::list<AstNodeStatementVariableDeclaration *> variables, unsigned int linkid);
		virtual ~AstNodeStatementExtracted();
		virtual AstNodeStatementExtracted *Copy();
		virtual void SetCodeGenerator(CodeGeneratorType type);
		
		virtual AstNodeStatement *Normalize(CompileContext &context);
		virtual void Link(std::map<std::string, AstNode *> &node_map);
		virtual std::list<AstNodeStatementVariableDeclaration *> GetVariableDeclarations(AstNode *stop = 0);
		virtual void ReplaceVariableReferences(std::map<AstNodeStatementVariableDeclaration *, AstNodeStatementVariableDeclaration *> replacements);
		virtual uint64_t GetStaticComplexityEstimate(ComputeSpec &spec);
		virtual AstNodeExpression *GetDynamicComplexityEstimate(ComputeSpec &spec);
		
		AstNodeStatement *GetStatement();
		std::list<AstNodeStatementVariableDeclaration *> GetVariables();
		unsigned int GetLinkId();
		
	private:
		virtual std::string GetDotName();
		virtual std::list<AstNode **> GetChildrenPointers();
		
		AstNodeStatement *m_statement;
		std::list<AstNodeStatementVariableDeclaration *> m_variables;
		unsigned int m_linkid;
};

class AstNodeTypeLock : public AstNodeType
{
	public:
		AstNodeTypeLock(AstNodeType *type);
		virtual ~AstNodeTypeLock();
		
		virtual AstNodeTypeLock *Copy();
		virtual void SetCodeGenerator(CodeGeneratorType type);
		virtual AstNodeType *GetCastType();
		
		virtual std::string GetName();
		virtual int Compare(AstNodeType *rhs, CompileContext &context); //Compares with child type
		
		AstNodeType *GetType();
		
	private:
		virtual std::string GetDotName();
		virtual std::list<AstNode **> GetChildrenPointers();
		
		AstNodeType *m_type;
};

class AstNodeStatementParallelAtomic : public AstNodeStatementParallel
{
	public:
		AstNodeStatementParallelAtomic(AstNodeStatement *statement);
		
		virtual AstNodeStatementParallelAtomic *Copy();
		virtual void SetCodeGenerator(CodeGeneratorType type);
		
	private:
		virtual std::string GetDotName();
};

class AstNodeExpressionLValueLock : public AstNodeExpressionLValue
{
	public:
		AstNodeExpressionLValueLock(AstNodeExpressionLValue *lvalue);
		virtual ~AstNodeExpressionLValueLock();
		
		virtual AstNodeExpressionLValueLock *Copy();
		virtual AstNodeExpressionLValue *Normalize(CompileContext &context); //This normalizes itself away if the lvalue type is not a lock
		virtual void SetCodeGenerator(CodeGeneratorType type);
		
		virtual void Link(std::map<std::string, AstNode *> &node_map);
		virtual void ReplaceVariableReferences(std::map<AstNodeStatementVariableDeclaration *, AstNodeStatementVariableDeclaration *> replacements);
		
		virtual uint64_t GetStaticComplexityEstimate(ComputeSpec &spec);
		virtual AstNodeExpression *GetDynamicComplexityEstimate(ComputeSpec &spec);
		
		virtual AstNodeType *GetType();
		
		AstNodeExpressionLValue *GetLValue();
		
	private:
		virtual std::string GetDotName();
		virtual std::list<AstNode **> GetChildrenPointers();
		
		AstNodeExpressionLValue *m_lvalue;
};


#endif
 
