/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef AST_STRUCT_HPP
#define AST_STRUCT_HPP

#include "../compilecontext.hpp"

class AstNodeStructItem : public AstNode
{
	public:
		AstNodeStructItem(AstNodeType *type, std::string identifier);
		virtual ~AstNodeStructItem();
		virtual AstNodeStructItem *Copy();
		virtual void SetCodeGenerator(CodeGeneratorType type);
		
		AstNodeType *GetType();
		std::string GetIdentifier();
		
	private:
		virtual std::string GetDotName();
		virtual std::list<AstNode **> GetChildrenPointers();
		
		AstNodeType *m_type;
		std::string m_identifier;
};

class AstNodeStructDeclaration : public AstNode
{
	public:
		AstNodeStructDeclaration(std::string identifier, std::list<AstNodeStructItem *> items);
		virtual ~AstNodeStructDeclaration();
		virtual AstNodeStructDeclaration *Copy();
		virtual void SetCodeGenerator(CodeGeneratorType type);
		
		virtual void Link(std::map<std::string, AstNode *> &node_map);
		void LinkToMap(std::map<std::string, AstNode *> &node_map);
		void LinkToOthers(std::map<std::string, AstNode *> &node_map);
		
		std::string GetIdentifier();
		std::list<AstNodeStructItem *> GetItems();
		
	private:
		virtual std::string GetDotName();
		virtual std::list<AstNode **> GetChildrenPointers();
		
		std::string m_identifier;
		std::list<AstNodeStructItem *> m_items;
};

class AstNodeTypeStruct : public AstNodeType
{
	public:
		AstNodeTypeStruct(std::string identifier);
		virtual AstNodeTypeStruct *Copy();
		virtual void SetCodeGenerator(CodeGeneratorType type);
		virtual int Compare(AstNodeType *rhs, CompileContext &context);
		
		virtual void Link(std::map<std::string, AstNode *> &node_map);
		virtual std::string GetName();
		
		std::string GetIdentifier();
		AstNodeStructDeclaration *GetLink();
		
	private:
		virtual std::string GetDotName();
		virtual std::list<AstNode **> GetLinksPointers();
		
		std::string m_identifier;
		AstNodeStructDeclaration *m_link;
};

class AstNodeExpressionLValueStructElement : public AstNodeExpressionLValue
{
	public:
		AstNodeExpressionLValueStructElement(AstNodeExpressionLValue *lvalue, std::string identifier);
		virtual ~AstNodeExpressionLValueStructElement();
		virtual AstNodeExpressionLValueStructElement *Copy();
		virtual AstNodeExpressionLValue *Normalize(CompileContext &context);
		virtual void SetCodeGenerator(CodeGeneratorType type);
		
		virtual AstNodeType *GetType();
		virtual void Link(std::map<std::string, AstNode *> &node_map);
		AstNodeTypeStruct *GetStructType();
		
		AstNodeExpressionLValue *GetLValue();
		std::string GetIdentifier();
		
		virtual void ReplaceVariableReferences(std::map<AstNodeStatementVariableDeclaration *, AstNodeStatementVariableDeclaration *> replacements);
		
		virtual uint64_t GetStaticComplexityEstimate(ComputeSpec &spec);
		virtual AstNodeExpression *GetDynamicComplexityEstimate(ComputeSpec &spec);
		
	private:
		virtual std::string GetDotName();
		virtual std::list<AstNode **> GetChildrenPointers();
		
		AstNodeExpressionLValue *m_lvalue;
		std::string m_identifier;
};

#endif
