/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef CODEGEN_CODEGEN_HPP
#define CODEGEN_CODEGEN_HPP

#include <string>

enum CodeGeneratorType
{
	CODEGEN_NONE,
	CODEGEN_LLVM,
	CODEGEN_OPENCL,
	CODEGEN_PSEUDOPC
};

#include "../error.hpp"
#include "../util.hpp"
#include "../compilecontext.hpp"


/*
 * This class is the base class for all code generators
 * It should be derrived for each AST node type
 */
class AstNode;
class CodeGenerator : public TrackedClass
{
	public:
		CodeGenerator(AstNode *node);
		
		/*
		 * This copies the code generator, but NOT the AST Node
		 */
		virtual CodeGenerator *Copy() = 0;
		
		virtual AstNode *GetNode() = 0;
		
		template <typename T> T *GetNodeBase()
		{
			T *node = dynamic_cast<T *>(m_node);
			if (!node){
				throw FormatError("Node is not of the currect type!");
			}
			
			return node;
		}
		
	private:
		//We're doing a lot of pointery stuff (for polymorphism)... don't allow
		//copying directly
		CodeGenerator(const CodeGenerator &) = delete;
		CodeGenerator &operator=(const CodeGenerator &) = delete;
		
		AstNode *m_node;
};
#define AstType(T) virtual T *GetNode(){return GetNodeBase<T>();}

#include "../ast/astnode.hpp"

#endif
