/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2012-2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../../ast/astnode.hpp"
#include "codegen.hpp"

CodeGeneratorStatementForLlvm::CodeGeneratorStatementForLlvm(AstNodeStatementFor *node) :
	CodeGeneratorStatementLlvm(node), m_id(GetUniqueId())
{
}

CodeGeneratorStatementForLlvm *CodeGeneratorStatementForLlvm::Copy()
{
	return new CodeGeneratorStatementForLlvm(GetNode());
}

std::string CodeGeneratorStatementForLlvm::Generate(CompileContext &context, unsigned int d)
{
	std::string identifier = GetNode()->GetVariableDeclaration()->GetIdentifier();
	std::string type = GetNode()->GetVariableDeclaration()->GetType()->GetCodeGenerator<CodeGeneratorTypeLlvm>()->Generate(context, d);
	std::string source;
	std::string debug = GetDebug(context, d);
	
	unsigned int index = GetUniqueId();
	unsigned int index_inc = GetUniqueId();
	unsigned int condition = GetUniqueId();
	unsigned int maximum = GetUniqueId();
	
	source += GetNode()->GetExpression()->GetCodeGenerator<CodeGeneratorExpressionLlvm>()->Generate(context, d, maximum); //calculate maximum
	source += PrintString(
		"; For loop %s\n"
			"\tstore %s 0, %s* %%.%u%s ; store 0 in the index\n"
			"\tbr label %%.for%ustart%s ; cope with the fact that we need a branch before the label\n"
			"\t.for%ustart: ; start of loop\n"
			"\t%%.%u = load %s* %%.%u%s ; load the index\n"
			"\t%%.%u = icmp ult %s %%.%u, %%.%u ; perform the comparison\n"
			"\tbr i1 %%.%u, label %%.for%ubody, label %%.for%uend%s ; finish the loop if needed\n"
			"\t.for%ubody: ; body of loop\n"
				"%s"
			"\t%%.%u = add %s %%.%u, 1 ; increment the index\n"
			"\tstore %s %%.%u, %s* %%.%u%s ; store the incremented index\n"
			"\tbr label %%.for%ustart%s ; loop to the beginning\n"
			"\t.for%uend: ; end of loop\n",
		GetNode()->GetSourceLocation().c_str(),
		type.c_str(), type.c_str(), GetNode()->GetVariableDeclaration()->GetId(), debug.c_str(),
		m_id, debug.c_str(),
		m_id,
		index, type.c_str(), GetNode()->GetVariableDeclaration()->GetId(), debug.c_str(),
		condition, type.c_str(), index, maximum,
		condition, m_id, m_id, debug.c_str(),
		m_id,
		Indent(GetNode()->GetStatement()->GetCodeGenerator<CodeGeneratorStatementLlvm>()->Generate(context, d), 2).c_str(),
		index_inc, type.c_str(), index, 
		type.c_str(), index_inc, type.c_str(), GetNode()->GetVariableDeclaration()->GetId(), debug.c_str(),
		m_id, debug.c_str(),
		m_id);
	
	return PrintString("; For statement at line %u\n", GetNode()->GetLine()) + Indent(source, 1);
}

std::string CodeGeneratorStatementForLlvm::GenerateFnBegin(CompileContext &context, unsigned int d)
{
	std::string source = GetNode()->GetStatement()->GetCodeGenerator<CodeGeneratorStatementLlvm>()->GenerateFnBegin(context, d);
	source += GetNode()->GetVariableDeclaration()->GetCodeGenerator<CodeGeneratorStatementLlvm>()->GenerateFnBegin(context, d);
	
	return PrintString("; For statement at line %u\n", GetNode()->GetLine()) + Indent(source, 1);
}

std::string CodeGeneratorStatementForLlvm::GenerateFnEnd(CompileContext &context, unsigned int d)
{
	std::string source = GetNode()->GetStatement()->GetCodeGenerator<CodeGeneratorStatementLlvm>()->GenerateFnEnd(context, d);
	source += GetNode()->GetVariableDeclaration()->GetCodeGenerator<CodeGeneratorStatementLlvm>()->GenerateFnEnd(context, d);
	
	return PrintString("; For statement at line %u\n", GetNode()->GetLine()) + Indent(source, 1);
}

std::string CodeGeneratorStatementForLlvm::GenerateTopLevel(CompileContext &context, unsigned int d)
{
	std::string source = GetNode()->GetStatement()->GetCodeGenerator<CodeGeneratorStatementLlvm>()->GenerateTopLevel(context, d);
	source += GetNode()->GetVariableDeclaration()->GetCodeGenerator<CodeGeneratorStatementLlvm>()->GenerateTopLevel(context, d);
	
	return PrintString("; For statement at line %u\n", GetNode()->GetLine()) + Indent(source, 1);
}

CodeGeneratorStatementWhileLlvm::CodeGeneratorStatementWhileLlvm(AstNodeStatementWhile *node) :
	CodeGeneratorStatementLlvm(node)
{
}

CodeGeneratorStatementWhileLlvm *CodeGeneratorStatementWhileLlvm::Copy()
{
	return new CodeGeneratorStatementWhileLlvm(GetNode());
}

std::string CodeGeneratorStatementWhileLlvm::Generate(CompileContext &context, unsigned int d)
{
	std::string source;
	unsigned int base = GetUniqueId();
	unsigned int cond = GetUniqueId();
	
	source += PrintString("br label %%.while%ustart\n.while%ustart: ; start of while loop\n", base, base);
	source += GetNode()->GetCondition()->GetCodeGenerator<CodeGeneratorExpressionLlvm>()->Generate(context, d, cond);
	source += PrintString("br i1 %%.%u, label %%.while%ubody, label %%.while%uend ; jump according to the condition\n.while%ubody: ; main body of the loop\n",
		cond, base, base, base);
	source += GetNode()->GetStatement()->GetCodeGenerator<CodeGeneratorStatementLlvm>()->Generate(context, d);
	source += PrintString("br label %%.while%ustart\n.while%uend: ; End of loop\n", base, base);
	
	return PrintString("; While statement at line %u\n", GetNode()->GetLine()) + Indent(source, 1);
}

std::string CodeGeneratorStatementWhileLlvm::GenerateFnBegin(CompileContext &context, unsigned int d)
{
	std::string source = GetNode()->GetStatement()->GetCodeGenerator<CodeGeneratorStatementLlvm>()->GenerateFnBegin(context, d);
	return PrintString("; While statement at line %u\n", GetNode()->GetLine()) + Indent(source, 1);
}

std::string CodeGeneratorStatementWhileLlvm::GenerateFnEnd(CompileContext &context, unsigned int d)
{
	std::string source = GetNode()->GetStatement()->GetCodeGenerator<CodeGeneratorStatementLlvm>()->GenerateFnEnd(context, d);
	return PrintString("; While statement at line %u\n", GetNode()->GetLine()) + Indent(source, 1);
}

std::string CodeGeneratorStatementWhileLlvm::GenerateTopLevel(CompileContext &context, unsigned int d)
{
	std::string source = GetNode()->GetStatement()->GetCodeGenerator<CodeGeneratorStatementLlvm>()->GenerateTopLevel(context, d);
	return PrintString("; While statement at line %u\n", GetNode()->GetLine()) + Indent(source, 1);
}

CodeGeneratorStatementIfLlvm::CodeGeneratorStatementIfLlvm(AstNodeStatementIf *node) :
	CodeGeneratorStatementLlvm(node)
{
}

CodeGeneratorStatementIfLlvm *CodeGeneratorStatementIfLlvm::Copy()
{
	return new CodeGeneratorStatementIfLlvm(GetNode());
}

std::string CodeGeneratorStatementIfLlvm::Generate(CompileContext &context, unsigned int d)
{
	std::string source;
	AstNodeStatement *else_statement = GetNode()->GetElseStatement();
	unsigned int base = GetUniqueId();
	unsigned int cond = GetUniqueId();
	
	if (else_statement){
		source += GetNode()->GetCondition()->GetCodeGenerator<CodeGeneratorExpressionLlvm>()->Generate(context, d, cond);
		source += PrintString("br i1 %%.%u, label %%.if%utrue, label %%.if%ufalse ; If statement\n.if%utrue:\n",
			cond, base, base, base);
		source += GetNode()->GetIfStatement()->GetCodeGenerator<CodeGeneratorStatementLlvm>()->Generate(context, d);
		source += PrintString("br label %%.if%uend ; adhere to block requirement\n.if%ufalse: ; end of loop\n",
			base, base);
		source += GetNode()->GetElseStatement()->GetCodeGenerator<CodeGeneratorStatementLlvm>()->Generate(context, d);
		source += PrintString("br label %%.if%uend ; adhere to block requirement\n.if%uend: ; end of loop\n",
			base, base);
	}
	else {
		source += GetNode()->GetCondition()->GetCodeGenerator<CodeGeneratorExpressionLlvm>()->Generate(context, d, cond);
		source += PrintString("br i1 %%.%u, label %%.if%utrue, label %%.if%ufalse ; If statement\n.if%utrue:\n",
			cond, base, base, base);
		source += GetNode()->GetIfStatement()->GetCodeGenerator<CodeGeneratorStatementLlvm>()->Generate(context, d);
		source += PrintString("br label %%.if%ufalse ; adhere to block requirement\n.if%ufalse: ; end of loop\n",
			base, base);
	}
	
	return PrintString("; If statement at line %u\n", GetNode()->GetLine()) + Indent(source, 1);
}

std::string CodeGeneratorStatementIfLlvm::GenerateFnBegin(CompileContext &context, unsigned int d)
{
	AstNodeStatement *else_statement = GetNode()->GetElseStatement();
	std::string source = (else_statement ? else_statement->GetCodeGenerator<CodeGeneratorStatementLlvm>()->GenerateFnBegin(context, d) : "") +
		GetNode()->GetIfStatement()->GetCodeGenerator<CodeGeneratorStatementLlvm>()->GenerateFnBegin(context, d);
	return PrintString("; If statement at line %u\n", GetNode()->GetLine()) + Indent(source, 1);
}

std::string CodeGeneratorStatementIfLlvm::GenerateFnEnd(CompileContext &context, unsigned int d)
{
	AstNodeStatement *else_statement = GetNode()->GetElseStatement();
	std::string source = (else_statement ? else_statement->GetCodeGenerator<CodeGeneratorStatementLlvm>()->GenerateFnEnd(context, d) : "") +
		GetNode()->GetIfStatement()->GetCodeGenerator<CodeGeneratorStatementLlvm>()->GenerateFnEnd(context, d);
	return PrintString("; If statement at line %u\n", GetNode()->GetLine()) + Indent(source, 1);
}

std::string CodeGeneratorStatementIfLlvm::GenerateTopLevel(CompileContext &context, unsigned int d)
{
	AstNodeStatement *else_statement = GetNode()->GetElseStatement();
	std::string source = (else_statement ? else_statement->GetCodeGenerator<CodeGeneratorStatementLlvm>()->GenerateTopLevel(context, d) : "") +
		GetNode()->GetIfStatement()->GetCodeGenerator<CodeGeneratorStatementLlvm>()->GenerateTopLevel(context, d);
	return PrintString("; If statement at line %u\n", GetNode()->GetLine()) + Indent(source, 1);
}
