/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2012-2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../../ast/astnode.hpp"
#include "codegen.hpp"



CodeGeneratorTypeDynamicVectorLlvm::CodeGeneratorTypeDynamicVectorLlvm(AstNodeTypeDynamicVector *node) :
	CodeGeneratorTypeLlvm(node)
{
}

CodeGeneratorTypeDynamicVectorLlvm *CodeGeneratorTypeDynamicVectorLlvm::Copy()
{
	return new CodeGeneratorTypeDynamicVectorLlvm(GetNode());
}

std::string CodeGeneratorTypeDynamicVectorLlvm::Generate(CompileContext &context, unsigned int d)
{
	return "%t.dynvect";
}

std::string CodeGeneratorTypeDynamicVectorLlvm::GenerateOperation
	(CompileContext &context, unsigned int d, unsigned int value, std::string loop_code, std::string dynvect_function, unsigned int element, unsigned int dst, unsigned int element_dst)
{
	std::string source;
	std::string debug = GetDebug(context, d);
	
	if (loop_code.size()){
		AstNodeType *type = GetNode()->GetType();
		std::string element_type = type->GetCodeGenerator<CodeGeneratorTypeLlvm>()->Generate(context, d);
		
		unsigned int loop_id = GetUniqueId();
		unsigned int vector_length = GetUniqueId();
		unsigned int counter_memory = GetUniqueId();
		unsigned int counter = GetUniqueId();
		unsigned int counter_new = GetUniqueId();
		unsigned int should_end = GetUniqueId();
		unsigned int vector_i8 = GetUniqueId();
		unsigned int vector_type = GetUniqueId();
		unsigned int element_pointer = GetUniqueId();
		
		source += PrintString(
			"%%.%u = call fastcc %%t.size (%%t.dynvect)* @.dynvect.element_count(%%t.dynvect %%.%u)%s ; Get the length of the vector\n"
			"%%.%u = call fastcc %%t.size* ()* @.get_tsize()%s ; Get a counter (a bit like alloca)\n"
			"store %%t.size 0, %%t.size* %%.%u%s ; store 0 in the counter\n"
			"%%.%u = call fastcc i8*(%%t.dynvect, %%t.size, %%t.size)* @.dynvect.readwrite (%%t.dynvect %%.%u, %%t.size 0, %%t.size %%.%u)%s ; Get the buffer from which we should read\n"
			"%%.%u = bitcast i8* %%.%u to %s*%s ; cast to the right type\n"
			"call fastcc void (%%t.dynvect)* @.dynvect.commit.before(%%t.dynvect %%.%u)%s ; Commit the read\n"
			
			"br label %%.dynamic_vector_copy_begin.%u\n"
			".dynamic_vector_copy_begin.%u:\n"
				"\t%%.%u = load %%t.size* %%.%u ; Get the index into the array\n"
				"\t%%.%u = icmp uge %%t.size %%.%u, %%.%u ; Compare it with the number of elements\n"
				"\tbr i1 %%.%u, label %%.dynamic_vector_copy_end.%u, label %%.dynamic_vector_copy_body.%u\n"
			".dynamic_vector_copy_body.%u:\n"
				
				"\t%%.%u = getelementptr %s* %%.%u, %%t.size %%.%u ; Get a pointer to the element\n"
				"\t%%.%u = load %s* %%.%u ; Load the element\n"
				
				"\t; Perform the operation on the elements of the dynamic vector\n"
				"%s"
				"%s"
				
				"\t%%.%u = add %%t.size %%.%u, 1 ; add one to the index\n"
				"\tstore %%t.size %%.%u, %%t.size* %%.%u ; store the new index\n"
				"\tbr label %%.dynamic_vector_copy_begin.%u ; Loop\n"
			".dynamic_vector_copy_end.%u:\n"
				"\tcall fastcc void (%%t.dynvect)* @.dynvect.commit.before(%%t.dynvect %%.%u)%s ; Commit the read\n"
				"\tcall fastcc void (%%t.size*)* @.release_tsize(%%t.size* %%.%u) ; free the counter\n"
				"\t%scall fastcc %s (%%t.dynvect)* @.dynvect.%s(%%t.dynvect %%.%u)%s ; Perform the operation on the vector\n",
			
			vector_length, value, debug.c_str(),
			counter_memory, debug.c_str(),
			counter_memory, debug.c_str(),
			vector_i8, value, vector_length, debug.c_str(),
			vector_type, vector_i8, element_type.c_str(), debug.c_str(),
			value, debug.c_str(),
			
			loop_id,
			loop_id,
			counter, counter_memory,
			should_end, counter, vector_length,
			should_end, loop_id, loop_id,
			loop_id,
			
			element_pointer, element_type.c_str(), vector_type, counter,
			element, element_type.c_str(), element_pointer,
			
			Indent(loop_code, 2).c_str(),
			element_dst ? Indent(PrintString("store %s %%.%u, %s* %%.%u%s", element_type.c_str(), element_dst, element_type.c_str(), element_pointer, debug.c_str()), 1).c_str() : "",
			
			counter_new, counter,
			counter_new, counter_memory,
			loop_id,
			loop_id,
			value, debug.c_str(),
			counter_memory,
			dst ? PrintString("%%.%u = ", dst).c_str() : "", dst ? "%t.dynvect" : "void", dynvect_function.c_str(), value, debug.c_str()
		);
	}
	else {
		source += PrintString("%scall fastcc %s (%%t.dynvect)* @.dynvect.%s(%%t.dynvect %%.%u)%s ; Perform the operation on the vector\n",
			dst ? PrintString("%%.%u = ", dst).c_str() : "", dst ? "%t.dynvect" : "void", dynvect_function.c_str(), value, debug.c_str());
	}
	
	return source;
}

std::string CodeGeneratorTypeDynamicVectorLlvm::GenerateCreate(CompileContext &context, unsigned int d, unsigned int dst)
{
	//TODO - Make this like the other three
	return PrintString("%%.%u = call fastcc %%t.dynvect (%%t.size, %%t.size)* @.dynvect.malloc (%%t.size 0, %%t.size 0)\n", dst);
}

std::string CodeGeneratorTypeDynamicVectorLlvm::GenerateDestroy(CompileContext &context, unsigned int d, unsigned int src)
{
	unsigned int element = GetUniqueId();
	return GenerateOperation(context, d, src, GetNode()->GetType()->GetCodeGenerator<CodeGeneratorTypeLlvm>()->GenerateDestroy(context, d, element), "free", element);
}

std::string CodeGeneratorTypeDynamicVectorLlvm::GenerateCopy(CompileContext &context, unsigned int d, unsigned int src, unsigned int dst)
{
	unsigned int element_src = GetUniqueId();
	unsigned int element_dst = GetUniqueId();
	std::string loopcode = GetNode()->GetType()->GetCodeGenerator<CodeGeneratorTypeLlvm>()->GenerateCopy(context, d, element_src, element_dst);
	return GenerateOperation(context, d, src, loopcode,
		loopcode.size() ? "copy_nomemcpy" : "copy", element_src, dst, element_dst);
}

std::string CodeGeneratorTypeDynamicVectorLlvm::GenerateDebugGeneric(CompileContext &context, unsigned int toplevel)
{
	return PrintString("metadata !%u", toplevel);
}

std::string CodeGeneratorTypeDynamicVectorLlvm::GenerateDebugTopLevelGeneric(CompileContext &context, unsigned int n, std::string type)
{
	return PrintString(
		"!%u = metadata !{\n"
			"\ti32 %u, ; Tag (DW_TAG_structure_type)\n"
			"\tnull, ; Reference to context\n"
			"\tmetadata !\"Dynamic Vector\", ; Name (may be "" for anonymous types)\n"
			"\tmetadata !1, ; Reference to file where defined (may be NULL)\n"
			"\ti32 0, ; Line number where defined (may be 0)\n"
			"\ti64 %u, ; Size in bits\n"
			"\ti64 8, ; Alignment in bits\n"
			"\ti32 0, ; Offset in bits\n"
			"\ti32 0, ; Flags\n"
			"\tnull, ; Reference to type derived from\n"
			
			"\tmetadata !{ ; Reference to array of member descriptors\n"
				"\t\tmetadata !{\n"
					"\t\t\ti32 %u, ; Tag (DW_TAG_member)\n"
					"\t\t\tmetadata !%u, ; Reference to context\n"
					"\t\t\tmetadata !\"type\", ; Name (may be "" for anonymous types)\n"
					"\t\t\tmetadata !1, ; Reference to file where defined (may be NULL)\n"
					"\t\t\ti32 0, ; Line number where defined (may be 0)\n"
					"\t\t\ti64 %u, ; Size in bits\n"
					"\t\t\ti64 8, ; Alignment in bits\n"
					"\t\t\ti64 %u, ; Offset in bits\n"
					"\t\t\ti32 0, ; Flags to encode attributes, e.g. private\n"
					"\t\t\tmetadata !{\n"
						"\t\t\t\ti32 %u, ; Tag (DW_TAG_base_type)\n"
						"\t\t\t\tnull, ; Reference to context\n"
						"\t\t\t\tmetadata !\"size_t\", ; Name (may be "" for anonymous types)\n"
						"\t\t\t\tnull, ; Reference to file where defined (may be NULL)\n"
						"\t\t\t\ti32 0, ; Line number where defined (may be 0)\n"
						"\t\t\t\ti64 %u, ; Size in bits\n"
						"\t\t\t\ti64 8, ; Alignment in bits\n"
						"\t\t\t\ti64 0, ; Offset in bits\n"
						"\t\t\t\ti32 0, ; Flags\n"
						"\t\t\t\ti32 7 ; DWARF type encoding\n"
					"\t\t\t}\n"
				"\t\t},\n"
				
				"\t\tmetadata !{\n"
					"\t\t\ti32 %u, ; Tag (DW_TAG_member)\n"
					"\t\t\tmetadata !%u, ; Reference to context\n"
					"\t\t\tmetadata !\"data\", ; Name (may be "" for anonymous types)\n"
					"\t\t\tmetadata !1, ; Reference to file where defined (may be NULL)\n"
					"\t\t\ti32 0, ; Line number where defined (may be 0)\n"
					"\t\t\ti64 %u, ; Size in bits\n"
					"\t\t\ti64 8, ; Alignment in bits\n"
					"\t\t\ti64 %u, ; Offset in bits\n"
					"\t\t\ti32 0, ; Flags to encode attributes, e.g. private\n"
					"\t\t\tmetadata !{ ; Reference to type derived from\n"
						"\t\t\t\ti32 %u, ; Tag (DW_TAG_pointer_type)\n"
						"\t\t\t\tnull, ; Reference to context\n"
						"\t\t\t\tmetadata !\"\", ; Name (may be "" for anonymous types)\n"
						"\t\t\t\tnull, ; Reference to file where defined (may be NULL)\n"
						"\t\t\t\ti32 0, ; Line number where defined (may be 0)\n"
						"\t\t\t\ti64 %u, ; Size in bits\n"
						"\t\t\t\ti64 8, ; Alignment in bits\n"
						"\t\t\t\ti64 0, ; Offset in bits\n"
						"\t\t\t\ti32 0, ; Flags to encode attributes, e.g. private\n"
						"%s\n"
					"\t\t\t}\n"
				"\t\t},\n"
				
				"\t\tmetadata !{\n"
					"\t\t\ti32 %u, ; Tag (DW_TAG_member)\n"
					"\t\t\tmetadata !%u, ; Reference to context\n"
					"\t\t\tmetadata !\"element_count\", ; Name (may be "" for anonymous types)\n"
					"\t\t\tmetadata !1, ; Reference to file where defined (may be NULL)\n"
					"\t\t\ti32 0, ; Line number where defined (may be 0)\n"
					"\t\t\ti64 %u, ; Size in bits\n"
					"\t\t\ti64 8, ; Alignment in bits\n"
					"\t\t\ti64 %u, ; Offset in bits\n"
					"\t\t\ti32 0, ; Flags to encode attributes, e.g. private\n"
					"\t\t\tmetadata !{\n"
						"\t\t\t\ti32 %u, ; Tag (DW_TAG_base_type)\n"
						"\t\t\t\tnull, ; Reference to context\n"
						"\t\t\t\tmetadata !\"size_t\", ; Name (may be "" for anonymous types)\n"
						"\t\t\t\tnull, ; Reference to file where defined (may be NULL)\n"
						"\t\t\t\ti32 0, ; Line number where defined (may be 0)\n"
						"\t\t\t\ti64 %u, ; Size in bits\n"
						"\t\t\t\ti64 8, ; Alignment in bits\n"
						"\t\t\t\ti64 0, ; Offset in bits\n"
						"\t\t\t\ti32 0, ; Flags\n"
						"\t\t\t\ti32 7 ; DWARF type encoding\n"
					"\t\t\t}\n"
				"\t\t},\n"
				
				"\t\tmetadata !{\n"
					"\t\t\ti32 %u, ; Tag (DW_TAG_member)\n"
					"\t\t\tmetadata !%u, ; Reference to context\n"
					"\t\t\tmetadata !\"element_size\", ; Name (may be "" for anonymous types)\n"
					"\t\t\tmetadata !1, ; Reference to file where defined (may be NULL)\n"
					"\t\t\ti32 0, ; Line number where defined (may be 0)\n"
					"\t\t\ti64 %u, ; Size in bits\n"
					"\t\t\ti64 8, ; Alignment in bits\n"
					"\t\t\ti64 %u, ; Offset in bits\n"
					"\t\t\ti32 0, ; Flags to encode attributes, e.g. private\n"
					"\t\t\tmetadata !{\n"
						"\t\t\t\ti32 %u, ; Tag (DW_TAG_base_type)\n"
						"\t\t\t\tnull, ; Reference to context\n"
						"\t\t\t\tmetadata !\"size_t\", ; Name (may be "" for anonymous types)\n"
						"\t\t\t\tnull, ; Reference to file where defined (may be NULL)\n"
						"\t\t\t\ti32 0, ; Line number where defined (may be 0)\n"
						"\t\t\t\ti64 %u, ; Size in bits\n"
						"\t\t\t\ti64 8, ; Alignment in bits\n"
						"\t\t\t\ti64 0, ; Offset in bits\n"
						"\t\t\t\ti32 0, ; Flags\n"
						"\t\t\t\ti32 7 ; DWARF type encoding\n"
					"\t\t\t}\n"
				"\t\t}\n"
			"\t},\n"
			
			"\ti32 0,\n"
			"\ti32 0,\n"
			"\ti32 0\n"
		"}\n",
		n,
		LlvmDebugVersion + 19,
		context.GetComputeSpec().GetCpu().GetSizeT() * 5,
		
		LlvmDebugVersion + 13,
		n,
		context.GetComputeSpec().GetCpu().GetSizeT(),
		context.GetComputeSpec().GetCpu().GetSizeT() * 0,
		LlvmDebugVersion + 36,
		context.GetComputeSpec().GetCpu().GetSizeT(),
		
		LlvmDebugVersion + 13,
		n,
		context.GetComputeSpec().GetCpu().GetSizeT(),
		context.GetComputeSpec().GetCpu().GetSizeT() * 1,
		LlvmDebugVersion + 15,
		context.GetComputeSpec().GetCpu().GetSizeT(),
		Indent(type, 4).c_str(),
		
		LlvmDebugVersion + 13,
		n,
		context.GetComputeSpec().GetCpu().GetSizeT(),
		context.GetComputeSpec().GetCpu().GetSizeT() * 2,
		LlvmDebugVersion + 36,
		context.GetComputeSpec().GetCpu().GetSizeT(),
		
		LlvmDebugVersion + 13,
		n,
		context.GetComputeSpec().GetCpu().GetSizeT(),
		context.GetComputeSpec().GetCpu().GetSizeT() * 3,
		LlvmDebugVersion + 36,
		context.GetComputeSpec().GetCpu().GetSizeT()
	);
}

std::string CodeGeneratorTypeDynamicVectorLlvm::GenerateDebug(CompileContext &context, unsigned int d)
{
	return GenerateDebugGeneric(context, GetNode()->GetId());
}

std::string CodeGeneratorTypeDynamicVectorLlvm::GenerateTopLevel(CompileContext &context, unsigned int d)
{
	std::string source = GetNode()->GetType()->GetCodeGenerator<CodeGeneratorTypeLlvm>()->GenerateTopLevel(context, d);
	
	if (context.GetDebug()){
		source += GenerateDebugTopLevelGeneric(context, GetNode()->GetId(), GetNode()->GetType()->GetCodeGenerator<CodeGeneratorTypeLlvm>()->GenerateDebug(context, d));
	}
	
	return source;
}


unsigned int CodeGeneratorTypeDynamicVectorLlvm::GetSize(CompileContext &context)
{
	return context.GetComputeSpec().GetCpu().GetSizeT()/2;
}

static unsigned int GreatestCommonDivisor(unsigned int element_size, unsigned int malloc_alignment)
{
	/* This is a *dreadful* algorithm, but it works:p */
	unsigned int smallest = (element_size < malloc_alignment) ? element_size : malloc_alignment;
	
	unsigned int gcd = 1;
	for (unsigned int i = 1; i < smallest; i++){
		if (!(element_size % i) && !(malloc_alignment % i)){
			gcd = i;
		}
	}
	
	return gcd;
}

unsigned int CodeGeneratorTypeDynamicVectorLlvm::GetAlignment(CompileContext &context)
{
	unsigned int element_size = GetNode()->GetType()->GetCodeGenerator<CodeGeneratorTypeLlvm>()->GetSize(context);
	unsigned int malloc_alignment = 64; //NOTE: This should be made target dependent (but it isn't in hearder.ll)
	
	return GreatestCommonDivisor(element_size, malloc_alignment);
}



CodeGeneratorExpressionDynamicVectorLlvm::CodeGeneratorExpressionDynamicVectorLlvm(AstNodeExpressionDynamicVector *node) :
	CodeGeneratorExpressionLlvm(node)
{
}


CodeGeneratorExpressionDynamicVectorLlvm *CodeGeneratorExpressionDynamicVectorLlvm::Copy()
{
	return new CodeGeneratorExpressionDynamicVectorLlvm(GetNode());
}

std::string CodeGeneratorExpressionDynamicVectorLlvm::Generate(CompileContext &context, unsigned int d, unsigned int n)
{
	std::string source;
	unsigned int created_element = GetUniqueId();
	std::string debug = GetDebug(context, d);
	AstNodeTypeDynamicVector *type = tracked_cast<AstNodeTypeDynamicVector>(GetNode()->GetType());
	std::string element_type = type->GetType()->GetCodeGenerator<CodeGeneratorTypeLlvm>()->Generate(context, d);
	std::string loop_code = type->GetType()->GetCodeGenerator<CodeGeneratorTypeLlvm>()->GenerateCreate(context, d, created_element);
	
	
	//Calculate the expression that represents the length of the vector
	unsigned int size_expression = GetUniqueId(); //The expression is of type %t.size
	source += GetNode()->GetExpression()->GetCodeGenerator<CodeGeneratorExpressionLlvm>()->Generate(context, d, size_expression);
	
	if (!loop_code.size()){
		source += PrintString("%%.%u = call fastcc %%t.dynvect (%%t.size, %%t.size)* @.dynvect.malloc (%%t.size %%.%u, %%t.size %u)%s ; Allocate a new vector\n",
			n, size_expression, type->GetType()->GetCodeGenerator<CodeGeneratorTypeLlvm>()->GetSize(context), debug.c_str());
	}
	else {
		unsigned int created_vector = GetUniqueId();
		unsigned int buffer_i8 = GetUniqueId();
		unsigned int buffer_type = GetUniqueId();
		
		unsigned int index = GetUniqueId();
		unsigned int new_index = GetUniqueId();
		unsigned int end_loop = GetUniqueId();
		unsigned int element_pointer = GetUniqueId();
		
		source += PrintString(
			"%%.%u = call fastcc %%t.dynvect (%%t.size, %%t.size)* @.dynvect.malloc (%%t.size %%.%u, %%t.size %u)%s ; Allocate a new vector\n"
			"store %%t.size 0, %%t.size* %%counter.%u%s\n"
			"%%.%u = call fastcc i8* @.dynvect.write (%%t.dynvect %%.%u, %%t.size 0, %%t.size %%.%u)%s ; Get the vector/buffer\n"
			"%%.%u = bitcast i8* %%.%u to %s*%s ; Convert the pointer type\n"
			"call fastcc void (%%t.dynvect)* @.dynvect.commit.before(%%t.dynvect %%.%u)%s ; Commit the write\n"
			
			"br label %%.dynamic_vector_create_begin.%u\n"
			".dynamic_vector_create_begin.%u:\n"
				"\t%%.%u = load %%t.size* %%counter.%u ; Get the index into the array\n"
				"\t%%.%u = icmp uge %%t.size %%.%u, %%.%u ; Compare it with the number of elements\n"
				"\tbr i1 %%.%u, label %%.dynamic_vector_create_end.%u, label %%.dynamic_vector_create_body.%u\n"
			".dynamic_vector_create_body.%u:\n"
				
				"%s"
				
				"\t%%.%u = getelementptr %s* %%.%u, %%t.size %%.%u ; calculate the pointer to the memory to store into\n"
				"\tstore %s %%.%u, %s* %%.%u ; store the element \n"
				
				"\t%%.%u = add %%t.size %%.%u, 1 ; add one to the index\n"
				"\tstore %%t.size %%.%u, %%t.size* %%counter.%u ; store the new index\n"
				"\tbr label %%.dynamic_vector_create_begin.%u ; Loop\n"
			".dynamic_vector_create_end.%u:\n"
				"\tcall fastcc void (%%t.dynvect)* @.dynvect.commit.after(%%t.dynvect %%.%u)%s ; Commit the write\n",
			
			created_vector, size_expression, type->GetType()->GetCodeGenerator<CodeGeneratorTypeLlvm>()->GetSize(context), debug.c_str(),
			GetNode()->GetId(), debug.c_str(),
			buffer_i8, created_vector, size_expression, debug.c_str(),
			buffer_type, buffer_i8, element_type.c_str(), debug.c_str(),
			n, debug.c_str(),
			
			GetNode()->GetId(),
			GetNode()->GetId(),
			index, GetNode()->GetId(),
			end_loop, index, size_expression,
			end_loop, GetNode()->GetId(), GetNode()->GetId(),
			GetNode()->GetId(),
			
			Indent(loop_code, 1).c_str(),
			
			element_pointer, element_type.c_str(), buffer_type, index,
			element_type.c_str(), created_element, element_type.c_str(), element_pointer,
			
			new_index, index,
			new_index, GetNode()->GetId(),
			GetNode()->GetId(),
			GetNode()->GetId(),
			n, debug.c_str()
		);
	}
	
	return source;
}

bool CodeGeneratorExpressionDynamicVectorLlvm::CopyIfCopyable()
{
	return false;
}

std::string CodeGeneratorExpressionDynamicVectorLlvm::GenerateFnBegin(CompileContext &context, unsigned int d)
{
	std::string source = GetNode()->GetExpression()->GetCodeGenerator<CodeGeneratorExpressionLlvm>()->GenerateFnBegin(context, d);
	source += PrintString("%%counter.%u = alloca %%t.size ; counter for the dynamic vector expression\n", GetNode()->GetId());
	return source;
}

std::string CodeGeneratorExpressionDynamicVectorLlvm::GenerateFnEnd(CompileContext &context, unsigned int d)
{
	return GetNode()->GetExpression()->GetCodeGenerator<CodeGeneratorExpressionLlvm>()->GenerateFnEnd(context, d);
}




CodeGeneratorDynamicVectorOperationLlvm::CodeGeneratorDynamicVectorOperationLlvm(AstNodeStatementDynamicVectorOperation *node, std::string op) :
	CodeGeneratorStatementLlvm(node), m_op(op)
{
}

std::string CodeGeneratorDynamicVectorOperationLlvm::Generate(CompileContext &context, unsigned int d)
{
	unsigned int dynvect = GetUniqueId();
	return PrintString(
		"; CodeGeneratorDynamicVectorOperationLlvm\n"
			"\t; Expression\n"
				"%s"
			"\t; Operation\n"
				"\t\tcall fastcc void @.dynvect.%s(%%t.dynvect %%.%u)\n",
		Indent(GetNode()->GetExpression()->GetCodeGenerator<CodeGeneratorExpressionLlvm>()->Generate(context, d, dynvect), 2).c_str(),
		m_op.c_str(), dynvect
	);
}

std::string CodeGeneratorDynamicVectorOperationLlvm::GenerateFnBegin(CompileContext &context, unsigned int d)
{
	return GetNode()->GetExpression()->GetCodeGenerator<CodeGeneratorExpressionLlvm>()->GenerateFnBegin(context, d);
}

std::string CodeGeneratorDynamicVectorOperationLlvm::GenerateFnEnd(CompileContext &context, unsigned int d)
{
	return GetNode()->GetExpression()->GetCodeGenerator<CodeGeneratorExpressionLlvm>()->GenerateFnEnd(context, d);
}

std::string CodeGeneratorDynamicVectorOperationLlvm::GenerateTopLevel(CompileContext &context, unsigned int d)
{
	return GetNode()->GetExpression()->GetCodeGenerator<CodeGeneratorExpressionLlvm>()->GenerateTopLevel(context, d);
}

CodeGeneratorDynamicVectorCommitBeforeLlvm::CodeGeneratorDynamicVectorCommitBeforeLlvm(AstNodeDynamicVectorCommitBefore *node) :
	CodeGeneratorDynamicVectorOperationLlvm(node, "commit.before")
{
}

CodeGeneratorDynamicVectorCommitBeforeLlvm *CodeGeneratorDynamicVectorCommitBeforeLlvm::Copy()
{
	return new CodeGeneratorDynamicVectorCommitBeforeLlvm(GetNode());
}

CodeGeneratorDynamicVectorCommitAfterLlvm::CodeGeneratorDynamicVectorCommitAfterLlvm(AstNodeDynamicVectorCommitAfter *node) :
	CodeGeneratorDynamicVectorOperationLlvm(node, "commit.after")
{
}

CodeGeneratorDynamicVectorCommitAfterLlvm *CodeGeneratorDynamicVectorCommitAfterLlvm::Copy()
{
	return new CodeGeneratorDynamicVectorCommitAfterLlvm(GetNode());
}




CodeGeneratorDynamicVectorGetPointerLlvm::CodeGeneratorDynamicVectorGetPointerLlvm(AstNodeDynamicVectorGetPointer *node) :
	CodeGeneratorExpressionLlvm(node)
{
}

CodeGeneratorDynamicVectorGetPointerLlvm *CodeGeneratorDynamicVectorGetPointerLlvm::Copy()
{
	return new CodeGeneratorDynamicVectorGetPointerLlvm(GetNode());
}

std::string CodeGeneratorDynamicVectorGetPointerLlvm::Generate(CompileContext &context, unsigned int d, unsigned int n)
{
	unsigned int dynvect = GetUniqueId();
	unsigned int length = GetUniqueId();
	unsigned int result_buffer = GetUniqueId();
	//And we return the bitcasted buffer into n
	
	return PrintString(
		"; CodeGeneratorDynamicVectorGetPointerLlvm\n"
			"\t; Expression\n"
				"%s"
			"\t; Dynamic Vector Operation\n"
				"\t\t%%.%u = call fastcc %%t.size @.dynvect.element_count(%%t.dynvect %%.%u) ; Get the length of the vector\n"
				"\t\t%%.%u = call fastcc i8*(%%t.dynvect, %%t.size, %%t.size)* @.dynvect.%s (%%t.dynvect %%.%u, %%t.size 0, %%t.size %%.%u) ; Get the pointer\n"
				"\t\t%%.%u = bitcast i8* %%.%u to %s* ; Cast the pointer\n",
		//TODO: do we need to run @.dynvect.free.0.0 on dynvect? (the original - the one that might be gotten rid of by the save)
		
		Indent(GetNode()->GetLValue()->GetCodeGenerator<CodeGeneratorExpressionLValueLlvm>()->Generate(context, d, dynvect), 2).c_str(),
		
		length, dynvect,
		result_buffer, GetNode()->GetWrite() ? "readwrite" : "read", dynvect, length,
		n, result_buffer, tracked_cast<AstNodeTypeDynamicVector>(GetNode()->GetLValue()->GetType())->GetType()->GetCodeGenerator<CodeGeneratorTypeLlvm>()->Generate(context, d).c_str()
	);
}

std::string CodeGeneratorDynamicVectorGetPointerLlvm::GenerateFnBegin(CompileContext &context, unsigned int d)
{
	return GetNode()->GetLValue()->GetCodeGenerator<CodeGeneratorExpressionLValueLlvm>()->GenerateFnBegin(context, d);
}

std::string CodeGeneratorDynamicVectorGetPointerLlvm::GenerateFnEnd(CompileContext &context, unsigned int d)
{
	return GetNode()->GetLValue()->GetCodeGenerator<CodeGeneratorExpressionLValueLlvm>()->GenerateFnEnd(context, d);
}

std::string CodeGeneratorDynamicVectorGetPointerLlvm::GenerateTopLevel(CompileContext &context, unsigned int d)
{
	return GetNode()->GetLValue()->GetCodeGenerator<CodeGeneratorExpressionLValueLlvm>()->GenerateTopLevel(context, d);
}
