/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2012-2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef CODEGEN_LLVM_DYNVECT_HPP
#define CODEGEN_LLVM_DYNVECT_HPP

#include "codegen.hpp"

class CodeGeneratorTypeDynamicVectorLlvm : public CodeGeneratorTypeLlvm
{
	public:
		CodeGeneratorTypeDynamicVectorLlvm(AstNodeTypeDynamicVector *node);
		virtual CodeGeneratorTypeDynamicVectorLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d);
		virtual unsigned int GetSize(CompileContext &context);
		virtual std::string GenerateDebug(CompileContext &context, unsigned int d);
		virtual std::string GenerateTopLevel(CompileContext &context, unsigned int d);
		
		virtual std::string GenerateCreate(CompileContext &context, unsigned int d, unsigned int value);
		virtual std::string GenerateDestroy(CompileContext &context, unsigned int d, unsigned int value);
		virtual std::string GenerateCopy(CompileContext &context, unsigned int d, unsigned int src, unsigned int dst);
		
		static std::string GenerateDebugGeneric(CompileContext &context, unsigned int toplevel);
		static std::string GenerateDebugTopLevelGeneric(CompileContext &context, unsigned int n, std::string type);
		
		unsigned int GetAlignment(CompileContext &context); //Returns the alignment of an element of this vector
		
		AstType(AstNodeTypeDynamicVector);
		
	private:
		std::string GenerateOperation(CompileContext &context, unsigned int d, unsigned int value, std::string loop_code,
			std::string dynvect_function, unsigned int element, unsigned int dst = 0, unsigned int element_dst = 0);
};

class CodeGeneratorExpressionDynamicVectorLlvm : public CodeGeneratorExpressionLlvm
{
	public:
		CodeGeneratorExpressionDynamicVectorLlvm(AstNodeExpressionDynamicVector *node);
		virtual CodeGeneratorExpressionDynamicVectorLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d, unsigned int n);
		virtual std::string GenerateFnBegin(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnEnd(CompileContext &context, unsigned int d);
		virtual bool CopyIfCopyable();
		
		AstType(AstNodeExpressionDynamicVector);
};

class CodeGeneratorDynamicVectorOperationLlvm : public CodeGeneratorStatementLlvm
{
	public:
		CodeGeneratorDynamicVectorOperationLlvm(AstNodeStatementDynamicVectorOperation *node, std::string op);
		virtual AstNodeStatementDynamicVectorOperation *GetNode() = 0;
		
		virtual std::string Generate(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnBegin(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnEnd(CompileContext &context, unsigned int d);
		virtual std::string GenerateTopLevel(CompileContext &context, unsigned int d);
		
	private:
		std::string m_op;
};

class CodeGeneratorDynamicVectorCommitBeforeLlvm : public CodeGeneratorDynamicVectorOperationLlvm
{
	public:
		CodeGeneratorDynamicVectorCommitBeforeLlvm(AstNodeDynamicVectorCommitBefore *node);
		virtual CodeGeneratorDynamicVectorCommitBeforeLlvm *Copy();
		AstType(AstNodeDynamicVectorCommitBefore);
};

class CodeGeneratorDynamicVectorCommitAfterLlvm : public CodeGeneratorDynamicVectorOperationLlvm
{
	public:
		CodeGeneratorDynamicVectorCommitAfterLlvm(AstNodeDynamicVectorCommitAfter *node);
		virtual CodeGeneratorDynamicVectorCommitAfterLlvm *Copy();
		AstType(AstNodeDynamicVectorCommitAfter);
};

class CodeGeneratorDynamicVectorGetPointerLlvm : public CodeGeneratorExpressionLlvm
{
	public:
		CodeGeneratorDynamicVectorGetPointerLlvm(AstNodeDynamicVectorGetPointer *node);
		virtual CodeGeneratorDynamicVectorGetPointerLlvm *Copy();
		AstType(AstNodeDynamicVectorGetPointer);
		
		/*
		 * Puts the value of the expression into %.n
		 */
		virtual std::string Generate(CompileContext &context, unsigned int d, unsigned int n);
		virtual std::string GenerateFnBegin(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnEnd(CompileContext &context, unsigned int d);
		virtual std::string GenerateTopLevel(CompileContext &context, unsigned int d);
};


#endif

