/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2012-2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef CODEGEN_LLVM_EXPRESSION_HPP
#define CODEGEN_LLVM_EXPRESSION_HPP

#include "codegen.hpp"

class CodeGeneratorExpressionLlvm : public CodeGeneratorLlvm
{
	public:
		CodeGeneratorExpressionLlvm(AstNodeExpression *node);
		
		virtual AstNodeExpression *GetNode() = 0;
		
		/*
		 * Puts the value of the expression into %.n
		 */
		std::string GenerateCopy(CompileContext &context, unsigned int d, unsigned int n);
		virtual std::string Generate(CompileContext &context, unsigned int d, unsigned int n) = 0; //returns the value of the expression into n
		virtual std::string GenerateFnBegin(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnEnd(CompileContext &context, unsigned int d);
		virtual std::string GenerateTopLevel(CompileContext &context, unsigned int d);
		virtual bool CopyIfCopyable(); //Returns true if we should copy when the type is copyable (avoids memory leaks/double frees/etc)
		
	private:
};

class CodeGeneratorExpressionLValueLlvm : public CodeGeneratorExpressionLlvm
{
	public:
		CodeGeneratorExpressionLValueLlvm(AstNodeExpressionLValue *node);
		
		virtual AstNodeExpressionLValue *GetNode() = 0;
		
		/*
		 * Stores %.n into the lvalue's storage
		 * This is NOT responsible for producing copies and stuff like that,
		 * which is the responsibility of variable assignment (and variable declaration)
		 */
		virtual std::string GenerateReference(CompileContext &context, unsigned int d, unsigned int n) = 0;
		virtual std::string GenerateStore(CompileContext &context, unsigned int d, unsigned int n) = 0;
		virtual std::string Generate(CompileContext &context, unsigned int d, unsigned int n) = 0;
		
	private:
};


class CodeGeneratorExpressionVoidLlvm : public CodeGeneratorExpressionLlvm
{
	public:
		CodeGeneratorExpressionVoidLlvm(AstNodeExpressionVoid *node);
		virtual CodeGeneratorExpressionVoidLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d, unsigned int n);
		
		AstType(AstNodeExpressionVoid);
};
class CodeGeneratorExpressionBoolLlvm : public CodeGeneratorExpressionLlvm
{
	public:
		CodeGeneratorExpressionBoolLlvm(AstNodeExpressionBool *node);
		virtual CodeGeneratorExpressionBoolLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d, unsigned int n);
		
		AstType(AstNodeExpressionBool);
};

class CodeGeneratorExpressionIntLlvm : public CodeGeneratorExpressionLlvm
{
	public:
		CodeGeneratorExpressionIntLlvm(AstNodeExpressionInt *node);
		virtual CodeGeneratorExpressionIntLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d, unsigned int n);
		
		AstType(AstNodeExpressionInt);
};

class CodeGeneratorExpressionFloatLlvm : public CodeGeneratorExpressionLlvm
{
	public:
		CodeGeneratorExpressionFloatLlvm(AstNodeExpressionFloat *node);
		virtual CodeGeneratorExpressionFloatLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d, unsigned int n);
		
		AstType(AstNodeExpressionFloat);
};

class CodeGeneratorExpressionLengthLlvm : public CodeGeneratorExpressionLlvm
{
	public:
		CodeGeneratorExpressionLengthLlvm(AstNodeExpressionLength *node);
		virtual CodeGeneratorExpressionLengthLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d, unsigned int n);
		virtual std::string GenerateFnBegin(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnEnd(CompileContext &context, unsigned int d);
		
		AstType(AstNodeExpressionLength);
};

class CodeGeneratorExpressionBlockLlvm : public CodeGeneratorExpressionLlvm
{
	public:
		CodeGeneratorExpressionBlockLlvm(AstNodeExpressionBlock *node);
		virtual CodeGeneratorExpressionBlockLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d, unsigned int n);
		virtual std::string GenerateFnBegin(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnEnd(CompileContext &context, unsigned int d);
		virtual std::string GenerateTopLevel(CompileContext &context, unsigned int d);
		
		AstType(AstNodeExpressionBlock);
};

class CodeGeneratorExpressionLValueIdentifierLlvm : public CodeGeneratorExpressionLValueLlvm
{
	public:
		CodeGeneratorExpressionLValueIdentifierLlvm(AstNodeExpressionLValueIdentifier *node);
		virtual CodeGeneratorExpressionLValueIdentifierLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d, unsigned int n);
		virtual std::string GenerateStore(CompileContext &context, unsigned int d, unsigned int n);
		virtual std::string GenerateReference(CompileContext &context, unsigned int d, unsigned int n);
		
		AstType(AstNodeExpressionLValueIdentifier);
};

class CodeGeneratorExpressionLValueVectorLlvm : public CodeGeneratorExpressionLValueLlvm
{
	public:
		CodeGeneratorExpressionLValueVectorLlvm(AstNodeExpressionLValueVector *node);
		virtual CodeGeneratorExpressionLValueVectorLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d, unsigned int n);
		virtual std::string GenerateStore(CompileContext &context, unsigned int d, unsigned int n);
		virtual std::string GenerateFnBegin(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnEnd(CompileContext &context, unsigned int d);
		virtual std::string GenerateReference(CompileContext &context, unsigned int d, unsigned int n);
		
		AstType(AstNodeExpressionLValueVector);
};

#include "struct.hpp"
#include "op.hpp"


#endif
