/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2012-2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef CODEGEN_LLVM_OP_HPP
#define CODEGEN_LLVM_OP_HPP

#include "expression.hpp"

class CodeGeneratorExpressionBinaryLlvm : public CodeGeneratorExpressionLlvm
{
	public:
		CodeGeneratorExpressionBinaryLlvm(AstNodeExpressionBinary *node);
		virtual AstNodeExpressionBinary *GetNode() = 0;
		
		virtual std::string GenerateFnBegin(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnEnd(CompileContext &context, unsigned int d);
		virtual std::string GenerateTopLevel(CompileContext &context, unsigned int d);
		
		virtual std::string Generate(CompileContext &context, unsigned int d, unsigned int n);
		virtual std::string Generate(CompileContext &context, unsigned int d, AstNodeType *type, unsigned int n, unsigned int a, unsigned int b, unsigned int id) = 0;
		
	protected:
		std::string GetFloatFlags(AstNodeType *type);
};


class CodeGeneratorExpressionCastLlvm : public CodeGeneratorExpressionLlvm
{
	public:
		CodeGeneratorExpressionCastLlvm(AstNodeExpressionCast *node);
		virtual CodeGeneratorExpressionCastLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d, unsigned int n);
		virtual std::string GenerateFnBegin(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnEnd(CompileContext &context, unsigned int d);
		virtual std::string GenerateTopLevel(CompileContext &context, unsigned int d);
		
		AstType(AstNodeExpressionCast);
};

class CodeGeneratorExpressionBinaryAdditionLlvm : public CodeGeneratorExpressionBinaryLlvm
{
	public:
		CodeGeneratorExpressionBinaryAdditionLlvm(AstNodeExpressionBinaryAddition *node);
		virtual CodeGeneratorExpressionBinaryAdditionLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d, AstNodeType *type, unsigned int n, unsigned int a, unsigned int b, unsigned int id);
		
		AstType(AstNodeExpressionBinaryAddition);
};

class CodeGeneratorExpressionBinarySubtractionLlvm : public CodeGeneratorExpressionBinaryLlvm
{
	public:
		CodeGeneratorExpressionBinarySubtractionLlvm(AstNodeExpressionBinarySubtraction *node);
		virtual CodeGeneratorExpressionBinarySubtractionLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d, AstNodeType *type, unsigned int n, unsigned int a, unsigned int b, unsigned int id);
		
		AstType(AstNodeExpressionBinarySubtraction);
};

class CodeGeneratorExpressionBinaryMultiplicationLlvm : public CodeGeneratorExpressionBinaryLlvm
{
	public:
		CodeGeneratorExpressionBinaryMultiplicationLlvm(AstNodeExpressionBinaryMultiplication *node);
		virtual CodeGeneratorExpressionBinaryMultiplicationLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d, AstNodeType *type, unsigned int n, unsigned int a, unsigned int b, unsigned int id);
		
		AstType(AstNodeExpressionBinaryMultiplication);
};

class CodeGeneratorExpressionBinaryDivisionLlvm : public CodeGeneratorExpressionBinaryLlvm
{
	public:
		CodeGeneratorExpressionBinaryDivisionLlvm(AstNodeExpressionBinaryDivision *node);
		virtual CodeGeneratorExpressionBinaryDivisionLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d, AstNodeType *type, unsigned int n, unsigned int a, unsigned int b, unsigned int id);
		
		AstType(AstNodeExpressionBinaryDivision);
};

class CodeGeneratorExpressionBinaryModuloLlvm : public CodeGeneratorExpressionBinaryLlvm
{
	public:
		CodeGeneratorExpressionBinaryModuloLlvm(AstNodeExpressionBinaryModulo *node);
		virtual CodeGeneratorExpressionBinaryModuloLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d, AstNodeType *type, unsigned int n, unsigned int a, unsigned int b, unsigned int id);
		
		AstType(AstNodeExpressionBinaryModulo);
};

class CodeGeneratorExpressionBinaryAndLlvm : public CodeGeneratorExpressionBinaryLlvm
{
	public:
		CodeGeneratorExpressionBinaryAndLlvm(AstNodeExpressionBinaryAnd *node);
		virtual CodeGeneratorExpressionBinaryAndLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d, AstNodeType *type, unsigned int n, unsigned int a, unsigned int b, unsigned int id);
		
		AstType(AstNodeExpressionBinaryAnd);
};

class CodeGeneratorExpressionBinaryOrLlvm : public CodeGeneratorExpressionBinaryLlvm
{
	public:
		CodeGeneratorExpressionBinaryOrLlvm(AstNodeExpressionBinaryOr *node);
		virtual CodeGeneratorExpressionBinaryOrLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d, AstNodeType *type, unsigned int n, unsigned int a, unsigned int b, unsigned int id);
		
		AstType(AstNodeExpressionBinaryOr);
};

class CodeGeneratorExpressionBinaryXorLlvm : public CodeGeneratorExpressionBinaryLlvm
{
	public:
		CodeGeneratorExpressionBinaryXorLlvm(AstNodeExpressionBinaryXor *node);
		virtual CodeGeneratorExpressionBinaryXorLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d, AstNodeType *type, unsigned int n, unsigned int a, unsigned int b, unsigned int id);
		
		AstType(AstNodeExpressionBinaryXor);
};

class CodeGeneratorExpressionBinaryShlLlvm : public CodeGeneratorExpressionBinaryLlvm
{
	public:
		CodeGeneratorExpressionBinaryShlLlvm(AstNodeExpressionBinaryShl *node);
		virtual CodeGeneratorExpressionBinaryShlLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d, AstNodeType *type, unsigned int n, unsigned int a, unsigned int b, unsigned int id);
		
		AstType(AstNodeExpressionBinaryShl);
};

class CodeGeneratorExpressionBinaryShrLlvm : public CodeGeneratorExpressionBinaryLlvm
{
	public:
		CodeGeneratorExpressionBinaryShrLlvm(AstNodeExpressionBinaryShr *node);
		virtual CodeGeneratorExpressionBinaryShrLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d, AstNodeType *type, unsigned int n, unsigned int a, unsigned int b, unsigned int id);
		
		AstType(AstNodeExpressionBinaryShr);
};

class CodeGeneratorExpressionBinaryCompareLlvm : public CodeGeneratorExpressionBinaryLlvm
{
	public:
		CodeGeneratorExpressionBinaryCompareLlvm(AstNodeExpressionBinaryCompare *node, std::string llvmop, bool uses_sign = true);
		virtual CodeGeneratorExpressionBinaryCompareLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d, unsigned int n);
		std::string Generate(CompileContext &context, unsigned int d, AstNodeType *type, unsigned int n, unsigned int a, unsigned int b, unsigned int id);
		
		AstType(AstNodeExpressionBinaryCompare);
		
	private:
		std::string m_llvmop;
		bool m_uses_sign;
};

#endif
