/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2012-2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef CODEGEN_LLVM_PARALLEL_HPP
#define CODEGEN_LLVM_PARALLEL_HPP

#include "statement.hpp"

class CodeGeneratorStatementParallelLlvm : public CodeGeneratorStatementLlvm
{
	public:
		CodeGeneratorStatementParallelLlvm(AstNodeStatementParallel *node);
		virtual CodeGeneratorStatementParallelLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnBegin(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnEnd(CompileContext &context, unsigned int d);
		virtual std::string GenerateTopLevel(CompileContext &context, unsigned int d);
		
		AstType(AstNodeStatementParallel);
};

class CodeGeneratorStatementParallelSyncLlvm : public CodeGeneratorStatementParallelLlvm
{
	public:
		CodeGeneratorStatementParallelSyncLlvm(AstNodeStatementParallelSync *node);
		virtual CodeGeneratorStatementParallelSyncLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnBegin(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnEnd(CompileContext &context, unsigned int d);
		virtual std::string GenerateTopLevel(CompileContext &context, unsigned int d);
		
		AstType(AstNodeStatementParallelSync);
};



class CodeGeneratorStatementVariableDeclarationForSpawnLlvm : public CodeGeneratorStatementVariableDeclarationLlvm
{
	public:
		CodeGeneratorStatementVariableDeclarationForSpawnLlvm(AstNodeStatementVariableDeclaration *node);
		virtual CodeGeneratorStatementVariableDeclarationForSpawnLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnBegin(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnEnd(CompileContext &context, unsigned int d);
		virtual std::string GenerateTopLevel(CompileContext &context, unsigned int d);
};

class CodeGeneratorStatementParallelSpawnExtractedLlvm : public CodeGeneratorStatementParallelLlvm
{
	public:
		CodeGeneratorStatementParallelSpawnExtractedLlvm(AstNodeStatementParallelSpawn *node, std::list<AstNodeStatementVariableDeclaration *> variables, unsigned int linkid);
		
		virtual CodeGeneratorStatementParallelSpawnExtractedLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnBegin(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnEnd(CompileContext &context, unsigned int d);
		virtual std::string GenerateTopLevel(CompileContext &context, unsigned int d);
		
	private:
		std::list<AstNodeStatementVariableDeclaration *> m_variables;
		unsigned int m_linkid;
};

class CodeGeneratorStatementExtractedLlvm : public CodeGeneratorStatementLlvm
{
	public:
		AstType(AstNodeStatementExtracted);
		
		CodeGeneratorStatementExtractedLlvm(AstNodeStatementExtracted *node);
		
		CodeGeneratorStatementExtractedLlvm *Copy();
		std::string Generate(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnBegin(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnEnd(CompileContext &context, unsigned int d);
		virtual std::string GenerateTopLevel(CompileContext &context, unsigned int d);
};

class CodeGeneratorTypeLockLlvm : public CodeGeneratorTypeLlvm
{
	public:
		CodeGeneratorTypeLockLlvm(AstNodeTypeLock *node);
		virtual CodeGeneratorTypeLockLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d);
		virtual unsigned int GetSize(CompileContext &context);
		virtual std::string GenerateDebug(CompileContext &context, unsigned int d);
		
		virtual std::string GenerateCreate(CompileContext &context, unsigned int d, unsigned int n);
		virtual std::string GenerateDestroy(CompileContext &context, unsigned int d, unsigned int n);
		virtual std::string GenerateCopy(CompileContext &context, unsigned int d, unsigned int src, unsigned int dst);
		
		AstType(AstNodeTypeLock);
};

class CodeGeneratorStatementParallelAtomicLlvm : public CodeGeneratorStatementParallelLlvm
{
	public:
		CodeGeneratorStatementParallelAtomicLlvm(AstNodeStatementParallelAtomic *node);
		virtual CodeGeneratorStatementParallelAtomicLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnBegin(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnEnd(CompileContext &context, unsigned int d);
		virtual std::string GenerateTopLevel(CompileContext &context, unsigned int d);
		
		bool GetUnlockedByAtomic();
		void SetUnlockedByAtomic(bool unlocked_by_atomic);
		
		AstType(AstNodeStatementParallelAtomic);
		
	private:
		bool m_unlocked_by_atomic;
};

class CodeGeneratorExpressionLValueLockLlvm : public CodeGeneratorExpressionLValueLlvm
{
	public:
		CodeGeneratorExpressionLValueLockLlvm(AstNodeExpressionLValueLock *node);
		virtual CodeGeneratorExpressionLValueLockLlvm *Copy();
		virtual std::string GenerateReference(CompileContext &context, unsigned int d, unsigned int n); //This passes through to the underlying lvalue
		virtual std::string GenerateStore(CompileContext &context, unsigned int d, unsigned int n);
		virtual std::string Generate(CompileContext &context, unsigned int d, unsigned int n);
		
		virtual std::string GenerateFnBegin(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnEnd(CompileContext &context, unsigned int d);
		virtual std::string GenerateTopLevel(CompileContext &context, unsigned int d);
		virtual bool CopyIfCopyable(); //This passes through to the underlying lvalue
		
		std::string GenerateLock(CompileContext &context, unsigned int d, unsigned int n); //Puts the lock object into n
		bool GetUnlockedByAtomic();
		void SetUnlockedByAtomic(bool unlocked_by_atomic);
		
		AstType(AstNodeExpressionLValueLock);
		
	private:
		std::string GenerateLoadStore(CompileContext &context, unsigned int d, unsigned int n, bool store);
		
		bool m_unlocked_by_atomic;
};


#endif
 
