/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "../../ast/astnode.hpp"
#include "codegen.hpp"



CodeGeneratorTypeReferenceLlvm::CodeGeneratorTypeReferenceLlvm(AstNodeTypeReference *node) :
	CodeGeneratorTypeLlvm(node)
{
}

CodeGeneratorTypeReferenceLlvm *CodeGeneratorTypeReferenceLlvm::Copy()
{
	return new CodeGeneratorTypeReferenceLlvm(GetNode());
}

unsigned int CodeGeneratorTypeReferenceLlvm::GetSize(CompileContext &context)
{
	return context.GetComputeSpec().GetCpu().GetSizeT() * 2;
}

std::string CodeGeneratorTypeReferenceLlvm::Generate(CompileContext &context, unsigned int d)
{
	return "%t.ref";
}

std::string CodeGeneratorTypeReferenceLlvm::GenerateDebug(CompileContext &context, unsigned int d)
{
	if (!context.GetDebug()){
		return "";
	}
	return GenerateDebugGeneric(context, GetNode()->GetId());
}

std::string CodeGeneratorTypeReferenceLlvm::GenerateTopLevel(CompileContext &context, unsigned int d)
{
	std::string source = GetNode()->GetType()->GetCodeGenerator<CodeGeneratorTypeLlvm>()->GenerateTopLevel(context, d);
	if (context.GetDebug()){
		source += GenerateDebugTopLevelGeneric(context, GetNode()->GetId(), GetNode()->GetType()->GetCodeGenerator<CodeGeneratorTypeLlvm>()->GenerateDebug(context, d));
	}
	return source;
}

std::string CodeGeneratorTypeReferenceLlvm::GenerateCreate(CompileContext &context, unsigned int d, unsigned int value)
{
	std::string debug = GetDebug(context, d);
	return PrintString("%%.%u = call fastcc %%t.ref (i8*)* @.ref.create(i8* null)%s ; Create a null reference\n", value, debug.c_str());
}

std::string CodeGeneratorTypeReferenceLlvm::GenerateDestroy(CompileContext &context, unsigned int d, unsigned int value)
{
	std::string debug = GetDebug(context, d);
	return PrintString("call fastcc void (%%t.ref)* @.ref.destroy(%%t.ref %%.%u)%s ; Destroy the reference\n", value, debug.c_str());
}

std::string CodeGeneratorTypeReferenceLlvm::GenerateDebugGeneric(CompileContext &context, unsigned int toplevel)
{
	return PrintString("metadata !%u", toplevel);
}

std::string CodeGeneratorTypeReferenceLlvm::GenerateDebugTopLevelGeneric(CompileContext &context, unsigned int n, std::string type)
{
	return PrintString(
		"!%u = metadata !{\n"
			"\ti32 %u, ; Tag (DW_TAG_structure_type)\n"
			"\tnull, ; Reference to context\n"
			"\tmetadata !\"Reference\", ; Name (may be "" for anonymous types)\n"
			"\tmetadata !1, ; Reference to file where defined (may be NULL)\n"
			"\ti32 0, ; Line number where defined (may be 0)\n"
			"\ti64 %u, ; Size in bits\n"
			"\ti64 8, ; Alignment in bits\n"
			"\ti32 0, ; Offset in bits\n"
			"\ti32 0, ; Flags\n"
			"\tnull, ; Reference to type derived from\n"
			
			"\tmetadata !{ ; Reference to array of member descriptors\n"
				"\t\tmetadata !{\n"
					"\t\t\ti32 %u, ; Tag (DW_TAG_member)\n"
					"\t\t\tmetadata !%u, ; Reference to context\n"
					"\t\t\tmetadata !\"type\", ; Name (may be "" for anonymous types)\n"
					"\t\t\tmetadata !1, ; Reference to file where defined (may be NULL)\n"
					"\t\t\ti32 0, ; Line number where defined (may be 0)\n"
					"\t\t\ti64 %u, ; Size in bits\n"
					"\t\t\ti64 8, ; Alignment in bits\n"
					"\t\t\ti64 %u, ; Offset in bits\n"
					"\t\t\ti32 0, ; Flags to encode attributes, e.g. private\n"
					"\t\t\tmetadata !{\n"
						"\t\t\t\ti32 %u, ; Tag (DW_TAG_base_type)\n"
						"\t\t\t\tnull, ; Reference to context\n"
						"\t\t\t\tmetadata !\"size_t\", ; Name (may be "" for anonymous types)\n"
						"\t\t\t\tnull, ; Reference to file where defined (may be NULL)\n"
						"\t\t\t\ti32 0, ; Line number where defined (may be 0)\n"
						"\t\t\t\ti64 %u, ; Size in bits\n"
						"\t\t\t\ti64 8, ; Alignment in bits\n"
						"\t\t\t\ti64 0, ; Offset in bits\n"
						"\t\t\t\ti32 0, ; Flags\n"
						"\t\t\t\ti32 7 ; DWARF type encoding\n"
					"\t\t\t}\n"
				"\t\t},\n"
				
				"\t\tmetadata !{\n"
					"\t\t\ti32 %u, ; Tag (DW_TAG_member)\n"
					"\t\t\tmetadata !%u, ; Reference to context\n"
					"\t\t\tmetadata !\"data\", ; Name (may be "" for anonymous types)\n"
					"\t\t\tmetadata !1, ; Reference to file where defined (may be NULL)\n"
					"\t\t\ti32 0, ; Line number where defined (may be 0)\n"
					"\t\t\ti64 %u, ; Size in bits\n"
					"\t\t\ti64 8, ; Alignment in bits\n"
					"\t\t\ti64 %u, ; Offset in bits\n"
					"\t\t\ti32 0, ; Flags to encode attributes, e.g. private\n"
					"\t\t\tmetadata !{ ; Reference to type derived from\n"
						"\t\t\t\ti32 %u, ; Tag (DW_TAG_pointer_type)\n"
						"\t\t\t\tnull, ; Reference to context\n"
						"\t\t\t\tmetadata !\"\", ; Name (may be "" for anonymous types)\n"
						"\t\t\t\tnull, ; Reference to file where defined (may be NULL)\n"
						"\t\t\t\ti32 0, ; Line number where defined (may be 0)\n"
						"\t\t\t\ti64 %u, ; Size in bits\n"
						"\t\t\t\ti64 8, ; Alignment in bits\n"
						"\t\t\t\ti64 0, ; Offset in bits\n"
						"\t\t\t\ti32 0, ; Flags to encode attributes, e.g. private\n"
						"%s\n"
					"\t\t\t}\n"
				"\t\t}\n"
			"\t},\n"
			
			"\ti32 0,\n"
			"\ti32 0,\n"
			"\ti32 0\n"
		"}\n",
		n,
		LlvmDebugVersion + 19,
		context.GetComputeSpec().GetCpu().GetSizeT() * 2,
		
		LlvmDebugVersion + 13,
		n,
		context.GetComputeSpec().GetCpu().GetSizeT(),
		context.GetComputeSpec().GetCpu().GetSizeT() * 0,
		LlvmDebugVersion + 36,
		context.GetComputeSpec().GetCpu().GetSizeT(),
		
		LlvmDebugVersion + 13,
		n,
		context.GetComputeSpec().GetCpu().GetSizeT(),
		context.GetComputeSpec().GetCpu().GetSizeT() * 1,
		LlvmDebugVersion + 15,
		context.GetComputeSpec().GetCpu().GetSizeT(),
		Indent(type, 4).c_str()
	);
}




CodeGeneratorExpressionReferenceLlvm::CodeGeneratorExpressionReferenceLlvm(AstNodeExpressionReference *node) :
	CodeGeneratorExpressionLlvm(node)
{
}

CodeGeneratorExpressionReferenceLlvm *CodeGeneratorExpressionReferenceLlvm::Copy()
{
	return new CodeGeneratorExpressionReferenceLlvm(GetNode());
}

std::string CodeGeneratorExpressionReferenceLlvm::Generate(CompileContext &context, unsigned int d, unsigned int n)
{
	return GetNode()->GetLValue()->GetCodeGenerator<CodeGeneratorExpressionLValueLlvm>()->GenerateReference(context, d, n);
}




CodeGeneratorExpressionLValueDereferenceLlvm::CodeGeneratorExpressionLValueDereferenceLlvm(AstNodeExpressionLValueDereference *node) :
	CodeGeneratorExpressionLValueLlvm(node)
{
}

CodeGeneratorExpressionLValueDereferenceLlvm *CodeGeneratorExpressionLValueDereferenceLlvm::Copy()
{
	return new CodeGeneratorExpressionLValueDereferenceLlvm(GetNode());
}

std::string CodeGeneratorExpressionLValueDereferenceLlvm::Generate(CompileContext &context, unsigned int d, unsigned int n)
{
	std::string debug = GetDebug(context, d);
	std::string type = tracked_cast<AstNodeTypeReference>(GetNode()->GetExpression()->GetType())->GetType()->GetCodeGenerator<CodeGeneratorTypeLlvm>()->Generate(context, d);
	
	unsigned int reference_expression = GetUniqueId();
	unsigned int pointer_i8 = GetUniqueId();
	unsigned int pointer_type = GetUniqueId();
	
	return PrintString(
		"; Dereference reference %s\n"
			"%s\n"
			"\t%%.%u = call fastcc i8* (%%t.ref)* @.ref.read(%%t.ref %%.%u)%s ; Put the reference into memory, and get its address\n"
			"\tcall fastcc void (%%t.ref)* @.ref.commit.before(%%t.ref %%.%u)%s ; Commit the read\n"
			"\t%%.%u = bitcast i8* %%.%u to %s* ; Convert the pointer type\n"
			"\t%%.%u = load %s* %%.%u\n"
			"\tcall fastcc void (%%t.ref)* @.ref.commit.after(%%t.ref %%.%u)%s ; Commit the read\n",
		
		GetNode()->GetSourceLocation().c_str(),
		Indent(GetNode()->GetExpression()->GetCodeGenerator<CodeGeneratorExpressionLlvm>()->Generate(context, d, reference_expression), 1).c_str(),
		pointer_i8, reference_expression, debug.c_str(),
		reference_expression, debug.c_str(),
		pointer_type, pointer_i8, type.c_str(),
		n, type.c_str(), pointer_type,
		reference_expression, debug.c_str()
	);
}

std::string CodeGeneratorExpressionLValueDereferenceLlvm::GenerateStore(CompileContext &context, unsigned int d, unsigned int n)
{
	std::string debug = GetDebug(context, d);
	std::string type = tracked_cast<AstNodeTypeReference>(GetNode()->GetExpression()->GetType())->GetType()->GetCodeGenerator<CodeGeneratorTypeLlvm>()->Generate(context, d);
	
	unsigned int reference_expression = GetUniqueId();
	unsigned int pointer_i8 = GetUniqueId();
	unsigned int pointer_type = GetUniqueId();
	
	return PrintString(
		"; Dereference reference %s\n"
			"%s\n"
			"\t%%.%u = call fastcc i8* (%%t.ref)* @.ref.write(%%t.ref %%.%u)%s ; Put the reference into memory, and get its address\n"
			"\tcall fastcc void (%%t.ref)* @.ref.commit.before(%%t.ref %%.%u)%s ; Commit the write\n"
			"\t%%.%u = bitcast i8* %%.%u to %s* ; Convert the pointer type\n"
			"\tstore %s %%.%u, %s* %%.%u\n"
			"\tcall fastcc void (%%t.ref)* @.ref.commit.after(%%t.ref %%.%u)%s ; Commit the write\n",
		
		GetNode()->GetSourceLocation().c_str(),
		Indent(GetNode()->GetExpression()->GetCodeGenerator<CodeGeneratorExpressionLlvm>()->Generate(context, d, reference_expression), 1).c_str(),
		pointer_i8, reference_expression, debug.c_str(),
		reference_expression, debug.c_str(),
		pointer_type, pointer_i8, type.c_str(),
		type.c_str(), n, type.c_str(), pointer_type,
		reference_expression, debug.c_str()
	);
}

std::string CodeGeneratorExpressionLValueDereferenceLlvm::GenerateReference(CompileContext &context, unsigned int d, unsigned int n)
{
	if (!dynamic_cast<AstNodeTypeReference *>(GetNode()->GetExpression()->GetType())){
		throw FormatCompilerError(GetNode(), "Dereference expression is not a reference");
	}
	
	return GetNode()->GetExpression()->GetCodeGenerator<CodeGeneratorExpressionLlvm>()->Generate(context, d, n);
}


CodeGeneratorExpressionAllocLlvm::CodeGeneratorExpressionAllocLlvm(AstNodeExpressionAlloc *node) :
	CodeGeneratorExpressionLlvm(node)
{
}

CodeGeneratorExpressionAllocLlvm *CodeGeneratorExpressionAllocLlvm::Copy()
{
	return new CodeGeneratorExpressionAllocLlvm(GetNode());
}

std::string CodeGeneratorExpressionAllocLlvm::Generate(CompileContext &context, unsigned int d, unsigned int n)
{
	unsigned int pointer_i8 = GetUniqueId();
	unsigned int pointer_type = GetUniqueId();
	unsigned int created_object = GetUniqueId();
	std::string type = GetNode()->GetType()->GetType()->GetCodeGenerator<CodeGeneratorTypeLlvm>()->Generate(context, d);
	
	return PrintString(
		"; Alloc %s\n"
			"\t%%.%u = call fastcc %%t.ref (%%t.size)* @.ref.create.dynamic(%%t.size %u)%s ; Create the reference\n"
			"\t%%.%u = call fastcc i8* (%%t.ref)* @.ref.write(%%t.ref %%.%u)%s ; Get a pointer to the object\n"
			"\t%%.%u = bitcast i8* %%.%u to %s* ; Convert pointer type\n"
			"\tcall fastcc void @.ref.commit.before(%%t.ref %%.%u)%s ; Enable writing to the pointer\n"
			"\t; Create the object\n"
				"%s"
			"\tstore %s %%.%u, %s* %%.%u%s ; Store the created object\n"
			"\tcall fastcc void @.ref.commit.after(%%t.ref %%.%u)%s ; Commit what we've written\n",
			
		GetNode()->GetSourceLocation().c_str(),
			n, GetNode()->GetType()->GetType()->GetCodeGenerator<CodeGeneratorTypeLlvm>()->GetSize(context), GetDebug(context, d).c_str(),
			pointer_i8, n, GetDebug(context, d).c_str(),
			pointer_type, pointer_i8, type.c_str(),
			n, GetDebug(context, d).c_str(),
				Indent(GetNode()->GetType()->GetType()->GetCodeGenerator<CodeGeneratorTypeLlvm>()->GenerateCreate(context, d, created_object), 2).c_str(),
			type.c_str(), created_object, type.c_str(), pointer_type, GetDebug(context, d).c_str(),
			n, GetDebug(context, d).c_str()
	);
}

std::string CodeGeneratorExpressionAllocLlvm::GenerateTopLevel(CompileContext &context, unsigned int d)
{
	return GetNode()->GetType()->GetCodeGenerator<CodeGeneratorTypeLlvm>()->GenerateTopLevel(context, d);
}


CodeGeneratorStatementDeallocLlvm::CodeGeneratorStatementDeallocLlvm(AstNodeStatementDealloc *node) :
	CodeGeneratorStatementLlvm(node)
{
}

CodeGeneratorStatementDeallocLlvm *CodeGeneratorStatementDeallocLlvm::Copy()
{
	return new CodeGeneratorStatementDeallocLlvm(GetNode());
}

std::string CodeGeneratorStatementDeallocLlvm::Generate(CompileContext &context, unsigned int d)
{
	unsigned int pointer_i8 = GetUniqueId();
	unsigned int pointer_type = GetUniqueId();
	unsigned int reference = GetUniqueId();
	unsigned int object = GetUniqueId();
	
	AstNodeTypeReference *reference_type = tracked_cast<AstNodeTypeReference>(GetNode()->GetExpression()->GetType());
	std::string type = reference_type->GetType()->GetCodeGenerator<CodeGeneratorTypeLlvm>()->Generate(context, d);
	
	return PrintString(
		"; Dealloc %s\n"
			"\t; Generate the expression\n"
				"%s"
			"\t%%.%u = call fastcc i8* (%%t.ref)* @.ref.write(%%t.ref %%.%u)%s ; Get a pointer to the object\n"
			"\t%%.%u = bitcast i8* %%.%u to %s* ; Convert pointer type\n"
			"\tcall fastcc void @.ref.commit.before(%%t.ref %%.%u)%s ; Enable writing to the pointer\n"
			"\t%%.%u = load %s* %%.%u%s ; Get the object\n"
			"\t; Destroy the object\n"
				"%s"
			"\tcall fastcc void @.ref.commit.after(%%t.ref %%.%u)%s ; Commit what we've written\n"
			"\tcall fastcc void @.ref.destroy.dynamic(%%t.ref %%.%u)%s ; Destroy the reference\n",
		
		GetNode()->GetSourceLocation().c_str(),
				Indent(GetNode()->GetExpression()->GetCodeGenerator<CodeGeneratorExpressionLlvm>()->Generate(context, d, reference), 2).c_str(),
			pointer_i8, reference, GetDebug(context, d).c_str(),
			pointer_type, pointer_i8, type.c_str(),
			reference, GetDebug(context, d).c_str(),
			object, type.c_str(), pointer_type, GetDebug(context, d).c_str(),
				Indent(reference_type->GetType()->GetCodeGenerator<CodeGeneratorTypeLlvm>()->GenerateDestroy(context, d, object), 2).c_str(),
			reference, GetDebug(context, d).c_str(),
			reference, GetDebug(context, d).c_str()
	);
}

std::string CodeGeneratorStatementDeallocLlvm::GenerateFnBegin(CompileContext &context, unsigned int d)
{
	return GetNode()->GetExpression()->GetCodeGenerator<CodeGeneratorExpressionLlvm>()->GenerateFnBegin(context, d);
}

std::string CodeGeneratorStatementDeallocLlvm::GenerateFnEnd(CompileContext &context, unsigned int d)
{
	return GetNode()->GetExpression()->GetCodeGenerator<CodeGeneratorExpressionLlvm>()->GenerateFnEnd(context, d);
}

std::string CodeGeneratorStatementDeallocLlvm::GenerateTopLevel(CompileContext &context, unsigned int d)
{
	return GetNode()->GetExpression()->GetCodeGenerator<CodeGeneratorExpressionLlvm>()->GenerateTopLevel(context, d);
}


