/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef CODEGEN_LLVM_REFERENCE_HPP
#define CODEGEN_LLVM_REFERENCE_HPP

#include "type.hpp"
#include "expression.hpp"

class CodeGeneratorTypeReferenceLlvm : public CodeGeneratorTypeLlvm
{
	public:
		CodeGeneratorTypeReferenceLlvm(AstNodeTypeReference *node);
		virtual CodeGeneratorTypeReferenceLlvm *Copy();
		
		
		virtual unsigned int GetSize(CompileContext &context);
		
		virtual std::string Generate(CompileContext &context, unsigned int d);
		virtual std::string GenerateDebug(CompileContext &context, unsigned int d);
		virtual std::string GenerateTopLevel(CompileContext &context, unsigned int d);
		virtual std::string GenerateCreate(CompileContext &context, unsigned int d, unsigned int value);
		virtual std::string GenerateDestroy(CompileContext &context, unsigned int d, unsigned int value);
		
		static std::string GenerateDebugGeneric(CompileContext &context, unsigned int toplevel);
		static std::string GenerateDebugTopLevelGeneric(CompileContext &context, unsigned int n, std::string type);
		
		AstType(AstNodeTypeReference);
};

class CodeGeneratorExpressionReferenceLlvm : public CodeGeneratorExpressionLlvm
{
	public:
		CodeGeneratorExpressionReferenceLlvm(AstNodeExpressionReference *node);
		virtual CodeGeneratorExpressionReferenceLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d, unsigned int n);
		
		AstType(AstNodeExpressionReference);
};

class CodeGeneratorExpressionLValueDereferenceLlvm : public CodeGeneratorExpressionLValueLlvm
{
	public:
		CodeGeneratorExpressionLValueDereferenceLlvm(AstNodeExpressionLValueDereference *node);
		virtual CodeGeneratorExpressionLValueDereferenceLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d, unsigned int n);
		virtual std::string GenerateStore(CompileContext &context, unsigned int d, unsigned int n);
		virtual std::string GenerateReference(CompileContext &context, unsigned int d, unsigned int n);
		
		AstType(AstNodeExpressionLValueDereference);
};

class CodeGeneratorExpressionAllocLlvm : public CodeGeneratorExpressionLlvm
{
	public:
		CodeGeneratorExpressionAllocLlvm(AstNodeExpressionAlloc *node);
		virtual CodeGeneratorExpressionAllocLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d, unsigned int n);
		virtual std::string GenerateTopLevel(CompileContext &context, unsigned int d);
		
		AstType(AstNodeExpressionAlloc);
};

class CodeGeneratorStatementDeallocLlvm : public CodeGeneratorStatementLlvm
{
	public:
		CodeGeneratorStatementDeallocLlvm(AstNodeStatementDealloc *node);
		virtual CodeGeneratorStatementDeallocLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d);
		
		virtual std::string GenerateFnBegin(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnEnd(CompileContext &context, unsigned int d);
		virtual std::string GenerateTopLevel(CompileContext &context, unsigned int d);
		
		AstType(AstNodeStatementDealloc);
};


#endif
