/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2012-2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef CODEGEN_LLVM_STATEMENT_HPP
#define CODEGEN_LLVM_STATEMENT_HPP

#include "codegen.hpp"


class CodeGeneratorStatementLlvm : public CodeGeneratorLlvm
{
	public:
		CodeGeneratorStatementLlvm(AstNodeStatement *node);
		
		virtual AstNodeStatement *GetNode() = 0;
		
		virtual std::string Generate(CompileContext &context, unsigned int d) = 0;
		virtual std::string GenerateFnBegin(CompileContext &context, unsigned int d) = 0;
		virtual std::string GenerateFnEnd(CompileContext &context, unsigned int d) = 0;
		virtual std::string GenerateTopLevel(CompileContext &context, unsigned int d) = 0; //Generates stuff that should be included in the top level (e.g debugging metadata, and strings)
};

class CodeGeneratorStatementReturnLlvm : public CodeGeneratorStatementLlvm
{
	public:
		CodeGeneratorStatementReturnLlvm(AstNodeStatementReturn *node);
		virtual CodeGeneratorStatementReturnLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnBegin(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnEnd(CompileContext &context, unsigned int d);
		virtual std::string GenerateTopLevel(CompileContext &context, unsigned int d);
		
		AstType(AstNodeStatementReturn);
};

class CodeGeneratorStatementVariableAssignmentLlvm : public CodeGeneratorStatementLlvm
{
	public:
		CodeGeneratorStatementVariableAssignmentLlvm(AstNodeStatementVariableAssignment *node);
		virtual CodeGeneratorStatementVariableAssignmentLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnBegin(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnEnd(CompileContext &context, unsigned int d);
		virtual std::string GenerateTopLevel(CompileContext &context, unsigned int d);
		
		AstType(AstNodeStatementVariableAssignment);
};

class CodeGeneratorStatementVariableDeclarationLlvm : public CodeGeneratorStatementLlvm
{
	public:
		CodeGeneratorStatementVariableDeclarationLlvm(AstNodeStatementVariableDeclaration *node);
		virtual CodeGeneratorStatementVariableDeclarationLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnBegin(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnEnd(CompileContext &context, unsigned int d);
		virtual std::string GenerateTopLevel(CompileContext &context, unsigned int d);
		
		AstType(AstNodeStatementVariableDeclaration);
};

class CodeGeneratorStatementVariableDeclarationLlvm_Highlighter : public CodeGeneratorStatementVariableDeclarationLlvm
{
	public:
		CodeGeneratorStatementVariableDeclarationLlvm_Highlighter(AstNodeStatementVariableDeclaration *node);
		virtual std::string GenerateFnBegin(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnEnd(CompileContext &context, unsigned int d);
};

class CodeGeneratorStatementExpressionLlvm : public CodeGeneratorStatementLlvm
{
	public:
		CodeGeneratorStatementExpressionLlvm(AstNodeStatementExpression *node);
		virtual CodeGeneratorStatementExpressionLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnBegin(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnEnd(CompileContext &context, unsigned int d);
		virtual std::string GenerateTopLevel(CompileContext &context, unsigned int d);
		
		AstType(AstNodeStatementExpression);
};

class CodeGeneratorStatementBlockLlvm : public CodeGeneratorStatementLlvm
{
	public:
		CodeGeneratorStatementBlockLlvm(AstNodeStatementBlock *node);
		virtual CodeGeneratorStatementBlockLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnBegin(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnEnd(CompileContext &context, unsigned int d);
		virtual std::string GenerateTopLevel(CompileContext &context, unsigned int d);
		
		AstType(AstNodeStatementBlock);
};

class CodeGeneratorStatementMarkedGroupLlvm : public CodeGeneratorStatementLlvm
{
	public:
		CodeGeneratorStatementMarkedGroupLlvm(AstNodeStatementMarkedGroup *node);
		virtual CodeGeneratorStatementMarkedGroupLlvm *Copy();
		virtual std::string Generate(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnBegin(CompileContext &context, unsigned int d);
		virtual std::string GenerateFnEnd(CompileContext &context, unsigned int d);
		virtual std::string GenerateTopLevel(CompileContext &context, unsigned int d);
		
		AstType(AstNodeStatementMarkedGroup);
};

#include "ctrlflow.hpp"

#endif
