/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../../ast/astnode.hpp"
#include "codegen.hpp"


CodeGeneratorOpencl::CodeGeneratorOpencl(AstNode *node) :
	CodeGenerator(node)
{
}

CodeGeneratorRootOpencl::CodeGeneratorRootOpencl(AstNodeRoot *node) :
	CodeGeneratorOpencl(node)
{
}

CodeGeneratorRootOpencl *CodeGeneratorRootOpencl::Copy()
{
	return new CodeGeneratorRootOpencl(GetNode());
}

std::string CodeGeneratorRootOpencl::Generate(CompileContext &context)
{
	std::string source;
	
	for (auto struct_declaration: GetNode()->GetStructDeclarations()){
		source += struct_declaration->GetCodeGenerator<CodeGeneratorStructDeclarationOpencl>()->Generate(context);
	}
	
	if (GetNode()->GetVariableDeclarations().size()){
		throw CodeGenerationException(CODEGEN_OPENCL, GetNode(), "Global variables not yet supported for OpenCL");
	}
	
	for (auto function_declaration: GetNode()->GetFunctionDeclarations()){
		if (!function_declaration->GetIsCompiler()){
			source += function_declaration->GetCodeGenerator<CodeGeneratorFunctionDeclarationOpencl>()->Generate(context);
		}
	}
	
	return source;
}

std::vector<uint8_t> CodeGeneratorRootOpencl::GenerateBinary(CompileContext &context)
{
	return std::vector<uint8_t>();
}

