/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../../ast/astnode.hpp"
#include "codegen.hpp"



CodeGeneratorStatementForOpencl::CodeGeneratorStatementForOpencl(AstNodeStatementFor *node) :
	CodeGeneratorStatementOpencl(node, true)
{
}

CodeGeneratorStatementForOpencl *CodeGeneratorStatementForOpencl::Copy()
{
	return new CodeGeneratorStatementForOpencl(GetNode());
}

std::string CodeGeneratorStatementForOpencl::Generate(CompileContext &context)
{
	std::string counter_name = GetNode()->GetVariableDeclaration()->GetIdentifier();
	std::string counter_type = GetNode()->GetVariableDeclaration()->GetType()->GetCodeGenerator<CodeGeneratorTypeOpencl>()->Generate(context);
	
	std::string counter_max_name = PrintString("__polycute_%u", GetUniqueId());
	std::string counter_max_type = GetNode()->GetExpression()->GetType()->GetCodeGenerator<CodeGeneratorTypeOpencl>()->Generate(context);
	
	return PrintString(
		"%s %s = %s; //Calculate the maximum expression\n"
		"for (%s %s = 0; %s < %s; %s++)%s\n",
		
		counter_max_type.c_str(), counter_max_name.c_str(), GetNode()->GetExpression()->GetCodeGenerator<CodeGeneratorExpressionOpencl>()->Generate(context).c_str(),
		counter_type.c_str(), counter_name.c_str(), counter_name.c_str(), counter_max_name.c_str(), counter_name.c_str(),
			GetNode()->GetStatement()->GetCodeGenerator<CodeGeneratorStatementOpencl>()->Generate(context).c_str()
	);
}



CodeGeneratorStatementWhileOpencl::CodeGeneratorStatementWhileOpencl(AstNodeStatementWhile *node) :
	CodeGeneratorStatementOpencl(node, true)
{
}

CodeGeneratorStatementWhileOpencl *CodeGeneratorStatementWhileOpencl::Copy()
{
	return new CodeGeneratorStatementWhileOpencl(GetNode());
}

std::string CodeGeneratorStatementWhileOpencl::Generate(CompileContext &context)
{
	return PrintString(
		"while (%s)%s\n",
		GetNode()->GetCondition()->GetCodeGenerator<CodeGeneratorExpressionOpencl>()->Generate(context).c_str(),
		GetNode()->GetStatement()->GetCodeGenerator<CodeGeneratorStatementOpencl>()->Generate(context).c_str()
	);
}



CodeGeneratorStatementIfOpencl::CodeGeneratorStatementIfOpencl(AstNodeStatementIf *node) :
	CodeGeneratorStatementOpencl(node, true)
{
}

CodeGeneratorStatementIfOpencl *CodeGeneratorStatementIfOpencl::Copy()
{
	return new CodeGeneratorStatementIfOpencl(GetNode());
}

std::string CodeGeneratorStatementIfOpencl::Generate(CompileContext &context)
{
	if (GetNode()->GetElseStatement()){
		return PrintString("if (%s) %s\nelse %s\n",
			GetNode()->GetCondition()->GetCodeGenerator<CodeGeneratorExpressionOpencl>()->Generate(context).c_str(),
			GetNode()->GetIfStatement()->GetCodeGenerator<CodeGeneratorStatementOpencl>()->Generate(context).c_str(),
			GetNode()->GetElseStatement()->GetCodeGenerator<CodeGeneratorStatementOpencl>()->Generate(context).c_str()
		);
	}
	else {
		return PrintString("if (%s) %s\n",
			GetNode()->GetCondition()->GetCodeGenerator<CodeGeneratorExpressionOpencl>()->Generate(context).c_str(),
			GetNode()->GetIfStatement()->GetCodeGenerator<CodeGeneratorStatementOpencl>()->Generate(context).c_str()
		);
	}
}

