/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../../ast/astnode.hpp"
#include "codegen.hpp"



CodeGeneratorExpressionOpencl::CodeGeneratorExpressionOpencl(AstNodeExpression *node) :
	CodeGeneratorOpencl(node)
{
}

CodeGeneratorExpressionLValueOpencl::CodeGeneratorExpressionLValueOpencl(AstNodeExpressionLValue *node) :
	CodeGeneratorExpressionOpencl(node)
{
}



CodeGeneratorExpressionBoolOpencl::CodeGeneratorExpressionBoolOpencl(AstNodeExpressionBool *node) :
	CodeGeneratorExpressionOpencl(node)
{
}

CodeGeneratorExpressionBoolOpencl *CodeGeneratorExpressionBoolOpencl::Copy()
{
	return new CodeGeneratorExpressionBoolOpencl(GetNode());
}

std::string CodeGeneratorExpressionBoolOpencl::Generate(CompileContext &context)
{
	return GetNode()->GetValue() ? "true" : "false";
}



CodeGeneratorExpressionIntOpencl::CodeGeneratorExpressionIntOpencl(AstNodeExpressionInt *node) :
	CodeGeneratorExpressionOpencl(node)
{
}

CodeGeneratorExpressionIntOpencl *CodeGeneratorExpressionIntOpencl::Copy()
{
	return new CodeGeneratorExpressionIntOpencl(GetNode());
}

std::string CodeGeneratorExpressionIntOpencl::Generate(CompileContext &context)
{
	return PrintString("%llu", GetNode()->GetValue());
}



CodeGeneratorExpressionFloatOpencl::CodeGeneratorExpressionFloatOpencl(AstNodeExpressionFloat *node) :
	CodeGeneratorExpressionOpencl(node)
{
}

CodeGeneratorExpressionFloatOpencl *CodeGeneratorExpressionFloatOpencl::Copy()
{
	return new CodeGeneratorExpressionFloatOpencl(GetNode());
}

std::string CodeGeneratorExpressionFloatOpencl::Generate(CompileContext &context)
{
	return PrintString("%f", GetNode()->GetValue());
}



CodeGeneratorExpressionLValueIdentifierOpencl::CodeGeneratorExpressionLValueIdentifierOpencl(AstNodeExpressionLValueIdentifier *node) :
	CodeGeneratorExpressionLValueOpencl(node)
{
}

CodeGeneratorExpressionLValueIdentifierOpencl *CodeGeneratorExpressionLValueIdentifierOpencl::Copy()
{
	return new CodeGeneratorExpressionLValueIdentifierOpencl(GetNode());
}

std::string CodeGeneratorExpressionLValueIdentifierOpencl::Generate(CompileContext &context)
{
	return GetNode()->GetIdentifier();
}



CodeGeneratorExpressionLValueVectorOpencl::CodeGeneratorExpressionLValueVectorOpencl(AstNodeExpressionLValueVector *node) :
	CodeGeneratorExpressionLValueOpencl(node)
{
}

CodeGeneratorExpressionLValueVectorOpencl *CodeGeneratorExpressionLValueVectorOpencl::Copy()
{
	return new CodeGeneratorExpressionLValueVectorOpencl(GetNode());
}

std::string CodeGeneratorExpressionLValueVectorOpencl::Generate(CompileContext &context)
{
	return PrintString("%s[%s]",
		GetNode()->GetLValue()->GetCodeGenerator<CodeGeneratorExpressionLValueOpencl>()->Generate(context).c_str(),
		GetNode()->GetExpression()->GetCodeGenerator<CodeGeneratorExpressionOpencl>()->Generate(context).c_str()
	);
}

