/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef CODEGEN_OPENCL_EXPRESSION_HPP
#define CODEGEN_OPENCL_EXPRESSION_HPP

#include "codegen.hpp"

class CodeGeneratorExpressionOpencl : public CodeGeneratorOpencl
{
	public:
		CodeGeneratorExpressionOpencl(AstNodeExpression *node);
		
		virtual AstNodeExpression *GetNode() = 0;
		
		/*
		 * Puts the value of the expression into %.n
		 */
		virtual std::string Generate(CompileContext &context) = 0;
		
	private:
};

class CodeGeneratorExpressionLValueOpencl : public CodeGeneratorExpressionOpencl
{
	public:
		CodeGeneratorExpressionLValueOpencl(AstNodeExpressionLValue *node);
		
		virtual AstNodeExpressionLValue *GetNode() = 0;
		
	private:
};

class CodeGeneratorExpressionBoolOpencl : public CodeGeneratorExpressionOpencl
{
	public:
		CodeGeneratorExpressionBoolOpencl(AstNodeExpressionBool *node);
		virtual CodeGeneratorExpressionBoolOpencl *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeExpressionBool);
};

class CodeGeneratorExpressionIntOpencl : public CodeGeneratorExpressionOpencl
{
	public:
		CodeGeneratorExpressionIntOpencl(AstNodeExpressionInt *node);
		virtual CodeGeneratorExpressionIntOpencl *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeExpressionInt);
};

class CodeGeneratorExpressionFloatOpencl : public CodeGeneratorExpressionOpencl
{
	public:
		CodeGeneratorExpressionFloatOpencl(AstNodeExpressionFloat *node);
		virtual CodeGeneratorExpressionFloatOpencl *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeExpressionFloat);
};

class CodeGeneratorExpressionLValueIdentifierOpencl : public CodeGeneratorExpressionLValueOpencl
{
	public:
		CodeGeneratorExpressionLValueIdentifierOpencl(AstNodeExpressionLValueIdentifier *node);
		virtual CodeGeneratorExpressionLValueIdentifierOpencl *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeExpressionLValueIdentifier);
};

class CodeGeneratorExpressionLValueVectorOpencl : public CodeGeneratorExpressionLValueOpencl
{
	public:
		CodeGeneratorExpressionLValueVectorOpencl(AstNodeExpressionLValueVector *node);
		virtual CodeGeneratorExpressionLValueVectorOpencl *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeExpressionLValueVector);
};


#include "struct.hpp"
#include "op.hpp"


#endif
