/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../../ast/astnode.hpp"
#include "codegen.hpp"



CodeGeneratorFunctionDeclarationOpencl::CodeGeneratorFunctionDeclarationOpencl(AstNodeFunctionDeclaration *node) :
	CodeGeneratorOpencl(node)
{
}

CodeGeneratorFunctionDeclarationOpencl *CodeGeneratorFunctionDeclarationOpencl::Copy()
{
	return new CodeGeneratorFunctionDeclarationOpencl(GetNode());
}

std::string CodeGeneratorFunctionDeclarationOpencl::Generate(CompileContext &context)
{
	std::string arguments;
	for (auto argument: GetNode()->GetArguments()){
		if (arguments.size()){
			arguments += ", ";
		}
		arguments += argument->GetCodeGenerator<CodeGeneratorStatementVariableDeclarationOpencl>()->GenerateNonStatement(context);
	}
	
	if (!GetNode()->GetStatement()){
		throw CodeGenerationException(CODEGEN_OPENCL, GetNode(), "Imported functions not supported for OpenCL");
	}
	
	return PrintString(
		"%s %s (%s){%s}\n",
		GetNode()->GetReturnType()->GetCodeGenerator<CodeGeneratorTypeOpencl>()->Generate(context).c_str(),
		GetNode()->GetIdentifier().c_str(),
		arguments.c_str(),
		Indent(GetNode()->GetStatement()->GetCodeGenerator<CodeGeneratorStatementOpencl>()->Generate(context), 1).c_str()
	);
	
	//TODO: Implement kernels
}



CodeGeneratorExpressionFunctionCallOpencl::CodeGeneratorExpressionFunctionCallOpencl(AstNodeFunctionCall *node) :
	CodeGeneratorExpressionOpencl(node)
{
}

CodeGeneratorExpressionFunctionCallOpencl *CodeGeneratorExpressionFunctionCallOpencl::Copy()
{
	return new CodeGeneratorExpressionFunctionCallOpencl(GetNode());
}

std::string CodeGeneratorExpressionFunctionCallOpencl::Generate(CompileContext &context)
{
	std::string arguments;
	for (auto argument: GetNode()->GetExpressions()){
		if (arguments.size()){
			arguments += ", ";
		}
		arguments += argument->GetCodeGenerator<CodeGeneratorExpressionOpencl>()->Generate(context);
	}
	
	return PrintString(
		"%s (%s)",
		GetNode()->GetLValue()->GetCodeGenerator<CodeGeneratorExpressionLValueOpencl>()->Generate(context).c_str(),
		arguments.c_str()
	);
}

