/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../../ast/astnode.hpp"
#include "codegen.hpp"




CodeGeneratorStatementOpencl::CodeGeneratorStatementOpencl(AstNodeStatement *node, bool multiline) :
	CodeGeneratorOpencl(node), m_multiline(multiline)
{
}

bool CodeGeneratorStatementOpencl::GetMultiline()
{
	return m_multiline;
}

CodeGeneratorStatementReturnOpencl::CodeGeneratorStatementReturnOpencl(AstNodeStatementReturn *node) :
	CodeGeneratorStatementOpencl(node, false)
{
}

CodeGeneratorStatementReturnOpencl *CodeGeneratorStatementReturnOpencl::Copy()
{
	return new CodeGeneratorStatementReturnOpencl(GetNode());
}

std::string CodeGeneratorStatementReturnOpencl::Generate(CompileContext &context)
{
	return PrintString("return%s%s;", GetNode()->GetExpression() ? " " : "",
		GetNode()->GetExpression() ? GetNode()->GetExpression()->GetCodeGenerator<CodeGeneratorExpressionOpencl>()->Generate(context).c_str() : "");
}



CodeGeneratorStatementVariableAssignmentOpencl::CodeGeneratorStatementVariableAssignmentOpencl(AstNodeStatementVariableAssignment *node) :
	CodeGeneratorStatementOpencl(node, false)
{
}

CodeGeneratorStatementVariableAssignmentOpencl *CodeGeneratorStatementVariableAssignmentOpencl::Copy()
{
	return new CodeGeneratorStatementVariableAssignmentOpencl(GetNode());
}

std::string CodeGeneratorStatementVariableAssignmentOpencl::Generate(CompileContext &context)
{
	return PrintString("%s = %s;",
		GetNode()->GetLValue()->GetCodeGenerator<CodeGeneratorExpressionOpencl>()->Generate(context).c_str(),
		GetNode()->GetExpression()->GetCodeGenerator<CodeGeneratorExpressionOpencl>()->Generate(context).c_str());
}



CodeGeneratorStatementVariableDeclarationOpencl::CodeGeneratorStatementVariableDeclarationOpencl(AstNodeStatementVariableDeclaration *node) :
	CodeGeneratorStatementOpencl(node, false)
{
}

CodeGeneratorStatementVariableDeclarationOpencl *CodeGeneratorStatementVariableDeclarationOpencl::Copy()
{
	return new CodeGeneratorStatementVariableDeclarationOpencl(GetNode());
}

std::string CodeGeneratorStatementVariableDeclarationOpencl::Generate(CompileContext &context)
{
	return PrintString("%s %s;",
		GetNode()->GetType()->GetCodeGenerator<CodeGeneratorTypeOpencl>()->Generate(context).c_str(),
		GetNode()->GetIdentifier().c_str());
}

std::string CodeGeneratorStatementVariableDeclarationOpencl::GenerateNonStatement(CompileContext &context)
{
	return PrintString("%s %s",
		GetNode()->GetType()->GetCodeGenerator<CodeGeneratorTypeOpencl>()->Generate(context).c_str(),
		GetNode()->GetIdentifier().c_str());
}



CodeGeneratorStatementExpressionOpencl::CodeGeneratorStatementExpressionOpencl(AstNodeStatementExpression *node) :
	CodeGeneratorStatementOpencl(node, false)
{
}

CodeGeneratorStatementExpressionOpencl *CodeGeneratorStatementExpressionOpencl::Copy()
{
	return new CodeGeneratorStatementExpressionOpencl(GetNode());
}

std::string CodeGeneratorStatementExpressionOpencl::Generate(CompileContext &context)
{
	return GetNode()->GetExpression()->GetCodeGenerator<CodeGeneratorExpressionOpencl>()->Generate(context) + ";";
}



CodeGeneratorStatementBlockOpencl::CodeGeneratorStatementBlockOpencl(AstNodeStatementBlock *node) :
	CodeGeneratorStatementOpencl(node, true)
{
}

CodeGeneratorStatementBlockOpencl *CodeGeneratorStatementBlockOpencl::Copy()
{
	return new CodeGeneratorStatementBlockOpencl(GetNode());
}

std::string CodeGeneratorStatementBlockOpencl::Generate(CompileContext &context)
{
	std::string statements;
	for (AstNodeStatement *statement: GetNode()->GetStatements()){
		statements += Indent(statement->GetCodeGenerator<CodeGeneratorStatementOpencl>()->Generate(context) + "\n", 1);
	}
	
	return PrintString("{\n%s}", statements.c_str());
}


