/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef CODEGEN_OPENCL_STATEMENT_HPP
#define CODEGEN_OPENCL_STATEMENT_HPP

#include "codegen.hpp"



class CodeGeneratorStatementOpencl : public CodeGeneratorOpencl
{
	public:
		CodeGeneratorStatementOpencl(AstNodeStatement *node, bool multiline);
		virtual AstNodeStatement *GetNode() = 0;
		virtual std::string Generate(CompileContext &context) = 0;
		
		bool GetMultiline();
		
	private:
		bool m_multiline;
};

class CodeGeneratorStatementReturnOpencl : public CodeGeneratorStatementOpencl
{
	public:
		CodeGeneratorStatementReturnOpencl(AstNodeStatementReturn *node);
		virtual CodeGeneratorStatementReturnOpencl *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeStatementReturn);
};

class CodeGeneratorStatementVariableAssignmentOpencl : public CodeGeneratorStatementOpencl
{
	public:
		CodeGeneratorStatementVariableAssignmentOpencl(AstNodeStatementVariableAssignment *node);
		virtual CodeGeneratorStatementVariableAssignmentOpencl *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeStatementVariableAssignment);
};

class CodeGeneratorStatementVariableDeclarationOpencl : public CodeGeneratorStatementOpencl
{
	public:
		CodeGeneratorStatementVariableDeclarationOpencl(AstNodeStatementVariableDeclaration *node);
		virtual CodeGeneratorStatementVariableDeclarationOpencl *Copy();
		virtual std::string Generate(CompileContext &context);
		std::string GenerateNonStatement(CompileContext &context);
		
		AstType(AstNodeStatementVariableDeclaration);
};

class CodeGeneratorStatementExpressionOpencl : public CodeGeneratorStatementOpencl
{
	public:
		CodeGeneratorStatementExpressionOpencl(AstNodeStatementExpression *node);
		virtual CodeGeneratorStatementExpressionOpencl *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeStatementExpression);
};

class CodeGeneratorStatementBlockOpencl : public CodeGeneratorStatementOpencl
{
	public:
		CodeGeneratorStatementBlockOpencl(AstNodeStatementBlock *node);
		virtual CodeGeneratorStatementBlockOpencl *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeStatementBlock);
};

#include "ctrlflow.hpp"

#endif
