/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../../ast/astnode.hpp"
#include "codegen.hpp"



CodeGeneratorStructDeclarationOpencl::CodeGeneratorStructDeclarationOpencl(AstNodeStructDeclaration *node) :
	CodeGeneratorOpencl(node)
{
}

CodeGeneratorStructDeclarationOpencl *CodeGeneratorStructDeclarationOpencl::Copy()
{
	return new CodeGeneratorStructDeclarationOpencl(GetNode());
}

std::string CodeGeneratorStructDeclarationOpencl::Generate(CompileContext &context)
{
	std::string declarations;
	for (auto item: GetNode()->GetItems()){
		declarations += PrintString("\t%s %s;\n",
			item->GetType()->GetCodeGenerator<CodeGeneratorTypeOpencl>()->Generate(context).c_str(),
			item->GetIdentifier().c_str()
		);
	}
	
	return PrintString("struct %s\n{\n%s}\n\n", GetNode()->GetIdentifier().c_str(), declarations.c_str());
}



CodeGeneratorTypeStructOpencl::CodeGeneratorTypeStructOpencl(AstNodeTypeStruct *node) :
	CodeGeneratorTypeOpencl(node)
{
}

CodeGeneratorTypeStructOpencl *CodeGeneratorTypeStructOpencl::Copy()
{
	return new CodeGeneratorTypeStructOpencl(GetNode());
}

std::string CodeGeneratorTypeStructOpencl::Generate(CompileContext &context)
{
	return PrintString("struct %s", GetNode()->GetIdentifier().c_str());
}



CodeGeneratorExpressionLValueStructElementOpencl::CodeGeneratorExpressionLValueStructElementOpencl(AstNodeExpressionLValueStructElement *node) :
	CodeGeneratorExpressionLValueOpencl(node)
{
}

CodeGeneratorExpressionLValueStructElementOpencl *CodeGeneratorExpressionLValueStructElementOpencl::Copy()
{
	return new CodeGeneratorExpressionLValueStructElementOpencl(GetNode());
}

std::string CodeGeneratorExpressionLValueStructElementOpencl::Generate(CompileContext &context)
{
	return PrintString("%s.%s", GetNode()->GetLValue()->GetCodeGenerator<CodeGeneratorExpressionOpencl>()->Generate(context).c_str(), GetNode()->GetIdentifier().c_str());
}


 
