/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../../ast/astnode.hpp"
#include "codegen.hpp"



CodeGeneratorTypeOpencl::CodeGeneratorTypeOpencl(AstNodeType *node) :
	CodeGeneratorOpencl(node)
{
}



CodeGeneratorTypeVoidOpencl::CodeGeneratorTypeVoidOpencl(AstNodeTypeVoid *node) :
	CodeGeneratorTypeOpencl(node)
{
}

CodeGeneratorTypeVoidOpencl *CodeGeneratorTypeVoidOpencl::Copy()
{
	return new CodeGeneratorTypeVoidOpencl(GetNode());
}

std::string CodeGeneratorTypeVoidOpencl::Generate(CompileContext &context)
{
	return "void";
}



CodeGeneratorTypeBoolOpencl::CodeGeneratorTypeBoolOpencl(AstNodeTypeBool *node) :
	CodeGeneratorTypeOpencl(node)
{
}

CodeGeneratorTypeBoolOpencl *CodeGeneratorTypeBoolOpencl::Copy()
{
	return new CodeGeneratorTypeBoolOpencl(GetNode());
}

std::string CodeGeneratorTypeBoolOpencl::Generate(CompileContext &context)
{
	return "bool";
}



CodeGeneratorTypeIntOpencl::CodeGeneratorTypeIntOpencl(AstNodeTypeInt *node) :
	CodeGeneratorTypeOpencl(node)
{
}

CodeGeneratorTypeIntOpencl *CodeGeneratorTypeIntOpencl::Copy()
{
	return new CodeGeneratorTypeIntOpencl(GetNode());
}

std::string CodeGeneratorTypeIntOpencl::Generate(CompileContext &context)
{
	unsigned int width = GetNode()->GetWidth() ? GetNode()->GetWidth() : context.GetComputeSpec().GetCpu().GetSizeT();
	switch (width)
	{
		case  8: return "unsigned char";
		case 16: return "unsigned short";
		case 32: return "unsigned int";
		case 64: return "unsigned long";
		default: throw CodeGenerationException(CODEGEN_OPENCL, GetNode(), "%u bit floating point not supported for OpenCL", GetNode()->GetWidth());
	}
}



CodeGeneratorTypeFloatOpencl::CodeGeneratorTypeFloatOpencl(AstNodeTypeFloat *node) :
	CodeGeneratorTypeOpencl(node)
{
}

CodeGeneratorTypeFloatOpencl *CodeGeneratorTypeFloatOpencl::Copy()
{
	return new CodeGeneratorTypeFloatOpencl(GetNode());
}

std::string CodeGeneratorTypeFloatOpencl::Generate(CompileContext &context)
{
	switch (GetNode()->GetWidth())
	{
		case 16: return "half";
		case 32: return "float";
		case 64: return "double";
		default: throw CodeGenerationException(CODEGEN_OPENCL, GetNode(), "%u bit floating point not supported for OpenCL", GetNode()->GetWidth());
	}
}



CodeGeneratorTypePointerOpencl::CodeGeneratorTypePointerOpencl(AstNodeTypePointer *node) :
	CodeGeneratorTypeOpencl(node)
{
}

CodeGeneratorTypePointerOpencl *CodeGeneratorTypePointerOpencl::Copy()
{
	return new CodeGeneratorTypePointerOpencl(GetNode());
}

std::string CodeGeneratorTypePointerOpencl::Generate(CompileContext &context)
{
	return PrintString("%s *", GetNode()->GetType()->GetCodeGenerator<CodeGeneratorTypeOpencl>()->Generate(context).c_str());
}

