/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef CODEGEN_OPENCL_TYPE_HPP
#define CODEGEN_OPENCL_TYPE_HPP

#include "codegen.hpp"


class CodeGeneratorTypeOpencl : public CodeGeneratorOpencl
{
	public:
		CodeGeneratorTypeOpencl(AstNodeType *node);
		
		virtual AstNodeType *GetNode() = 0;
		
		virtual std::string Generate(CompileContext &context) = 0;
		
	private:
};

class CodeGeneratorTypeVoidOpencl : public CodeGeneratorTypeOpencl
{
	public:
		CodeGeneratorTypeVoidOpencl(AstNodeTypeVoid *node);
		virtual CodeGeneratorTypeVoidOpencl *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeTypeVoid);
};

class CodeGeneratorTypeBoolOpencl : public CodeGeneratorTypeOpencl
{
	public:
		CodeGeneratorTypeBoolOpencl(AstNodeTypeBool *node);
		virtual CodeGeneratorTypeBoolOpencl *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeTypeBool);
};

class CodeGeneratorTypeIntOpencl : public CodeGeneratorTypeOpencl
{
	public:
		CodeGeneratorTypeIntOpencl(AstNodeTypeInt *node);
		virtual CodeGeneratorTypeIntOpencl *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeTypeInt);
};

class CodeGeneratorTypeFloatOpencl : public CodeGeneratorTypeOpencl
{
	public:
		CodeGeneratorTypeFloatOpencl(AstNodeTypeFloat *node);
		virtual CodeGeneratorTypeFloatOpencl *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeTypeFloat);
};

class CodeGeneratorTypePointerOpencl : public CodeGeneratorTypeOpencl
{
	public:
		CodeGeneratorTypePointerOpencl(AstNodeTypePointer *node);
		virtual CodeGeneratorTypePointerOpencl *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeTypePointer);
};

//I think I've decided that struct will be a simple structure class (with a packed variant (packed keyword?))
//class will be more complicated with the ability to have associated metadata
//a sizeof like thing would probably be useful for use with struct

#endif
