/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../../ast/astnode.hpp"
#include "codegen.hpp"



CodeGeneratorPseudopc::CodeGeneratorPseudopc(AstNode *node) :
	CodeGenerator(node)
{
}


CodeGeneratorRootPseudopc::CodeGeneratorRootPseudopc(AstNodeRoot *node) :
	CodeGeneratorPseudopc(node)
{
}

CodeGeneratorRootPseudopc *CodeGeneratorRootPseudopc::Copy()
{
	return new CodeGeneratorRootPseudopc(GetNode());
}

std::string CodeGeneratorRootPseudopc::Generate(CompileContext &context)
{
	std::string source;
	
	for (auto struct_declaration: GetNode()->GetStructDeclarations()){
		source += struct_declaration->GetCodeGenerator<CodeGeneratorStructDeclarationPseudopc>()->Generate(context);
	}
	
	for (auto variable_declaration: GetNode()->GetVariableDeclarations()){
		source += variable_declaration->GetCodeGenerator<CodeGeneratorStatementVariableDeclarationPseudopc>()->Generate(context);
	}
	
	for (auto function_declaration: GetNode()->GetFunctionDeclarations()){
		if (!function_declaration->GetIsCompiler()){
			source += function_declaration->GetCodeGenerator<CodeGeneratorFunctionDeclarationPseudopc>()->Generate(context);
		}
	}
	
	return source;
}
