/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../../ast/astnode.hpp"
#include "codegen.hpp"



CodeGeneratorStatementForPseudopc::CodeGeneratorStatementForPseudopc(AstNodeStatementFor *node) :
	CodeGeneratorStatementPseudopc(node, true)
{
}

CodeGeneratorStatementForPseudopc *CodeGeneratorStatementForPseudopc::Copy()
{
	return new CodeGeneratorStatementForPseudopc(GetNode());
}

std::string CodeGeneratorStatementForPseudopc::Generate(CompileContext &context)
{
	return PrintString("for (%s to %s) %s\n",
		GetNode()->GetVariableDeclaration()->GetCodeGenerator<CodeGeneratorStatementVariableDeclarationPseudopc>()->GenerateNonStatement(context).c_str(),
		GetNode()->GetExpression()->GetCodeGenerator<CodeGeneratorExpressionPseudopc>()->Generate(context).c_str(),
		GetNode()->GetStatement()->GetCodeGenerator<CodeGeneratorStatementPseudopc>()->Generate(context).c_str()
	);
}



CodeGeneratorStatementWhilePseudopc::CodeGeneratorStatementWhilePseudopc(AstNodeStatementWhile *node) :
	CodeGeneratorStatementPseudopc(node, true)
{
}

CodeGeneratorStatementWhilePseudopc *CodeGeneratorStatementWhilePseudopc::Copy()
{
	return new CodeGeneratorStatementWhilePseudopc(GetNode());
}

std::string CodeGeneratorStatementWhilePseudopc::Generate(CompileContext &context)
{
	return PrintString("while (%s) %s\n",
		GetNode()->GetCondition()->GetCodeGenerator<CodeGeneratorExpressionPseudopc>()->Generate(context).c_str(),
		GetNode()->GetStatement()->GetCodeGenerator<CodeGeneratorStatementPseudopc>()->Generate(context).c_str()
	);
}



CodeGeneratorStatementIfPseudopc::CodeGeneratorStatementIfPseudopc(AstNodeStatementIf *node) :
	CodeGeneratorStatementPseudopc(node, true)
{
}

CodeGeneratorStatementIfPseudopc *CodeGeneratorStatementIfPseudopc::Copy()
{
	return new CodeGeneratorStatementIfPseudopc(GetNode());
}

std::string CodeGeneratorStatementIfPseudopc::Generate(CompileContext &context)
{
	if (GetNode()->GetElseStatement()){
		return PrintString("if (%s) %s\nelse %s\n",
			GetNode()->GetCondition()->GetCodeGenerator<CodeGeneratorExpressionPseudopc>()->Generate(context).c_str(),
			GetNode()->GetIfStatement()->GetCodeGenerator<CodeGeneratorStatementPseudopc>()->Generate(context).c_str(),
			GetNode()->GetElseStatement()->GetCodeGenerator<CodeGeneratorStatementPseudopc>()->Generate(context).c_str()
		);
	}
	else {
		return PrintString("if (%s) %s\n",
			GetNode()->GetCondition()->GetCodeGenerator<CodeGeneratorExpressionPseudopc>()->Generate(context).c_str(),
			GetNode()->GetIfStatement()->GetCodeGenerator<CodeGeneratorStatementPseudopc>()->Generate(context).c_str()
		);
	}
}

