/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../../ast/astnode.hpp"
#include "codegen.hpp"


CodeGeneratorTypeDynamicVectorPseudopc::CodeGeneratorTypeDynamicVectorPseudopc(AstNodeTypeDynamicVector *node) :
	CodeGeneratorTypePseudopc(node)
{
}

CodeGeneratorTypeDynamicVectorPseudopc *CodeGeneratorTypeDynamicVectorPseudopc::Copy()
{
	return new CodeGeneratorTypeDynamicVectorPseudopc(GetNode());
}

std::string CodeGeneratorTypeDynamicVectorPseudopc::Generate(CompileContext &context)
{
	return PrintString("%s []", GetNode()->GetType()->GetCodeGenerator<CodeGeneratorTypePseudopc>()->Generate(context).c_str());
}



CodeGeneratorExpressionDynamicVectorPseudopc::CodeGeneratorExpressionDynamicVectorPseudopc(AstNodeExpressionDynamicVector *node) :
	CodeGeneratorExpressionPseudopc(node)
{
}

CodeGeneratorExpressionDynamicVectorPseudopc *CodeGeneratorExpressionDynamicVectorPseudopc::Copy()
{
	return new CodeGeneratorExpressionDynamicVectorPseudopc(GetNode());
}

std::string CodeGeneratorExpressionDynamicVectorPseudopc::Generate(CompileContext &context)
{
	return PrintString("%s [%s]",
		tracked_cast<AstNodeTypeDynamicVector>(GetNode()->GetType())->GetType()->GetCodeGenerator<CodeGeneratorTypePseudopc>()->Generate(context).c_str(),
		GetNode()->GetExpression()->GetCodeGenerator<CodeGeneratorExpressionPseudopc>()->Generate(context).c_str()
	);
}



CodeGeneratorDynamicVectorOperationPseudopc::CodeGeneratorDynamicVectorOperationPseudopc(AstNodeStatementDynamicVectorOperation *node, std::string op) :
	CodeGeneratorStatementPseudopc(node, false), m_op(op)
{
}

CodeGeneratorDynamicVectorOperationPseudopc *CodeGeneratorDynamicVectorOperationPseudopc::Copy()
{
	return new CodeGeneratorDynamicVectorOperationPseudopc(GetNode(), m_op);
}

std::string CodeGeneratorDynamicVectorOperationPseudopc::Generate(CompileContext &context)
{
	return PrintString("DynamicVectorOperation(%s, %s);", m_op.c_str(),
		GetNode()->GetExpression()->GetCodeGenerator<CodeGeneratorExpressionPseudopc>()->Generate(context).c_str()
	);
}


CodeGeneratorDynamicVectorGetPointerPseudopc::CodeGeneratorDynamicVectorGetPointerPseudopc(AstNodeDynamicVectorGetPointer *node) :
	CodeGeneratorExpressionPseudopc(node)
{
}

CodeGeneratorDynamicVectorGetPointerPseudopc *CodeGeneratorDynamicVectorGetPointerPseudopc::Copy()
{
	return new CodeGeneratorDynamicVectorGetPointerPseudopc(GetNode());
}

std::string CodeGeneratorDynamicVectorGetPointerPseudopc::Generate(CompileContext &context)
{
	return PrintString("DynamicVectorGetPointer(%s)", GetNode()->GetLValue()->GetCodeGenerator<CodeGeneratorExpressionPseudopc>()->Generate(context).c_str());
}

