/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../../ast/astnode.hpp"
#include "codegen.hpp"



CodeGeneratorExpressionPseudopc::CodeGeneratorExpressionPseudopc(AstNodeExpression *node) :
	CodeGeneratorPseudopc(node)
{
}



CodeGeneratorExpressionLValuePseudopc::CodeGeneratorExpressionLValuePseudopc(AstNodeExpressionLValue *node) :
	CodeGeneratorExpressionPseudopc(node)
{
}



CodeGeneratorExpressionVoidPseudopc::CodeGeneratorExpressionVoidPseudopc(AstNodeExpressionVoid *node) :
	CodeGeneratorExpressionPseudopc(node)
{
}

CodeGeneratorExpressionVoidPseudopc *CodeGeneratorExpressionVoidPseudopc::Copy()
{
	return new CodeGeneratorExpressionVoidPseudopc(GetNode());
}

std::string CodeGeneratorExpressionVoidPseudopc::Generate(CompileContext &context)
{
	return "void";
}



CodeGeneratorExpressionBoolPseudopc::CodeGeneratorExpressionBoolPseudopc(AstNodeExpressionBool *node) :
	CodeGeneratorExpressionPseudopc(node)
{
}

CodeGeneratorExpressionBoolPseudopc *CodeGeneratorExpressionBoolPseudopc::Copy()
{
	return new CodeGeneratorExpressionBoolPseudopc(GetNode());
}

std::string CodeGeneratorExpressionBoolPseudopc::Generate(CompileContext &context)
{
	return PrintString("%s", GetNode()->GetValue() ? "true" : "false");
}



CodeGeneratorExpressionIntPseudopc::CodeGeneratorExpressionIntPseudopc(AstNodeExpressionInt *node) :
	CodeGeneratorExpressionPseudopc(node)
{
}

CodeGeneratorExpressionIntPseudopc *CodeGeneratorExpressionIntPseudopc::Copy()
{
	return new CodeGeneratorExpressionIntPseudopc(GetNode());
}

std::string CodeGeneratorExpressionIntPseudopc::Generate(CompileContext &context)
{
	return PrintString("%u", (unsigned int)GetNode()->GetValue());
}



CodeGeneratorExpressionFloatPseudopc::CodeGeneratorExpressionFloatPseudopc(AstNodeExpressionFloat *node) :
	CodeGeneratorExpressionPseudopc(node)
{
}

CodeGeneratorExpressionFloatPseudopc *CodeGeneratorExpressionFloatPseudopc::Copy()
{
	return new CodeGeneratorExpressionFloatPseudopc(GetNode());
}

std::string CodeGeneratorExpressionFloatPseudopc::Generate(CompileContext &context)
{
	return PrintString("%f", GetNode()->GetValue());
}



CodeGeneratorExpressionLengthPseudopc::CodeGeneratorExpressionLengthPseudopc(AstNodeExpressionLength *node) :
	CodeGeneratorExpressionPseudopc(node)
{
}

CodeGeneratorExpressionLengthPseudopc *CodeGeneratorExpressionLengthPseudopc::Copy()
{
	return new CodeGeneratorExpressionLengthPseudopc(GetNode());
}

std::string CodeGeneratorExpressionLengthPseudopc::Generate(CompileContext &context)
{
	return PrintString("length(%s)", GetNode()->GetExpression()->GetCodeGenerator<CodeGeneratorExpressionPseudopc>()->Generate(context).c_str());
}



CodeGeneratorExpressionBlockPseudopc::CodeGeneratorExpressionBlockPseudopc(AstNodeExpressionBlock *node) :
	CodeGeneratorExpressionPseudopc(node)
{
}

CodeGeneratorExpressionBlockPseudopc *CodeGeneratorExpressionBlockPseudopc::Copy()
{
	return new CodeGeneratorExpressionBlockPseudopc(GetNode());
}

std::string CodeGeneratorExpressionBlockPseudopc::Generate(CompileContext &context)
{
	std::string statements;
	for (AstNodeStatement *statement: GetNode()->GetStatements()){
		const char *ending = dynamic_cast<AstNodeStatementBlock *>(statement) ? "\n" : ";\n";
		statements += Indent(statement->GetCodeGenerator<CodeGeneratorStatementPseudopc>()->Generate(context) + ending, 1);
	}
	
	return PrintString(
		"ExpressionBlock(%s){\n%s}\n",
		GetNode()->GetExpression()->GetCodeGenerator<CodeGeneratorExpressionPseudopc>()->Generate(context).c_str(),
		statements.c_str()
	);
}



CodeGeneratorExpressionLValueIdentifierPseudopc::CodeGeneratorExpressionLValueIdentifierPseudopc(AstNodeExpressionLValueIdentifier *node) :
	CodeGeneratorExpressionLValuePseudopc(node)
{
}

CodeGeneratorExpressionLValueIdentifierPseudopc *CodeGeneratorExpressionLValueIdentifierPseudopc::Copy()
{
	return new CodeGeneratorExpressionLValueIdentifierPseudopc(GetNode());
}

std::string CodeGeneratorExpressionLValueIdentifierPseudopc::Generate(CompileContext &context)
{
	return GetNode()->GetIdentifier();
}



CodeGeneratorExpressionLValueVectorPseudopc::CodeGeneratorExpressionLValueVectorPseudopc(AstNodeExpressionLValueVector *node) :
	CodeGeneratorExpressionLValuePseudopc(node)
{
}

CodeGeneratorExpressionLValueVectorPseudopc *CodeGeneratorExpressionLValueVectorPseudopc::Copy()
{
	return new CodeGeneratorExpressionLValueVectorPseudopc(GetNode());
}

std::string CodeGeneratorExpressionLValueVectorPseudopc::Generate(CompileContext &context)
{
	return PrintString("%s[%s]",
		GetNode()->GetLValue()->GetCodeGenerator<CodeGeneratorExpressionPseudopc>()->Generate(context).c_str(),
		GetNode()->GetExpression()->GetCodeGenerator<CodeGeneratorExpressionPseudopc>()->Generate(context).c_str()
	);
}


