/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef CODEGEN_PSEUDOPC_EXPRESSION_HPP
#define CODEGEN_PSEUDOPC_EXPRESSION_HPP

#include "codegen.hpp"

class CodeGeneratorExpressionPseudopc : public CodeGeneratorPseudopc
{
	public:
		CodeGeneratorExpressionPseudopc(AstNodeExpression *node);
		
		virtual AstNodeExpression *GetNode() = 0;
		
		/*
		 * Puts the value of the expression into %.n
		 */
		virtual std::string Generate(CompileContext &context) = 0;
		
	private:
};

class CodeGeneratorExpressionLValuePseudopc : public CodeGeneratorExpressionPseudopc
{
	public:
		CodeGeneratorExpressionLValuePseudopc(AstNodeExpressionLValue *node);
		
		virtual AstNodeExpressionLValue *GetNode() = 0;
		
	private:
};


class CodeGeneratorExpressionVoidPseudopc : public CodeGeneratorExpressionPseudopc
{
	public:
		CodeGeneratorExpressionVoidPseudopc(AstNodeExpressionVoid *node);
		virtual CodeGeneratorExpressionVoidPseudopc *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeExpressionVoid);
};
class CodeGeneratorExpressionBoolPseudopc : public CodeGeneratorExpressionPseudopc
{
	public:
		CodeGeneratorExpressionBoolPseudopc(AstNodeExpressionBool *node);
		virtual CodeGeneratorExpressionBoolPseudopc *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeExpressionBool);
};

class CodeGeneratorExpressionIntPseudopc : public CodeGeneratorExpressionPseudopc
{
	public:
		CodeGeneratorExpressionIntPseudopc(AstNodeExpressionInt *node);
		virtual CodeGeneratorExpressionIntPseudopc *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeExpressionInt);
};

class CodeGeneratorExpressionFloatPseudopc : public CodeGeneratorExpressionPseudopc
{
	public:
		CodeGeneratorExpressionFloatPseudopc(AstNodeExpressionFloat *node);
		virtual CodeGeneratorExpressionFloatPseudopc *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeExpressionFloat);
};

class CodeGeneratorExpressionLengthPseudopc : public CodeGeneratorExpressionPseudopc
{
	public:
		CodeGeneratorExpressionLengthPseudopc(AstNodeExpressionLength *node);
		virtual CodeGeneratorExpressionLengthPseudopc *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeExpressionLength);
};

class CodeGeneratorExpressionBlockPseudopc : public CodeGeneratorExpressionPseudopc
{
	public:
		CodeGeneratorExpressionBlockPseudopc(AstNodeExpressionBlock *node);
		virtual CodeGeneratorExpressionBlockPseudopc *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeExpressionBlock);
};

class CodeGeneratorExpressionLValueIdentifierPseudopc : public CodeGeneratorExpressionLValuePseudopc
{
	public:
		CodeGeneratorExpressionLValueIdentifierPseudopc(AstNodeExpressionLValueIdentifier *node);
		virtual CodeGeneratorExpressionLValueIdentifierPseudopc *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeExpressionLValueIdentifier);
};

class CodeGeneratorExpressionLValueVectorPseudopc : public CodeGeneratorExpressionLValuePseudopc
{
	public:
		CodeGeneratorExpressionLValueVectorPseudopc(AstNodeExpressionLValueVector *node);
		virtual CodeGeneratorExpressionLValueVectorPseudopc *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeExpressionLValueVector);
};


#include "struct.hpp"
#include "op.hpp"


#endif
