/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../../ast/astnode.hpp"
#include "codegen.hpp"



CodeGeneratorFunctionDeclarationPseudopc::CodeGeneratorFunctionDeclarationPseudopc(AstNodeFunctionDeclaration *node) :
	CodeGeneratorPseudopc(node)
{
}

CodeGeneratorFunctionDeclarationPseudopc *CodeGeneratorFunctionDeclarationPseudopc::Copy()
{
	return new CodeGeneratorFunctionDeclarationPseudopc(GetNode());
}

std::string CodeGeneratorFunctionDeclarationPseudopc::Generate(CompileContext &context)
{
	std::string arguments;
	for (auto argument: GetNode()->GetArguments()){
		if (arguments.size()){
			arguments += ", ";
		}
		arguments += argument->GetCodeGenerator<CodeGeneratorStatementVariableDeclarationPseudopc>()->GenerateNonStatement(context);
	}
	
	std::string statement = GetNode()->GetStatement() ? GetNode()->GetStatement()->GetCodeGenerator<CodeGeneratorStatementPseudopc>()->Generate(context) : "";
	
	return PrintString(
		"/* %s */\n%s%s %s (%s)%s\n%s%s",
		GetNode()->GetSourceLocation().c_str(),
		GetNode()->GetExport() ? "export " : (GetNode()->GetStatement() ? "" : "import "),
		GetNode()->GetReturnType()->GetCodeGenerator<CodeGeneratorTypePseudopc>()->Generate(context).c_str(),
		GetNode()->GetIdentifier().c_str(),
		arguments.c_str(),
		GetNode()->GetStatement() ? "" : ";",
		(GetNode()->GetStatement() && dynamic_cast<AstNodeStatementBlock *>(GetNode()->GetStatement())) ? statement.c_str() : Indent(statement, 1).c_str(),
		GetNode()->GetStatement() ? "\n\n" : ""
	);
}



CodeGeneratorExpressionFunctionCallPseudopc::CodeGeneratorExpressionFunctionCallPseudopc(AstNodeFunctionCall *node) :
	CodeGeneratorExpressionPseudopc(node)
{
}

CodeGeneratorExpressionFunctionCallPseudopc *CodeGeneratorExpressionFunctionCallPseudopc::Copy()
{
	return new CodeGeneratorExpressionFunctionCallPseudopc(GetNode());
}

std::string CodeGeneratorExpressionFunctionCallPseudopc::Generate(CompileContext &context)
{
	std::string arguments;
	for (auto argument: GetNode()->GetExpressions()){
		if (arguments.size()){
			arguments += ", ";
		}
		arguments += argument->GetCodeGenerator<CodeGeneratorExpressionPseudopc>()->Generate(context);
	}
	
	return PrintString(
		"%s (%s)",
		GetNode()->GetLValue()->GetCodeGenerator<CodeGeneratorExpressionLValuePseudopc>()->Generate(context).c_str(),
		arguments.c_str()
	);
}

