/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../../ast/astnode.hpp"
#include "codegen.hpp"




CodeGeneratorStatementParallelPseudopc::CodeGeneratorStatementParallelPseudopc(AstNodeStatementParallel *node) :
	CodeGeneratorStatementPseudopc(node, true)
{
}

CodeGeneratorStatementParallelPseudopc *CodeGeneratorStatementParallelPseudopc::Copy()
{
	return new CodeGeneratorStatementParallelPseudopc(GetNode());
}

std::string CodeGeneratorStatementParallelPseudopc::Generate(CompileContext &context)
{
	return "parallel_statement";
}


CodeGeneratorStatementParallelSyncPseudopc::CodeGeneratorStatementParallelSyncPseudopc(AstNodeStatementParallelSync *node) :
	CodeGeneratorStatementParallelPseudopc(node)
{
}

CodeGeneratorStatementParallelSyncPseudopc *CodeGeneratorStatementParallelSyncPseudopc::Copy()
{
	return new CodeGeneratorStatementParallelSyncPseudopc(GetNode());
}

std::string CodeGeneratorStatementParallelSyncPseudopc::Generate(CompileContext &context)
{
	return PrintString("sync %s", GetNode()->GetStatement()->GetCodeGenerator<CodeGeneratorStatementPseudopc>()->Generate(context).c_str());
}

CodeGeneratorStatementParallelSpawnPseudopc::CodeGeneratorStatementParallelSpawnPseudopc(AstNodeStatementParallelSpawn *node) :
	CodeGeneratorStatementParallelPseudopc(node)
{
}

CodeGeneratorStatementParallelSpawnPseudopc *CodeGeneratorStatementParallelSpawnPseudopc::Copy()
{
	return new CodeGeneratorStatementParallelSpawnPseudopc(GetNode());
}

std::string CodeGeneratorStatementParallelSpawnPseudopc::Generate(CompileContext &context)
{
	//TODO: Condition
	return PrintString("spawn %s", GetNode()->GetStatement() ? GetNode()->GetStatement()->GetCodeGenerator<CodeGeneratorStatementPseudopc>()->Generate(context).c_str() : "();");
}


CodeGeneratorStatementVariableDeclarationForSpawnPseudopc::CodeGeneratorStatementVariableDeclarationForSpawnPseudopc(AstNodeStatementVariableDeclaration *node) :
	CodeGeneratorStatementVariableDeclarationPseudopc(node)
{
}

CodeGeneratorStatementVariableDeclarationForSpawnPseudopc *CodeGeneratorStatementVariableDeclarationForSpawnPseudopc::Copy()
{
	return new CodeGeneratorStatementVariableDeclarationForSpawnPseudopc(GetNode());
}

std::string CodeGeneratorStatementVariableDeclarationForSpawnPseudopc::Generate(CompileContext &context)
{
	return "CodeGeneratorStatementVariableDeclarationPseudopc";
}



CodeGeneratorStatementParallelSpawnExtractedPseudopc::CodeGeneratorStatementParallelSpawnExtractedPseudopc
	(AstNodeStatementParallelSpawn *node, std::list<AstNodeStatementVariableDeclaration *> variables, unsigned int linkid) :
	CodeGeneratorStatementParallelPseudopc(node)
{
}

CodeGeneratorStatementParallelSpawnExtractedPseudopc *CodeGeneratorStatementParallelSpawnExtractedPseudopc::Copy()
{
	return new CodeGeneratorStatementParallelSpawnExtractedPseudopc(GetNode(), m_variables, m_linkid);
}

std::string CodeGeneratorStatementParallelSpawnExtractedPseudopc::Generate(CompileContext &context)
{
	std::string arguments;
	for (auto variable: m_variables){
		arguments += PrintString(", %s", variable->GetIdentifier().c_str());
	}
	
	return PrintString("StatementParallelSpawnExtracted(spawn.%u%s)", m_linkid, arguments.c_str());
}



CodeGeneratorStatementExtractedPseudopc::CodeGeneratorStatementExtractedPseudopc(AstNodeStatementExtracted *node) :
	CodeGeneratorStatementPseudopc(node, true)
{
}

CodeGeneratorStatementExtractedPseudopc *CodeGeneratorStatementExtractedPseudopc::Copy()
{
	return new CodeGeneratorStatementExtractedPseudopc(GetNode());
}

std::string CodeGeneratorStatementExtractedPseudopc::Generate(CompileContext &context)
{
	return PrintString("StatementExtracted %s",
		GetNode()->GetStatement()->GetCodeGenerator<CodeGeneratorStatementPseudopc>()->Generate(context).c_str());
}



CodeGeneratorTypeLockPseudopc::CodeGeneratorTypeLockPseudopc(AstNodeTypeLock *node) :
	CodeGeneratorTypePseudopc(node)
{
}

CodeGeneratorTypeLockPseudopc *CodeGeneratorTypeLockPseudopc::Copy()
{
	return new CodeGeneratorTypeLockPseudopc(GetNode());
}

std::string CodeGeneratorTypeLockPseudopc::Generate(CompileContext &context)
{
	return PrintString("%s lock", GetNode()->GetType()->GetCodeGenerator<CodeGeneratorTypePseudopc>()->Generate(context).c_str());
}



CodeGeneratorStatementParallelAtomicPseudopc::CodeGeneratorStatementParallelAtomicPseudopc(AstNodeStatementParallelAtomic *node) :
	CodeGeneratorStatementParallelPseudopc(node)
{
}

CodeGeneratorStatementParallelAtomicPseudopc *CodeGeneratorStatementParallelAtomicPseudopc::Copy()
{
	return new CodeGeneratorStatementParallelAtomicPseudopc(GetNode());
}

std::string CodeGeneratorStatementParallelAtomicPseudopc::Generate(CompileContext &context)
{
	return PrintString("atomic %s", GetNode()->GetStatement()->GetCodeGenerator<CodeGeneratorStatementPseudopc>()->Generate(context).c_str());
}



CodeGeneratorExpressionLValueLockPseudopc::CodeGeneratorExpressionLValueLockPseudopc(AstNodeExpressionLValueLock *node) :
	CodeGeneratorExpressionLValuePseudopc(node)
{
}

CodeGeneratorExpressionLValueLockPseudopc *CodeGeneratorExpressionLValueLockPseudopc::Copy()
{
	return new CodeGeneratorExpressionLValueLockPseudopc(GetNode());
}

std::string CodeGeneratorExpressionLValueLockPseudopc::Generate(CompileContext &context)
{
	return PrintString("lock_access (%s)", GetNode()->GetLValue()->GetCodeGenerator<CodeGeneratorExpressionLValuePseudopc>()->Generate(context).c_str());
}








