/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef CODEGEN_PSEUDOPC_PARALLEL_HPP
#define CODEGEN_PSEUDOPC_PARALLEL_HPP

#include "statement.hpp"

class CodeGeneratorStatementParallelPseudopc : public CodeGeneratorStatementPseudopc
{
	public:
		CodeGeneratorStatementParallelPseudopc(AstNodeStatementParallel *node);
		virtual CodeGeneratorStatementParallelPseudopc *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeStatementParallel);
};

class CodeGeneratorStatementParallelSyncPseudopc : public CodeGeneratorStatementParallelPseudopc
{
	public:
		CodeGeneratorStatementParallelSyncPseudopc(AstNodeStatementParallelSync *node);
		virtual CodeGeneratorStatementParallelSyncPseudopc *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeStatementParallelSync);
};

class CodeGeneratorStatementParallelSpawnPseudopc : public CodeGeneratorStatementParallelPseudopc
{
	public:
		CodeGeneratorStatementParallelSpawnPseudopc(AstNodeStatementParallelSpawn *node);
		virtual CodeGeneratorStatementParallelSpawnPseudopc *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeStatementParallelSpawn);
};

class CodeGeneratorStatementVariableDeclarationForSpawnPseudopc : public CodeGeneratorStatementVariableDeclarationPseudopc
{
	public:
		CodeGeneratorStatementVariableDeclarationForSpawnPseudopc(AstNodeStatementVariableDeclaration *node);
		virtual CodeGeneratorStatementVariableDeclarationForSpawnPseudopc *Copy();
		virtual std::string Generate(CompileContext &context);
};

class CodeGeneratorStatementParallelSpawnExtractedPseudopc : public CodeGeneratorStatementParallelPseudopc
{
	public:
		CodeGeneratorStatementParallelSpawnExtractedPseudopc(AstNodeStatementParallelSpawn *node, std::list<AstNodeStatementVariableDeclaration *> variables, unsigned int linkid);
		
		virtual CodeGeneratorStatementParallelSpawnExtractedPseudopc *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeStatementParallelSpawn);
		
	private:
		std::list<AstNodeStatementVariableDeclaration *> m_variables;
		unsigned int m_linkid;
};

class CodeGeneratorStatementExtractedPseudopc : public CodeGeneratorStatementPseudopc
{
	public:
		AstType(AstNodeStatementExtracted);
		
		CodeGeneratorStatementExtractedPseudopc(AstNodeStatementExtracted *node);
		
		CodeGeneratorStatementExtractedPseudopc *Copy();
		std::string Generate(CompileContext &context);
};

class CodeGeneratorTypeLockPseudopc : public CodeGeneratorTypePseudopc
{
	public:
		CodeGeneratorTypeLockPseudopc(AstNodeTypeLock *node);
		virtual CodeGeneratorTypeLockPseudopc *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeTypeLock);
};

class CodeGeneratorStatementParallelAtomicPseudopc : public CodeGeneratorStatementParallelPseudopc
{
	public:
		CodeGeneratorStatementParallelAtomicPseudopc(AstNodeStatementParallelAtomic *node);
		virtual CodeGeneratorStatementParallelAtomicPseudopc *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeStatementParallelAtomic);
};

class CodeGeneratorExpressionLValueLockPseudopc : public CodeGeneratorExpressionLValuePseudopc
{
	public:
		CodeGeneratorExpressionLValueLockPseudopc(AstNodeExpressionLValueLock *node);
		virtual CodeGeneratorExpressionLValueLockPseudopc *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeExpressionLValueLock);
};



#endif
 
