/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../../ast/astnode.hpp"
#include "codegen.hpp"



CodeGeneratorTypeReferencePseudopc::CodeGeneratorTypeReferencePseudopc(AstNodeTypeReference *node) :
	CodeGeneratorTypePseudopc(node)
{
}

CodeGeneratorTypeReferencePseudopc *CodeGeneratorTypeReferencePseudopc::Copy()
{
	return new CodeGeneratorTypeReferencePseudopc(GetNode());
}

std::string CodeGeneratorTypeReferencePseudopc::Generate(CompileContext &context)
{
	return PrintString("%s ref", GetNode()->GetType()->GetCodeGenerator<CodeGeneratorTypePseudopc>()->Generate(context).c_str());
}




CodeGeneratorExpressionReferencePseudopc::CodeGeneratorExpressionReferencePseudopc(AstNodeExpressionReference *node) :
	CodeGeneratorExpressionPseudopc(node)
{
}

CodeGeneratorExpressionReferencePseudopc *CodeGeneratorExpressionReferencePseudopc::Copy()
{
	return new CodeGeneratorExpressionReferencePseudopc(GetNode());
}

std::string CodeGeneratorExpressionReferencePseudopc::Generate(CompileContext &context)
{
	return PrintString("ref (%s)", GetNode()->GetLValue()->GetCodeGenerator<CodeGeneratorExpressionLValuePseudopc>()->Generate(context).c_str());
}


CodeGeneratorExpressionLValueDereferencePseudopc::CodeGeneratorExpressionLValueDereferencePseudopc(AstNodeExpressionLValueDereference *node) :
	CodeGeneratorExpressionLValuePseudopc(node)
{
}

CodeGeneratorExpressionLValueDereferencePseudopc *CodeGeneratorExpressionLValueDereferencePseudopc::Copy()
{
	return new CodeGeneratorExpressionLValueDereferencePseudopc(GetNode());
}

std::string CodeGeneratorExpressionLValueDereferencePseudopc::Generate(CompileContext &context)
{
	return PrintString("deref (%s)", GetNode()->GetExpression()->GetCodeGenerator<CodeGeneratorExpressionPseudopc>()->Generate(context).c_str());
}

CodeGeneratorExpressionAllocPseudopc::CodeGeneratorExpressionAllocPseudopc(AstNodeExpressionAlloc *node) :
	CodeGeneratorExpressionPseudopc(node)
{
}

CodeGeneratorExpressionAllocPseudopc *CodeGeneratorExpressionAllocPseudopc::Copy()
{
	return new CodeGeneratorExpressionAllocPseudopc(GetNode());
}

std::string CodeGeneratorExpressionAllocPseudopc::Generate(CompileContext &context)
{
	return PrintString("alloc %s", GetNode()->GetType()->GetType()->GetCodeGenerator<CodeGeneratorTypePseudopc>()->Generate(context).c_str());
}


CodeGeneratorStatementDeallocPseudopc::CodeGeneratorStatementDeallocPseudopc(AstNodeStatementDealloc *node) :
	CodeGeneratorStatementPseudopc(node, false)
{
}

CodeGeneratorStatementDeallocPseudopc *CodeGeneratorStatementDeallocPseudopc::Copy()
{
	return new CodeGeneratorStatementDeallocPseudopc(GetNode());
}

std::string CodeGeneratorStatementDeallocPseudopc::Generate(CompileContext &context)
{
	return PrintString("dealloc %s", GetNode()->GetExpression()->GetCodeGenerator<CodeGeneratorExpressionPseudopc>()->Generate(context).c_str());
}
