/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../../ast/astnode.hpp"
#include "codegen.hpp"




CodeGeneratorStatementPseudopc::CodeGeneratorStatementPseudopc(AstNodeStatement *node, bool multiline) :
	CodeGeneratorPseudopc(node), m_multiline(multiline)
{
}

bool CodeGeneratorStatementPseudopc::GetMultiline()
{
	return m_multiline;
}

CodeGeneratorStatementReturnPseudopc::CodeGeneratorStatementReturnPseudopc(AstNodeStatementReturn *node) :
	CodeGeneratorStatementPseudopc(node, false)
{
}

CodeGeneratorStatementReturnPseudopc *CodeGeneratorStatementReturnPseudopc::Copy()
{
	return new CodeGeneratorStatementReturnPseudopc(GetNode());
}

std::string CodeGeneratorStatementReturnPseudopc::Generate(CompileContext &context)
{
	return PrintString("return%s%s;", GetNode()->GetExpression() ? " " : "",
		GetNode()->GetExpression() ? GetNode()->GetExpression()->GetCodeGenerator<CodeGeneratorExpressionPseudopc>()->Generate(context).c_str() : "");
}



CodeGeneratorStatementVariableAssignmentPseudopc::CodeGeneratorStatementVariableAssignmentPseudopc(AstNodeStatementVariableAssignment *node) :
	CodeGeneratorStatementPseudopc(node, false)
{
}

CodeGeneratorStatementVariableAssignmentPseudopc *CodeGeneratorStatementVariableAssignmentPseudopc::Copy()
{
	return new CodeGeneratorStatementVariableAssignmentPseudopc(GetNode());
}

std::string CodeGeneratorStatementVariableAssignmentPseudopc::Generate(CompileContext &context)
{
	return PrintString("%s = %s;",
		GetNode()->GetLValue()->GetCodeGenerator<CodeGeneratorExpressionPseudopc>()->Generate(context).c_str(),
		GetNode()->GetExpression()->GetCodeGenerator<CodeGeneratorExpressionPseudopc>()->Generate(context).c_str());
}



CodeGeneratorStatementVariableDeclarationPseudopc::CodeGeneratorStatementVariableDeclarationPseudopc(AstNodeStatementVariableDeclaration *node) :
	CodeGeneratorStatementPseudopc(node, false)
{
}

CodeGeneratorStatementVariableDeclarationPseudopc *CodeGeneratorStatementVariableDeclarationPseudopc::Copy()
{
	return new CodeGeneratorStatementVariableDeclarationPseudopc(GetNode());
}

std::string CodeGeneratorStatementVariableDeclarationPseudopc::Generate(CompileContext &context)
{
	return PrintString("%s %s;",
		GetNode()->GetType()->GetCodeGenerator<CodeGeneratorTypePseudopc>()->Generate(context).c_str(),
		GetNode()->GetIdentifier().c_str());
}

std::string CodeGeneratorStatementVariableDeclarationPseudopc::GenerateNonStatement(CompileContext &context)
{
	return PrintString("%s %s",
		GetNode()->GetType()->GetCodeGenerator<CodeGeneratorTypePseudopc>()->Generate(context).c_str(),
		GetNode()->GetIdentifier().c_str());
}



CodeGeneratorStatementExpressionPseudopc::CodeGeneratorStatementExpressionPseudopc(AstNodeStatementExpression *node) :
	CodeGeneratorStatementPseudopc(node, false)
{
}

CodeGeneratorStatementExpressionPseudopc *CodeGeneratorStatementExpressionPseudopc::Copy()
{
	return new CodeGeneratorStatementExpressionPseudopc(GetNode());
}

std::string CodeGeneratorStatementExpressionPseudopc::Generate(CompileContext &context)
{
	return GetNode()->GetExpression()->GetCodeGenerator<CodeGeneratorExpressionPseudopc>()->Generate(context) + ";";
}



CodeGeneratorStatementBlockPseudopc::CodeGeneratorStatementBlockPseudopc(AstNodeStatementBlock *node) :
	CodeGeneratorStatementPseudopc(node, true)
{
}

CodeGeneratorStatementBlockPseudopc *CodeGeneratorStatementBlockPseudopc::Copy()
{
	return new CodeGeneratorStatementBlockPseudopc(GetNode());
}

std::string CodeGeneratorStatementBlockPseudopc::Generate(CompileContext &context)
{
	std::string statements;
	for (AstNodeStatement *statement: GetNode()->GetStatements()){
		if (statement->GetCodeGenerator<CodeGeneratorStatementPseudopc>()->GetMultiline()){
			statements += Indent(PrintString("\n/* %s */\n", statement->GetSourceLocation().c_str()) + statement->GetCodeGenerator<CodeGeneratorStatementPseudopc>()->Generate(context) + "\n", 1);
		}
		else {
			statements += Indent(statement->GetCodeGenerator<CodeGeneratorStatementPseudopc>()->Generate(context) +
				PrintString(" //%s\n", statement->GetSourceLocation().c_str()), 1);
		}
	}
	
	return PrintString("{\n%s}", statements.c_str());
}



CodeGeneratorStatementMarkedGroupPseudopc::CodeGeneratorStatementMarkedGroupPseudopc(AstNodeStatementMarkedGroup *node) :
	CodeGeneratorStatementPseudopc(node, true)
{
}

CodeGeneratorStatementMarkedGroupPseudopc *CodeGeneratorStatementMarkedGroupPseudopc::Copy()
{
	return new CodeGeneratorStatementMarkedGroupPseudopc(GetNode());
}

std::string CodeGeneratorStatementMarkedGroupPseudopc::Generate(CompileContext &context)
{
	std::string statements;
	for (AstNodeStatement *statement: GetNode()->GetStatements()){
		statements += Indent(statement->GetCodeGenerator<CodeGeneratorStatementPseudopc>()->Generate(context) + "\n", 1);
	}
	
	return PrintString("marked_group(%s){\n%s}", GetNode()->GetMarkName().c_str(), statements.c_str());
}


