/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef CODEGEN_PSEUDOPC_STATEMENT_HPP
#define CODEGEN_PSEUDOPC_STATEMENT_HPP

#include "codegen.hpp"


class CodeGeneratorStatementPseudopc : public CodeGeneratorPseudopc
{
	public:
		CodeGeneratorStatementPseudopc(AstNodeStatement *node, bool multiline);
		virtual AstNodeStatement *GetNode() = 0;
		virtual std::string Generate(CompileContext &context) = 0;
		
		bool GetMultiline();
		
	private:
		bool m_multiline;
};

class CodeGeneratorStatementReturnPseudopc : public CodeGeneratorStatementPseudopc
{
	public:
		CodeGeneratorStatementReturnPseudopc(AstNodeStatementReturn *node);
		virtual CodeGeneratorStatementReturnPseudopc *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeStatementReturn);
};

class CodeGeneratorStatementVariableAssignmentPseudopc : public CodeGeneratorStatementPseudopc
{
	public:
		CodeGeneratorStatementVariableAssignmentPseudopc(AstNodeStatementVariableAssignment *node);
		virtual CodeGeneratorStatementVariableAssignmentPseudopc *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeStatementVariableAssignment);
};

class CodeGeneratorStatementVariableDeclarationPseudopc : public CodeGeneratorStatementPseudopc
{
	public:
		CodeGeneratorStatementVariableDeclarationPseudopc(AstNodeStatementVariableDeclaration *node);
		virtual CodeGeneratorStatementVariableDeclarationPseudopc *Copy();
		virtual std::string Generate(CompileContext &context);
		std::string GenerateNonStatement(CompileContext &context);
		
		AstType(AstNodeStatementVariableDeclaration);
};

class CodeGeneratorStatementExpressionPseudopc : public CodeGeneratorStatementPseudopc
{
	public:
		CodeGeneratorStatementExpressionPseudopc(AstNodeStatementExpression *node);
		virtual CodeGeneratorStatementExpressionPseudopc *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeStatementExpression);
};

class CodeGeneratorStatementBlockPseudopc : public CodeGeneratorStatementPseudopc
{
	public:
		CodeGeneratorStatementBlockPseudopc(AstNodeStatementBlock *node);
		virtual CodeGeneratorStatementBlockPseudopc *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeStatementBlock);
};

class CodeGeneratorStatementMarkedGroupPseudopc : public CodeGeneratorStatementPseudopc
{
	public:
		CodeGeneratorStatementMarkedGroupPseudopc(AstNodeStatementMarkedGroup *node);
		virtual CodeGeneratorStatementMarkedGroupPseudopc *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeStatementMarkedGroup);
};

#include "ctrlflow.hpp"

#endif
