/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../../ast/astnode.hpp"
#include "codegen.hpp"



CodeGeneratorStructDeclarationPseudopc::CodeGeneratorStructDeclarationPseudopc(AstNodeStructDeclaration *node) :
	CodeGeneratorPseudopc(node)
{
}

CodeGeneratorStructDeclarationPseudopc *CodeGeneratorStructDeclarationPseudopc::Copy()
{
	return new CodeGeneratorStructDeclarationPseudopc(GetNode());
}

std::string CodeGeneratorStructDeclarationPseudopc::Generate(CompileContext &context)
{
	std::string declarations;
	for (auto item: GetNode()->GetItems()){
		declarations += PrintString("\t%s %s;\n",
			item->GetType()->GetCodeGenerator<CodeGeneratorTypePseudopc>()->Generate(context).c_str(),
			item->GetIdentifier().c_str()
		);
	}
	
	return PrintString("struct %s\n{\n%s}\n\n", GetNode()->GetIdentifier().c_str(), declarations.c_str());
}



CodeGeneratorTypeStructPseudopc::CodeGeneratorTypeStructPseudopc(AstNodeTypeStruct *node) :
	CodeGeneratorTypePseudopc(node)
{
}

CodeGeneratorTypeStructPseudopc *CodeGeneratorTypeStructPseudopc::Copy()
{
	return new CodeGeneratorTypeStructPseudopc(GetNode());
}

std::string CodeGeneratorTypeStructPseudopc::Generate(CompileContext &context)
{
	return PrintString("struct %s", GetNode()->GetIdentifier().c_str());
}



CodeGeneratorExpressionLValueStructElementPseudopc::CodeGeneratorExpressionLValueStructElementPseudopc(AstNodeExpressionLValueStructElement *node) :
	CodeGeneratorExpressionLValuePseudopc(node)
{
}

CodeGeneratorExpressionLValueStructElementPseudopc *CodeGeneratorExpressionLValueStructElementPseudopc::Copy()
{
	return new CodeGeneratorExpressionLValueStructElementPseudopc(GetNode());
}

std::string CodeGeneratorExpressionLValueStructElementPseudopc::Generate(CompileContext &context)
{
	return PrintString("%s.%s", GetNode()->GetLValue()->GetCodeGenerator<CodeGeneratorExpressionPseudopc>()->Generate(context).c_str(), GetNode()->GetIdentifier().c_str());
}


