/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../../ast/astnode.hpp"
#include "codegen.hpp"



CodeGeneratorTypePseudopc::CodeGeneratorTypePseudopc(AstNodeType *node) :
	CodeGeneratorPseudopc(node)
{
}



CodeGeneratorTypeVoidPseudopc::CodeGeneratorTypeVoidPseudopc(AstNodeTypeVoid *node) :
	CodeGeneratorTypePseudopc(node)
{
}

CodeGeneratorTypeVoidPseudopc *CodeGeneratorTypeVoidPseudopc::Copy()
{
	return new CodeGeneratorTypeVoidPseudopc(GetNode());
}

std::string CodeGeneratorTypeVoidPseudopc::Generate(CompileContext &context)
{
	return "void";
}



CodeGeneratorTypeBoolPseudopc::CodeGeneratorTypeBoolPseudopc(AstNodeTypeBool *node) :
	CodeGeneratorTypePseudopc(node)
{
}

CodeGeneratorTypeBoolPseudopc *CodeGeneratorTypeBoolPseudopc::Copy()
{
	return new CodeGeneratorTypeBoolPseudopc(GetNode());
}

std::string CodeGeneratorTypeBoolPseudopc::Generate(CompileContext &context)
{
	return "bool";
}



CodeGeneratorTypeIntPseudopc::CodeGeneratorTypeIntPseudopc(AstNodeTypeInt *node) :
	CodeGeneratorTypePseudopc(node)
{
}

CodeGeneratorTypeIntPseudopc *CodeGeneratorTypeIntPseudopc::Copy()
{
	return new CodeGeneratorTypeIntPseudopc(GetNode());
}

std::string CodeGeneratorTypeIntPseudopc::Generate(CompileContext &context)
{
	return PrintString("int %u", GetNode()->GetWidth() ? GetNode()->GetWidth() : context.GetComputeSpec().GetCpu().GetSizeT());
}



CodeGeneratorTypeFloatPseudopc::CodeGeneratorTypeFloatPseudopc(AstNodeTypeFloat *node) :
	CodeGeneratorTypePseudopc(node)
{
}

CodeGeneratorTypeFloatPseudopc *CodeGeneratorTypeFloatPseudopc::Copy()
{
	return new CodeGeneratorTypeFloatPseudopc(GetNode());
}

std::string CodeGeneratorTypeFloatPseudopc::Generate(CompileContext &context)
{
	return PrintString("float %u", GetNode()->GetWidth());
}



CodeGeneratorTypePointerPseudopc::CodeGeneratorTypePointerPseudopc(AstNodeTypePointer *node) :
	CodeGeneratorTypePseudopc(node)
{
}

CodeGeneratorTypePointerPseudopc *CodeGeneratorTypePointerPseudopc::Copy()
{
	return new CodeGeneratorTypePointerPseudopc(GetNode());
}

std::string CodeGeneratorTypePointerPseudopc::Generate(CompileContext &context)
{
	return PrintString("%s pointer", GetNode()->GetType()->GetCodeGenerator<CodeGeneratorTypePseudopc>()->Generate(context).c_str());
}



CodeGeneratorTypeStaticVectorPseudopc::CodeGeneratorTypeStaticVectorPseudopc(AstNodeTypeStaticVector *node) :
	CodeGeneratorTypePseudopc(node)
{
}

CodeGeneratorTypeStaticVectorPseudopc *CodeGeneratorTypeStaticVectorPseudopc::Copy()
{
	return new CodeGeneratorTypeStaticVectorPseudopc(GetNode());
}

std::string CodeGeneratorTypeStaticVectorPseudopc::Generate(CompileContext &context)
{
	return PrintString("%s[%u]", GetNode()->GetType()->GetCodeGenerator<CodeGeneratorTypePseudopc>()->Generate(context).c_str(), GetNode()->GetCount());
}



CodeGeneratorTypeFunctionPointerPseudopc::CodeGeneratorTypeFunctionPointerPseudopc(AstNodeTypeFunctionPointer *node) :
	CodeGeneratorTypePseudopc(node)
{
}

CodeGeneratorTypeFunctionPointerPseudopc *CodeGeneratorTypeFunctionPointerPseudopc::Copy()
{
	return new CodeGeneratorTypeFunctionPointerPseudopc(GetNode());
}

std::string CodeGeneratorTypeFunctionPointerPseudopc::Generate(CompileContext &context)
{
	return "function_pointer";
}


