/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef CODEGEN_PSEUDOPC_TYPE_HPP
#define CODEGEN_PSEUDOPC_TYPE_HPP

#include "codegen.hpp"


class CodeGeneratorTypePseudopc : public CodeGeneratorPseudopc
{
	public:
		CodeGeneratorTypePseudopc(AstNodeType *node);
		
		virtual AstNodeType *GetNode() = 0;
		
		virtual std::string Generate(CompileContext &context) = 0;
		
	private:
};

class CodeGeneratorTypeVoidPseudopc : public CodeGeneratorTypePseudopc
{
	public:
		CodeGeneratorTypeVoidPseudopc(AstNodeTypeVoid *node);
		virtual CodeGeneratorTypeVoidPseudopc *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeTypeVoid);
};

class CodeGeneratorTypeBoolPseudopc : public CodeGeneratorTypePseudopc
{
	public:
		CodeGeneratorTypeBoolPseudopc(AstNodeTypeBool *node);
		virtual CodeGeneratorTypeBoolPseudopc *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeTypeBool);
};

class CodeGeneratorTypeIntPseudopc : public CodeGeneratorTypePseudopc
{
	public:
		CodeGeneratorTypeIntPseudopc(AstNodeTypeInt *node);
		virtual CodeGeneratorTypeIntPseudopc *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeTypeInt);
};

class CodeGeneratorTypeFloatPseudopc : public CodeGeneratorTypePseudopc
{
	public:
		CodeGeneratorTypeFloatPseudopc(AstNodeTypeFloat *node);
		virtual CodeGeneratorTypeFloatPseudopc *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeTypeFloat);
};

class CodeGeneratorTypePointerPseudopc : public CodeGeneratorTypePseudopc
{
	public:
		CodeGeneratorTypePointerPseudopc(AstNodeTypePointer *node);
		virtual CodeGeneratorTypePointerPseudopc *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeTypePointer);
};

class CodeGeneratorTypeStaticVectorPseudopc : public CodeGeneratorTypePseudopc
{
	public:
		CodeGeneratorTypeStaticVectorPseudopc(AstNodeTypeStaticVector *node);
		virtual CodeGeneratorTypeStaticVectorPseudopc *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeTypeStaticVector);
};

class CodeGeneratorTypeFunctionPointerPseudopc : public CodeGeneratorTypePseudopc
{
	public:
		CodeGeneratorTypeFunctionPointerPseudopc(AstNodeTypeFunctionPointer *node);
		virtual CodeGeneratorTypeFunctionPointerPseudopc *Copy();
		virtual std::string Generate(CompileContext &context);
		
		AstType(AstNodeTypeFunctionPointer);
};

//I think I've decided that struct will be a simple structure class (with a packed variant (packed keyword?))
//class will be more complicated with the ability to have associated metadata
//a sizeof like thing would probably be useful for use with struct

#endif
