/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef COMPUTESPEC_COMPUTESPEC_HPP
#define COMPUTESPEC_COMPUTESPEC_HPP

#include <stdint.h>
#include <set>
#include <string>

#include "../error.hpp"

#ifndef YYSTYPE
	#define YYSTYPE COMPUTESPEC_STYPE
	#define YYLTYPE COMPUTESPEC_LTYPE
#endif

class ComputeSpecMachine : public TrackedClass
{
	public:
		virtual ~ComputeSpecMachine() = default;
		virtual void Autodetect() = 0;
		virtual std::string GetSpecSource() = 0;
		
	private:
};

#include "cpu.hpp"

#ifdef OPENCL
	#include "gpu.hpp"
#endif

class ComputeSpec : public TrackedClass
{
	public:
		ComputeSpec();
		virtual ~ComputeSpec() = default;
		
		void Autodetect();
		std::string GetSpecSource();
		
		ComputeSpecMachineCpu GetCpu();
		
		#ifdef OPENCL
			ComputeSpecMachineCpu GetGpu();
		#endif
		
	private:
		ComputeSpec(const ComputeSpec &) = delete;
		ComputeSpec &operator=(const ComputeSpec &) = delete;
		
		ComputeSpecMachineCpu m_cpu;
		
		#ifdef OPENCL
			ComputeSpecMachineGpu m_gpu;
		#endif
};

typedef struct ComputeSpecParserData
{
	void *scanner;
	const char *filename;
	bool error;
	ComputeSpec spec;
} ComputeSpecParserData;

#endif
