/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <cinttypes>
#include <unistd.h>
#include <cstring>
#include <list>
#include <map>

#include "computespec.hpp"
#include "../util.hpp"

ComputeSpecMachineGpu::ComputeSpecMachineGpu() :
	m_size_t(0), m_simd_width(0), m_memory_size(0)
{
}

unsigned int ComputeSpecMachineGpu::GetSizeT()
{
	return m_size_t;
}

unsigned int ComputeSpecMachineGpu::GetCoreCount()
{
	return m_core_count;
}

unsigned int ComputeSpecMachineGpu::GetSimdWidth()
{
	return m_simd_width;
}

uint64_t ComputeSpecMachineGpu::GetMemorySize()
{
	return m_memory_size;
}


void ComputeSpecMachineGpu::Autodetect()
{
	std::set<OpenCL::Platform> platforms = OpenCL::Platform::GetPlatforms();
	for (OpenCL::Platform platform: platforms){
		std::set<OpenCL::Device> devices = platform.GetDevices();
		for (OpenCL::Device device: devices){
			if (CL_DEVICE_TYPE_GPU == device.GetType()){
				m_size_t = device.GetAddressBits();
				m_core_count = device.GetMaxComputeUnits() * device.GetKernelPreferredWorkGroupSizeMultiple();
				m_simd_width = device.GetKernelPreferredWorkGroupSizeMultiple();
				m_memory_size = device.GetGlobalMemSize();
			}
		}
	}
}

std::string ComputeSpecMachineGpu::GetSpecSource()
{
	return PrintString("machine gpu\n"
		"{\n"
			"\tsize_t: %u\n"
			"\tcores: %u\n"
			"\tsimd_width: %u\n"
			"\tmemory: %" PRIu64 "\n"
		"}\n",
		
		m_size_t,
		m_core_count,
		m_simd_width,
		m_memory_size
	);
}

std::string ComputeSpecMachineGpu::GetOpenCLInfo()
{
	return "";
}

