;
; The output of the reference Polycute implementation's compiler (or any
; derivative of it) is is a derivative work of the input Polycute Language
; source code and any templates used by that compiler (including, but not
; limited, to those templates that are part of the compiler) to generate the
; compiler's output. The output is a derivative work of both the reference
; Polycute implementation (including templates it contains), and the Polycute
; Language source code that was used as input to the compiler to generate its
; output, and therefore must be licensed under the GPL. Any program that links
; to the reference Polycute implementation library (the source of which is
; contained in src/lib/libpolycute of the reference Polycute implementation
; source distribution produced by Nicholas Tomlinson) or any derivative of it is
; a derivative work of the reference Polycute implementation, and therefore must
; be licensed under the GPL.
; The "Polycute Language" source code refers to the input to the compiler,
; written in the Polycute Language.
; "Template" refers to any program fragment or code fragment (even if they
; include replacement markers such as those used by printf or similar) that is
; copyrightable and that is copied into the target code (even if after
; replacement such as (but not limited to) that performed by printf or similar).
;
;
;
;
; ======== Polycute LLVM Header ======== ;
;
;	Polycute - The reference Polycute implementation
;	Copyright (C) 2012-2013  Nicholas Tomlinson
;	
;	This program is free software: you can redistribute it and/or modify
;	it under the terms of the GNU General Public License as published by
;	the Free Software Foundation, either version 3 of the License, or
;	(at your option) any later version.
;	
;	This program is distributed in the hope that it will be useful,
;	but WITHOUT ANY WARRANTY; without even the implied warranty of
;	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;	GNU General Public License for more details.
;	
;	You should have received a copy of the GNU General Public License
;	along with this program.  If not, see <http://www.gnu.org/licenses/>.

; ==== Types ==== ;

; Dynamic vector type
%t.dynvect = type <{
	%t.size, ; type
	i8*, ; data
	%t.size, ; element count
	%t.size ; element size
}>

; Reference type
%t.ref = type <{
	%t.size, ; type
	i8* ; data
}>

; Lock type
%t.lock = type <{
	i8* ; Pointer to the lock allocated with libpolycute
}>

; Lock tree - used to get locks in the correct order
; These should be allocated dynamically
%t.lock.list = type {
	%t.lock, ; The lock of the current node
	%t.lock.list* ; next
}

; ==== LLVM Intrinsics ====
declare void @llvm.debugtrap() nounwind

; ==== External Libc Functions ==== ;
declare i32 @posix_memalign(i8** nocapture, %t.size, %t.size) nonlazybind nounwind ; the malloc() we're using - allows aligned malloc
declare void @free(i8* nocapture) nonlazybind nounwind ; free() from libc

; ==== External Polycute Functions ==== ;
declare ccc void @_libpolycute_sync_start() nonlazybind nounwind ; polycute library function to start a synchronisation block
declare ccc void @_libpolycute_sync_end() nonlazybind nounwind ; polycute library function to end a synchronisation block
declare ccc void @_libpolycute_enqueue(void (i8*)*, i8* nocapture noalias, %t.size) nonlazybind nounwind ; the function to enqueue a function call
declare ccc i8*  @_libpolycute_allocate_lock() nonlazybind nounwind ; the function to create a lock
declare ccc void @_libpolycute_deallocate_lock(i8*) nonlazybind nounwind ; the function to destroy a lock
declare ccc void @_libpolycute_lock_lock(i8*) nonlazybind nounwind ; the function to lock a lock
declare ccc void @_libpolycute_unlock_lock(i8*) nonlazybind nounwind ; the function to unlock a lock
declare ccc %t.size @_libpolycute_alloc_id() nonlazybind nounwind ; allocates a unique id
declare ccc void @_libpolycute_dealloc_id(%t.size %id) nonlazybind nounwind ; deallocates a unique id

; ==== Private Library Functions ==== ;
; The malloc used by generated code
define private fastcc i8* @.malloc (%t.size %count, %t.size %element_size) alwaysinline
{
	; TODO: Perhaps at some point we should add error handling code?
	%memory = alloca i8* ; we need to write into this with posiz_memalign
	%size = mul %t.size %count, %element_size ; calculate the length in bytes of the memory to be allocated
	%r = call i32 (i8**, %t.size, %t.size)* @posix_memalign(i8** %memory, %t.size 64, %t.size %size) ; allocate it
	%memory.val = load i8** %memory ; Get the pointer of the memory allocated
	ret i8* %memory.val ; return the memory
}

define private fastcc void @.free (i8* %memory) alwaysinline ; the free used by our code
{
	call ccc void (i8*)* @free(i8* %memory) ; call free to free the memory allocated with memalign()
	ret void ; end of function
}

; Functions to get %t.size values (for counters)
define private fastcc %t.size* @.get_tsize() alwaysinline
{
	%tsize_size_pointer = getelementptr %t.size* null, %t.size 1 ; work out how big a vector is
	%tsize_size = ptrtoint %t.size* %tsize_size_pointer to %t.size ; convert it to a size type
	%i8 = call fastcc i8* (%t.size, %t.size)* @.malloc (%t.size 1, %t.size %tsize_size) ; allocate the memory
	%type = bitcast i8* %i8 to %t.size*
	ret %t.size* %type
}

define private fastcc void @.release_tsize(%t.size* %type) alwaysinline
{
	%i8 = bitcast %t.size* %type to i8*
	call fastcc void (i8*)* @.free (i8* %i8)
	ret void
}

define private fastcc %t.size @.alloc_id () alwaysinline
{
	%r = call ccc %t.size ()* @_libpolycute_alloc_id()
	ret %t.size %r
}

define private fastcc void @.dealloc_id (%t.size %id) alwaysinline
{
	call ccc void @_libpolycute_dealloc_id(%t.size %id)
	ret void
}

define private fastcc i8* @.allocatelock () alwaysinline
{
	%lock = call ccc i8* ()*  @_libpolycute_allocate_lock()
	ret i8* %lock
}

define private fastcc void @.deallocatelock (i8* %lock) alwaysinline
{
	call ccc void @_libpolycute_deallocate_lock(i8* %lock)
	ret void
}

define private fastcc void @.lock_lock (i8* %lock) alwaysinline
{
	call ccc void @_libpolycute_lock_lock(i8* %lock)
	ret void
}

define private fastcc void @.unlock_lock (i8* %lock) alwaysinline
{
	call ccc void @_libpolycute_unlock_lock(i8* %lock)
	ret void
}

; ==== Lock Stuff ==== ;

define private fastcc %t.lock @.lock.create() alwaysinline
{
	%pointer = call fastcc i8* ()* @.allocatelock ()
	%r2 = insertvalue %t.lock undef, i8* %pointer, 0
	ret %t.lock %r2
}

define private fastcc void @.lock.destroy(%t.lock %lock) alwaysinline
{
	%pointer = extractvalue %t.lock %lock, 0
	call fastcc void @.deallocatelock (i8* %pointer)
	ret void
}

define private fastcc void @.lock.lock(%t.lock %lock) alwaysinline
{
	%pointer = extractvalue %t.lock %lock, 0
	call fastcc void @.lock_lock (i8* %pointer)
	ret void
}

define private fastcc void @.lock.unlock(%t.lock %lock) alwaysinline
{
	%pointer = extractvalue %t.lock %lock, 0
	call fastcc void @.unlock_lock (i8* %pointer)
	ret void
}


; This may return %list, or a new list with %list as next
define private fastcc %t.lock.list* @.lock.list.malloc(%t.lock %lock, %t.lock.list* %list)
{
	; Get size
	%size.lock.list.pointer = getelementptr %t.lock.list* null, i32 1 ; Use getelementptr to create a pointer whose value is the size of %t.lock.list
	%size.lock.list = ptrtoint %t.lock.list* %size.lock.list.pointer to %t.size ; Convert to %t.size
	
	; See if the list is empty
	%list.empty = icmp eq %t.lock.list* %list, null
	br i1 %list.empty, label %empty, label %not_empty
	
	empty: ; The list is empty, so we just return a new one
		%empty.r.i8 = call fastcc i8* (%t.size, %t.size)* @.malloc(%t.size 1, %t.size %size.lock.list) ; Allocate memory
		%empty.r = bitcast i8* %empty.r.i8 to %t.lock.list* ; Convert pointer type
		
		%empty.r.lock = getelementptr %t.lock.list* %empty.r, i32 0, i32 0 ; Get pointer to lock
		%empty.r.next = getelementptr %t.lock.list* %empty.r, i32 0, i32 1 ; Get pointer to next
		
		store %t.lock %lock, %t.lock* %empty.r.lock ; Store the lock
		store %t.lock.list* null, %t.lock.list** %empty.r.next ; Store the null next pointer
		
		ret %t.lock.list* %empty.r ; Return the new list
	
	not_empty:
		%list.val = load %t.lock.list* %list ; Get the list object
		%list.val.lock = extractvalue %t.lock.list %list.val, 0 ; Get the list's first lock
		%list.val.next = extractvalue %t.lock.list %list.val, 1 ; Get pointer to the list
		
		%lock.pointer = extractvalue %t.lock %lock, 0 ; Get the lock's pointer
		%list.val.lock.pointer = extractvalue %t.lock %list.val.lock, 0 ; Get the list's first item's lock pointer
		
		%lock.eq = icmp eq i8* %lock.pointer, %list.val.lock.pointer
		br i1 %lock.eq, label %equal, label %not_equal
		
	equal:
		ret %t.lock.list* %list ; If they're equal, do not insert since we already have it
		
	not_equal:
		%lock.less = icmp ult i8* %lock.pointer, %list.val.lock.pointer
		br i1 %lock.less, label %less, label %greater
		
	less: ; We need to stick this lock onto the beginning of the list
		%less.r.i8 = call fastcc i8* (%t.size, %t.size)* @.malloc(%t.size 1, %t.size %size.lock.list) ; Allocate memory
		%less.r = bitcast i8* %less.r.i8 to %t.lock.list* ; Convert pointer type
		
		%less.r.lock = getelementptr %t.lock.list* %less.r, i32 0, i32 0 ; Get pointer to lock
		%less.r.next = getelementptr %t.lock.list* %less.r, i32 0, i32 1 ; Get pointer to next
		
		store %t.lock %lock, %t.lock* %less.r.lock ; Store the lock
		store %t.lock.list* %list, %t.lock.list** %less.r.next ; Store the list next pointer
		
		ret %t.lock.list* %less.r ; Return the new list
		
	greater: ; The lock needs to be inserted somewhere later, so we're actually going to return the same list
		%new_next = call fastcc %t.lock.list* (%t.lock, %t.lock.list*)* @.lock.list.malloc(%t.lock %lock, %t.lock.list* %list.val.next) ; Actually perform the insert
		
		; Now we need to update the list node we got so its next pointer is %new_next
		%greater.r1 = insertvalue %t.lock.list undef, %t.lock %list.val.lock, 0 ; Insert the list node's lock
		%greater.r2 = insertvalue %t.lock.list %greater.r1, %t.lock.list* %new_next, 1 ; Insert the next
		store %t.lock.list %greater.r2, %t.lock.list* %list ; Store the list node
		
		ret %t.lock.list* %list ; Return it
}

; This deallocates the entire list, but NOT the locks
define fastcc void @.lock.list.free(%t.lock.list* %list)
{
	%null = icmp eq %t.lock.list* %list, null ; compare %tree to null
	br i1 %null, label %end, label %free ; If null, we should just end, otherwise free
	
	free:
		%list.val = load %t.lock.list* %list ; Load the tree structure
		%next = extractvalue %t.lock.list %list.val, 1 ; Get next
		
		call fastcc void @.lock.list.free(%t.lock.list* %next) ; Free left
		
		%list.i8 = bitcast %t.lock.list* %list to i8* ; Convert pointer type
		call fastcc void @.free(i8* %list.i8) ; Free %tree
	
	br label %end
	end:
		ret void
}


define fastcc void @.lock.list.lock(%t.lock.list* %list)
{
	%null = icmp eq %t.lock.list* %list, null ; compare %tree to null
	br i1 %null, label %end, label %not_empty ; If null, we should just end, otherwise free
	
	not_empty:
		%list.val = load %t.lock.list* %list ; Load the tree structure
		%lock = extractvalue %t.lock.list %list.val, 0 ; Get lock
		%next = extractvalue %t.lock.list %list.val, 1 ; Get next
		
		call fastcc void @.lock.lock(%t.lock %lock)
		call fastcc void @.lock.list.lock(%t.lock.list* %next)
	
	br label %end
	end:
		ret void
}

define fastcc void @.lock.list.unlock(%t.lock.list* %list)
{
	%null = icmp eq %t.lock.list* %list, null ; compare %tree to null
	br i1 %null, label %end, label %not_empty ; If null, we should just end, otherwise free
	
	not_empty:
		%list.val = load %t.lock.list* %list ; Load the tree structure
		%lock = extractvalue %t.lock.list %list.val, 0 ; Get lock
		%next = extractvalue %t.lock.list %list.val, 1 ; Get next
		
		call fastcc void @.lock.unlock(%t.lock %lock)
		call fastcc void @.lock.list.unlock(%t.lock.list* %next)
	
	br label %end
	end:
		ret void
}



; ==== Dynamic Vector Stuff ==== ;

; TODO: Add debugging information (but let polycute add the line numbers)
; Allocates a new dynamic vector of size %size with a reference count of zero
define private fastcc %t.dynvect @.dynvect.malloc (%t.size %element_count, %t.size %element_size) alwaysinline
{
	%data = call fastcc i8* (%t.size, %t.size)* @.malloc (%t.size %element_count, %t.size %element_size)
	%r1 = insertvalue %t.dynvect zeroinitializer, %t.size 0, 0
	%r2 = insertvalue %t.dynvect %r1, i8* %data, 1
	%r3 = insertvalue %t.dynvect %r2, %t.size %element_count, 2
	%r4 = insertvalue %t.dynvect %r3, %t.size  %element_size, 3
	
	ret %t.dynvect %r4
}

; Frees the dynamic vector %vector if it's unreferenced by anything else
; Frees the vector if the reference count is less than or equal to max,
; and decrements  the reference count by ref_dec if not
define private fastcc void @.dynvect.free (%t.dynvect %vector) alwaysinline
{
	%vector.data = extractvalue %t.dynvect %vector, 1
	call fastcc void (i8*)* @.free (i8* %vector.data)
	ret void
}

define private fastcc %t.dynvect @.dynvect.copy (%t.dynvect %vector) alwaysinline
{
	%type = extractvalue %t.dynvect %vector, 0
	%data = extractvalue %t.dynvect %vector, 1
	%element_count = extractvalue %t.dynvect %vector, 2
	%element_size = extractvalue %t.dynvect %vector, 3
	
	%r = call fastcc %t.dynvect (%t.size, %t.size)* @.dynvect.malloc (%t.size %element_count, %t.size %element_size)
	
	%length = mul %t.size %element_count, %element_size
	%new.data = extractvalue %t.dynvect %r, 1
	call fastcc void @.memcpy_align (i8* %new.data, i8* %data, %t.size %length)
	
	ret %t.dynvect %r
}

define private fastcc %t.dynvect @.dynvect.copy_nomemcpy (%t.dynvect %vector) alwaysinline
{
	%type = extractvalue %t.dynvect %vector, 0
	%data = extractvalue %t.dynvect %vector, 1
	%element_count = extractvalue %t.dynvect %vector, 2
	%element_size = extractvalue %t.dynvect %vector, 3
	
	%r = call fastcc %t.dynvect (%t.size, %t.size)* @.dynvect.malloc (%t.size %element_count, %t.size %element_size)
	ret %t.dynvect %r
}

; TODO: Maybe we could have bounds checking if we wanted it
; Reads %size bytes from %vector at offset %offset into %dst
; Returns a pointer to read from
; See @.dynvect.commit
define private fastcc i8* @.dynvect.read (%t.dynvect %vector, %t.size %offset, %t.size %element_count) alwaysinline
{
	%data = call fastcc i8* (%t.dynvect, %t.size)* @.dynvect.offset (%t.dynvect %vector, %t.size %offset)
	ret i8* %data
}


; Writes %size bytes to %vector at offset %offset from %src, copying the vector if necessary; the new vector is returned
; Returns a pointer to write to, along with the potentially replacement vector
; See @.dynvect.commit
define private fastcc i8* @.dynvect.write (%t.dynvect %vector, %t.size %offset, %t.size %element_count) alwaysinline
{
	%data = call fastcc i8* (%t.dynvect, %t.size)* @.dynvect.offset (%t.dynvect %vector, %t.size %offset)
	ret i8* %data
}

define private fastcc i8* @.dynvect.readwrite (%t.dynvect %vector, %t.size %offset, %t.size %element_count) alwaysinline
{
	%data = call fastcc i8* (%t.dynvect, %t.size)* @.dynvect.offset (%t.dynvect %vector, %t.size %offset)
	ret i8* %data
}

define private fastcc i8* @.dynvect.offset (%t.dynvect %vector, %t.size %offset) alwaysinline
{
	%vector.data = extractvalue %t.dynvect %vector, 1
	%vector.element_size = extractvalue %t.dynvect %vector, 3
	
	%data_offset = mul %t.size %offset, %vector.element_size
	%data = getelementptr i8* %vector.data, %t.size %data_offset
	
	ret i8* %data
}

; Commits read and write operations
; must be called before we start with the pointers gotten from @.dynvect.read and @.dynvect.write (prevents overwriting of stale values)
define private fastcc void @.dynvect.commit.before(%t.dynvect %vector) alwaysinline
{
	ret void ; immediately return - currently, we do not have a commit phase (it's here for when we have OpenCL stuff, etc)
}

; must be called after we're finished with the pointers gotten from @.dynvect.read and @.dynvect.write (prevents overwriting of stale values)
define private fastcc void @.dynvect.commit.after(%t.dynvect %vector) alwaysinline
{
	ret void ; immediately return - currently, we do not have a commit phase (it's here for when we have OpenCL stuff, etc)
}

; Get the size of the vector
define private fastcc %t.size @.dynvect.element_count(%t.dynvect %vector) alwaysinline
{
	%vector.element_count = extractvalue %t.dynvect %vector, 2
	ret %t.size %vector.element_count
}



; ==== Reference Stuff ==== ;

; Reference types:
;     0 = Managed memory reference (should not (de)allocate memory)
;     1 = Dynamic memory reference (should (de)allocate memory)

; Does NOT allocate memory
define private fastcc %t.ref @.ref.create(i8* %data) alwaysinline
{
	%r1 = insertvalue %t.ref undef, %t.size 0, 0
	%r2 = insertvalue %t.ref %r1, i8* %data, 1
	ret %t.ref %r2
}

; Does allocate memory
define private fastcc %t.ref @.ref.create.dynamic(%t.size %object_size) alwaysinline
{
	%data = call fastcc i8* (%t.size, %t.size)* @.malloc (%t.size 1, %t.size %object_size) ; Allocate memory for the object
	
	%r1 = insertvalue %t.ref undef, %t.size 1, 0
	%r2 = insertvalue %t.ref %r1, i8* %data, 1
	ret %t.ref %r2
}

; Does not freee memory
define private fastcc void @.ref.destroy(%t.ref %ref) alwaysinline
{
	ret void
}

; Frees memory
define private fastcc void @.ref.destroy.dynamic(%t.ref %ref) alwaysinline
{
	%data = extractvalue %t.ref %ref, 1
	call fastcc void @.free (i8* %data)
	ret void
}

define private fastcc i8* @.ref.read(%t.ref %ref) alwaysinline
{
	%ref.data = extractvalue %t.ref %ref, 1
	ret i8* %ref.data
}

define private fastcc i8* @.ref.write(%t.ref %ref) alwaysinline
{
	%ref.data = extractvalue %t.ref %ref, 1
	ret i8* %ref.data
}

; Works similarly to dynvect, except that we never have to update the ref that's stored
; must be called before reading from the buffer returned by @.ref.read
; must be called after writing to the buffer returned by @.ref.write
define private fastcc void @.ref.commit.before(%t.ref %ref) alwaysinline
{
	ret void
}

define private fastcc void @.ref.commit.after(%t.ref %ref) alwaysinline
{
	ret void
}




; ==== Spawn/Sync Stuff ==== ;

define private fastcc void @.sync_start () alwaysinline ; used when we start a sync block
{
	call ccc void ()* @_libpolycute_sync_start()
	ret void
}

define private fastcc void @.sync_end () alwaysinline ; used when we end a sync block
{
	call ccc void ()* @_libpolycute_sync_end()
	ret void
}

; The on arguments is valid because the C++ library copies the arguments
define private fastcc void @.enqueue(void (i8*)* %function, i8* %arguments, %t.size %arguments_size) alwaysinline ; enqueue a function call
{
	call ccc void (void (i8*)*, i8*, %t.size)* @_libpolycute_enqueue(void (i8*)* %function, i8* %arguments, %t.size %arguments_size) ; call libpolycute
	ret void
}

; ======== End of Polycute LLVM Header ======== ;