/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "nodematcher.hpp"



bool NodeMatcherTrue::Match(AstNode *node)
{
	return true;
}



NodeMatcherNode::NodeMatcherNode(AstNode *node) :
	m_node(node)
{
}

bool NodeMatcherNode::Match(AstNode *node)
{
	return node == m_node;
}



NodeMatcherNot::NodeMatcherNot(NodeMatcher *matcher) :
	m_matcher(matcher)
{
}

NodeMatcherNot::~NodeMatcherNot()
{
	delete m_matcher;
}

bool NodeMatcherNot::Match(AstNode *node)
{
	return !m_matcher->Match(node);
}


NodeMatcherOr::NodeMatcherOr(std::list<NodeMatcher *> matchers) :
	m_matchers(matchers)
{
}

NodeMatcherOr::~NodeMatcherOr()
{
	for (NodeMatcher *matcher: m_matchers){
		delete matcher;
	}
}

bool NodeMatcherOr::Match(AstNode *node)
{
	for (NodeMatcher *matcher: m_matchers){
		if (matcher->Match(node)){
			return true;
		}
	}
	return false;
}



NodeMatcherAnd::NodeMatcherAnd(std::list<NodeMatcher *> matchers) :
	m_matchers(matchers)
{
}

NodeMatcherAnd::~NodeMatcherAnd()
{
	for (NodeMatcher *matcher: m_matchers){
		delete matcher;
	}
}

bool NodeMatcherAnd::Match(AstNode *node)
{
	for (NodeMatcher *matcher: m_matchers){
		if (!matcher->Match(node)){
			return false;
		}
	}
	
	return true;
}
