/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef OPTIMISER_NODEMATCHER_HPP
#define OPTIMISER_NODEMATCHER_HPP

#include <list>

#include "../error.hpp"
#include "../ast/astnode.hpp"


class NodeMatcher : public TrackedClass
{
	public:
		NodeMatcher() = default;
		virtual ~NodeMatcher() = default;
		virtual bool Match(AstNode *node) = 0;
		
	private:
		NodeMatcher(const NodeMatcher &) = delete;
		NodeMatcher &operator=(const NodeMatcher &) = delete;
};

class NodeMatcherTrue : public NodeMatcher
{
	public:
		bool Match(AstNode *node);
};

class NodeMatcherNode : public NodeMatcher
{
	public:
		NodeMatcherNode(AstNode *node);
		bool Match(AstNode *node);
		
	private:
		AstNode *m_node;
};

template <typename T> class NodeMatcherType : public NodeMatcher
{
	public:
		bool Match(AstNode *node)
		{
			return node && dynamic_cast<T *>(node);
		}
};

class NodeMatcherNot : public NodeMatcher
{
	public:
		NodeMatcherNot(NodeMatcher *matcher);
		virtual ~NodeMatcherNot();
		bool Match(AstNode *node);
		
	private:
		NodeMatcher *m_matcher;
};

class NodeMatcherOr : public NodeMatcher
{
	public:
		NodeMatcherOr(std::list<NodeMatcher *> matchers);
		virtual ~NodeMatcherOr();
		bool Match(AstNode *node);
		
	private:
		std::list<NodeMatcher *> m_matchers;
};

class NodeMatcherAnd : public NodeMatcher
{
	public:
		NodeMatcherAnd(std::list<NodeMatcher *> matchers);
		virtual ~NodeMatcherAnd();
		bool Match(AstNode *node);
		
	private:
		std::list<NodeMatcher *> m_matchers;
};




#endif
