/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2012-2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef OPTIMISER_OPTIMISER_HPP
#define OPTIMISER_OPTIMISER_HPP

#include <list>
#include <string>

#include "../error.hpp"
#include "../ast/astnode.hpp"
#include "../computespec/computespec.hpp"


class AstNode;
class AstNodeRoot;

class Optimiser : public TrackedClass
{
	public:
		static std::string PerformOptimisations(std::list<std::string> optimisations, AstNodeRoot *node, CompileContext &context);
		
		/*
		 * The return value of this is a report on the optimisations done
		 */
		virtual std::string Optimise(AstNodeRoot *root, CompileContext &context) = 0;
		
		/*
		 * This returns the name of the optimiser
		 */
		virtual std::string GetName() = 0;
		
	protected:
		static std::list<Optimiser *> GetOptimisers(std::list<std::string> optimisations);
		
		//Gets the path in the AST from src to dst (inclusive), or the empty list if no such path exists
		static std::list<AstNode *> FindPathToNode(AstNode *src, AstNode *dst);
		
		template <typename T> static T *FindParentOf(AstNode *top, AstNode *node)
		{
			std::list<AstNode *> path = FindPathToNode(top, node);
			
			T *parent = 0;
			for (AstNode *candidate: path){
				if (dynamic_cast<T *>(candidate)){
					parent = dynamic_cast<T *>(candidate);
				}
			}
			
			return parent;
		}
};

#endif
 
