/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "pattern.hpp"

PatternMatch::PatternMatch() :
	m_node(0), m_index(false), m_valid(false)
{
}

PatternMatch::PatternMatch(AstNode *node, std::list<PatternMatch> children, bool index) :
	m_node(node), m_index(index), m_valid(true), m_children(children)
{
}

PatternMatch::operator bool() const
{
	return m_valid;
}

PatternMatch PatternMatch::operator[](unsigned int depth)
{
	if (!depth && m_index && m_node){
		return *this;
	}
	else if (!m_children.size()){
		throw FormatError("Internal error: No child matches to access [%u left] with index", depth);
	}
	else {
		return m_children.front()[(m_index && m_node) ? (depth - 1) : depth];
	}
}

PatternMatch PatternMatch::operator()(unsigned int n, unsigned int depth)
{
	if (!n){
		if (!depth){
			return *this;
		}
		else if (!m_children.size()){
			throw FormatError("Internal error: No child matches to access (%u left) without index", depth);
		}
		else {
			return (m_children.front())(0, depth - 1);
		}
	}
	else {
		for (PatternMatch &match: m_children){
			if (!n){
				return match(0, depth);
			}
			n--;
		}
		throw FormatError("Internal error: could not find child");
	}
}


#if DEBUG
void PatternMatch::PrintMatchLeft(bool all)
{
	printf("======== Match ========\n");
	for (unsigned int i = 0; true; i++){
		try {
			AstNode *node = all ? (*this)(0, i) : (*this)[i];
			printf("%c%u%c: %s (%s)\n", all ? '(' : '[', i, all ? ')' : ']', node ? node->GetNormalName().c_str() : "Null", node ? node->GetClassName().c_str() : "Null");
		}
		catch (Error e) {
			break;
		}
	}
	printf("======== ----- ========\n");
}
#endif

