/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef OPTIMISER_PATTERN_HPP
#define OPTIMISER_PATTERN_HPP

#include <list>
#include <vector>

#include "../error.hpp"
#include "../ast/astnode.hpp"


class PatternMatch : public TrackedClass
{
	public:
		PatternMatch();
		PatternMatch(AstNode *node, std::list<PatternMatch> children, bool index);
		
		operator bool () const;
		template <typename T> operator T* () const
		{
			if (!m_node){
				return 0;
			}
			return tracked_cast<T>(m_node);
		}
		PatternMatch operator[](unsigned int depth); //Indexes into the match, only counting index = true
		PatternMatch operator()(unsigned int n, unsigned int depth = 0); //Indexes into the match, counting index = true|false
			//Indexes depth first, then n is the child index
		
		#if DEBUG
			void PrintMatchLeft(bool all);
		#endif
		
	private:
		AstNode *m_node;
		bool m_index;
		bool m_valid;
		std::list<PatternMatch> m_children;
};

#include "nodematcher.hpp"
#include "patternmatcher.hpp"


#endif
