/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2012-2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

%{
	#include "parser.hpp"
	#include "polycute.yy.hpp"
	
	#include "../error.hpp"
	
	#define YY_USER_ACTION yylloc->first_line = yylineno; yylloc->last_line = yylineno;
%}

%option header-file="src/parser/polycute.ll.hpp"
%option prefix="polycute_parser_"
%option reentrant noyywrap bison-bridge bison-locations never-interactive yylineno

%x MCOMMENT
%x SCOMMENT

%%

 /* Environments */
"/*" BEGIN(MCOMMENT);
"//" BEGIN(SCOMMENT);
"#" BEGIN(SCOMMENT);

 /* Keywords */
"spawn" {return SPAWN;}
"sync" {return SYNC;}

"void" {return VOID;}
"bool" {return BOOL;}
"int" {return INT;}
"float" {return FLOAT;}
"struct" {return STRUCT;}

"strict" {return STRICT;}
"signed" {return SIGNED;}

"export" {return EXPORT;}
"import" {return IMPORT;}
"ccall" {return CCALL;}

"return" {return RETURN;}

"true" {return TRUE;}
"false" {return FALSE;}
"null" {return NULL_REF;}

"if" {return IF;}
"else" {return ELSE;}
"while" {return WHILE;}
"for" {return FOR;}

"pointer" {return POINTER;}
"ref" {return REF;}
"deref" {return DEREF;}
"alloc" {return ALLOC;}
"dealloc" {return DEALLOC;}

"to" {return TO;}
"length" {return LENGTH;}

"lock" {return LOCK;}
"atomic" {return ATOMIC;}

 /* Operators */
"&" {return '&';}
"|" {return '|';}
"^" {return '^';}
"+" {return '+';}
"-" {return '-';}
"*" {return '*';}
"/" {return '/';}
"%" {return '%';}
"(" {return '(';}
")" {return ')';}
"[" {return '[';}
"]" {return ']';}
"{" {return '{';}
"}" {return '}';}
"=" {return '=';}
";" {return ';';}
"." {return '.';}
"," {return ',';}
"<" {return '<';}
">" {return '>';}
"<=" {return LTE;}
">=" {return GTE;}
"==" {return EQU;}
"!=" {return NEQ;}
"<<" {return SHL;}
">>" {return SHR;}
"->" {return SPLAT;}

 /*Patterns*/
[A-Za-z_][A-Za-z0-9_]* {
	snprintf(yylval->identifier, 1024, "%s", yytext); /*yylval->text = yytext;*/ return IDENTIFIER;
}

[0-9]+ {
	yylval->integer = atoi(yytext);
	/*yylval->text = yytext;*/ return INTEGER;
}

[0-9]*\.[0-9]+ {
	yylval->floating_point = atof(yytext);
	/*yylval->text = yytext;*/ return DECIMAL;
}

[ \t\r\n]+ /* Whitespace */

<MCOMMENT>{
	"*/" BEGIN(INITIAL);
	.|\n
}

<SCOMMENT>{
	\n BEGIN(INITIAL);
	.
}

 /* Error Handling */
. {
	PrintCompilerErrorYyl("", (*yylloc), "Unknown token '%s'", yytext);
	return 0;
}

%%

void __polycute_parser_SuppressWarning()
{
	yyunput(0, 0, 0);
}

