/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2012-2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <cstdio>

#include <limits.h>
#include <stdarg.h>

#include <sys/time.h>
#include <sys/timeb.h>

#include "error.hpp"
#include "util.hpp"

std::string PrintString(const char *format, ...)
{
	char b[1024];
	int maxlength = 1024;
	
	va_list argp;
	va_start(argp, format);
	int length = vsnprintf(b, maxlength, format, argp);
	va_end(argp);
	
	std::string r = b;
	while (length >= maxlength){
		maxlength = length + 1;
		char *hb = new char[maxlength];
		va_list argp2;
		va_start(argp2, format);
		length = vsnprintf(hb, maxlength, format, argp2);
		va_end(argp2);
		r = hb;
		delete []hb;
	}
	return r;
} 

unsigned int GetUniqueId()
{
	static unsigned int id = 1024; //Have 1024 IDs that are reserved and known free (useful for special metadata)
	if (UINT_MAX == id){
		throw FormatError("Internal limit reached for unique IDs");
	}
	id++;
	return id;
}

std::string Indent(std::string s, unsigned int n)
{
	if (!s.size()){
		return "";
	}
	
	std::string r;
	for (unsigned int i = 0; i < n; i++){
		r += '\t';
	}
	
	for (char c: s){
		r += c;
		if ('\n' == c){
			for (unsigned int i = 0; i < n; i++){
				r += '\t';
			}
		}
	}
	
	size_t p = r.find_last_not_of('\t');
	if (std::string::npos == p){
		return r;
	}
	
	return r.substr(0, p + 1);
}

int64_t GetMilliTime()
{
	struct timeb t;
	ftime(&t);
	return (int64_t)t.time * (int64_t)1000 + (int64_t)t.millitm;
}

void WriteToFile(std::string text, std::string filename)
{
	FILE *f = ("-" == text) ? stdout : fopen(filename.c_str(), "w");
	if (!f){
		throw FormatError("Could not open \"%s\"", filename.c_str());
	}
	
	fprintf(f, "%s", text.c_str());
	
	if ("-" != text){
		fclose(f);
	}
}
