/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2012-2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "stdio.h"
#include "stdint.h"
#include "stdlib.h"


void CalculateMandlebrot(uint8_t *data, uint32_t width, uint32_t height);

typedef struct Image
{
	uint8_t *data;
	unsigned int width;
	unsigned int height;
} Image;

Image CreateImage(unsigned int width, unsigned int height)
{
	Image img;
	img.width = width;
	img.height = height;
	img.data = malloc(width * height * 3);
	return img;
}

void FreeImage(Image img)
{
	free(img.data);
}

uint8_t GetPixel(Image img, unsigned int x, unsigned int y, unsigned int c)
{
	return img.data[(y * img.width + x) * 3 + c];
}

void Error(const char *s)
{
	fprintf(stderr, "Error: %s\nExiting\n", s);
	exit(1);
}

void WritePmn(Image img)
{
	printf("P6\n%u %u\n255\n", img.width, img.height);
	for (unsigned int y = 0; y < img.height; y++){
		for (unsigned int x = 0; x < img.width; x++){
			for (unsigned int c = 0; c < 3; c++){
				uint8_t v = GetPixel(img, x, y, c);
				if (fwrite(&v, 1, 1, stdout) != 1){
					Error("Could not write image to standard out");
				}
			}
		}
	}
}

Image CalculateMandlebrotImage()
{
	unsigned int size_factor = 128;
	Image img = CreateImage(7*size_factor, 4*size_factor);
	fprintf(stderr, "[+] Calculating mandlebrot\n");
	CalculateMandlebrot(img.data, img.width, img.height);
	fprintf(stderr, "[+] Finished calculating mandlebrot\n");
	return img;
}

int main(int argc, char **argv)
{
	Image img = CalculateMandlebrotImage();
	WritePmn(img);
	FreeImage(img);
	return 0;
}
 
