/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2012  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "stdint.h"

void SetPixel(uint8_t *data, uint32_t width, uint32_t x, uint32_t y, uint32_t c, uint8_t v)
{
	data[(y * width + x) * 3 + c] = v;
}

//This function is based on the pseudocode from:
//http://en.wikipedia.org/w/index.php?title=Mandelbrot_set&oldid=521415115#For_programmers
void CalculateMandlebrot(uint8_t *data, uint32_t width, uint32_t height)
{
	unsigned int factor = 16;
	for (unsigned int imgy = 0; imgy < height; imgy++){
		for (unsigned int imgx = 0; imgx < width; imgx++){
			unsigned int total = 0;
			for (unsigned int suby = 0; suby < factor; suby++){
				for (unsigned int subx = 0; subx < factor; subx++){
					float x0 = -2.5 + (3.5 * (float)(imgx * factor + subx)/(float)(width * factor));
					float y0 = -1.0 + (2.0 * (float)(imgy * factor + suby)/(float)(height * factor));
					float x = 0.0;
					float y = 0.0;
					
					unsigned int i = 0;
					for (; i < 255 && (x*x + y*y < 4); i++){
						float xtemp = x*x - y*y + x0;
						y = 2.0*x*y + y0;
						x = xtemp;
					}
					
					total += i;
				}
			}
			
			total /= factor*factor;
			if (total > 255){
				total = 255;
			}
			
			SetPixel(data, width, imgx, imgy, 0, total);
			SetPixel(data, width, imgx, imgy, 1, total);
			SetPixel(data, width, imgx, imgy, 2, total);
		}
	}
}