/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <cstdio>
#include <cinttypes>
#include <random>

#include <sys/time.h>
#include <sys/timeb.h>

extern "C"
{
	void Mem(uint64_t n, uint32_t *array);
}


static void Generate(uint64_t n, uint32_t *array)
{
	std::uniform_int_distribution<uint32_t> distribution(0, 0xFFFFFFFF);
	std::mt19937 engine; // Mersenne twister MT19937
	
	for (uint64_t i = 0; i < n; i++){
		array[i] = distribution(engine);
	}
}

static void PrintArray(uint64_t n, uint32_t *array)
{
	for (uint64_t i = 0; i < n; i++){
		printf("\t%" PRIu32 "\n", array[i]);
	}
}

static void Test(uint64_t n, bool print)
{
	printf("[*] Allocating\n");
	uint32_t *array = new uint32_t[n];
	printf("[*] Generating\n");
	Generate(n, array);
	
	if (print){
		printf("[*] Printing initial array\n");
		PrintArray(n, array);
	}
	
	printf("[*] Running Polycute code\n");
	Mem(n, array);
	
	if (print){
		printf("[*] Printing Polycuted array\n");
		PrintArray(n, array);
	}
	
	printf("[*] Deallocating\n");
	delete []array;
}

int main(int argc, char **argv)
{
	Test(100, false);
	return 0;
}
