/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef ERROR_HPP
#define ERROR_HPP

#include <cstdio>
#include <exception>
#include <stdarg.h>

#ifdef DEBUG
	#define ERROR_POS_DEFAULT true
#else
	#define ERROR_POS_DEFAULT false
#endif

#define ERROR_MSG_SIZE 4096
#define FormatError(format, args...) Error(__FILE__, __LINE__, format, ## args)
#define TestRealValue(v) if ((isnan(v)) || (isinf(v)))\
	{throw FormatError("Real value is not valid: %f!", v);}

#ifdef DEBUG
	#define AssertError(condition, format, args...) if (!(condition)) throw Error(__FILE__, __LINE__, format, ## args)
#else
	#define AssertError(condition, format, args...) /*Do nothing*/
#endif

class Error
{
	public:
		Error(const char *pfile, int pline, const char *format, ...)
		{
			file = pfile;
			line = pline;
			
			va_list argp;
			va_start(argp, format);
			vsnprintf(str, ERROR_MSG_SIZE, format, argp);
			va_end(argp);
			
			s_current = this;
			
			/*int *p = 0;
			p[0] = 0;*/
		}
		void OutputMessage(bool pos = ERROR_POS_DEFAULT, FILE *f = stderr)
		{
			if (pos){
				fprintf(f, "Error in file %s at line %i: %s\n", file, line, str);
			}
			else {
				fprintf(f, "Error: %s\n", str);
			}
		}
		
	private:
		static void TerminateHandler()
		{
			if (s_current){
				s_current->OutputMessage();
			}
			else {
				fprintf(stderr, "Current error is zero");
			}
		}
		
		static Error *InitialiseCurrent()
		{
			std::set_terminate(TerminateHandler);
			return 0;
		}
		
		char str[ERROR_MSG_SIZE];
		const char *file;
		int line;
		
		static Error *s_current;
};
Error *Error::s_current = InitialiseCurrent();

#endif
