/*
	Polycute - The reference Polycute implementation
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <cstdio>
#include <cinttypes>
#include <random>

#include <sys/time.h>
#include <sys/timeb.h>

extern "C"
{
	int Sort(uint64_t n, uint32_t *a1, uint32_t *a2);
	
	uint32_t *AddPointer(uint32_t *p, uint64_t offset)
	{
		return p + offset;
	}
}

static int64_t GetMilliTime()
{
	struct timeb t;
	ftime(&t);
	return (int64_t)t.time * (int64_t)1000 + (int64_t)t.millitm;
}

static void Generate(uint64_t n, uint32_t *array)
{
	std::uniform_int_distribution<uint32_t> distribution(0, 0xFFFFFFFF);
	std::mt19937 engine; // Mersenne twister MT19937
	
	for (uint64_t i = 0; i < n; i++){
		array[i] = distribution(engine);
	}
}

static void PrintArray(uint64_t n, uint32_t *array)
{
	for (uint64_t i = 0; i < n; i++){
		printf("\t%" PRIu32 "\n", array[i]);
	}
}

static void Test(uint64_t n, bool print)
{
	fprintf(stderr, "[-] Allocating Memory\n");
	uint32_t *a1 = new uint32_t[n];
	uint32_t *a2 = new uint32_t[n];
	fprintf(stderr, "[-] Generating Random Numbers\n");
	Generate(n, a1);
	
	if (print){
		printf("======== Unsorted ========\n");
		PrintArray(n, a1);
	}
	
	fprintf(stderr, "[+] Sorting\n");
	int64_t a = GetMilliTime();
	Sort(n, a1, a2);
	int64_t b = GetMilliTime();
	
	if (print){
		printf("======== Sorted ========\n");
		PrintArray(n, a1);
	}
	
	fprintf(stderr, "[+] Time taken to sort: %0.03f s\n", (double)(b-a)/1000.0);
	
	delete []a1;
	delete []a2;
}

int main(int argc, char **argv)
{
	//Test(16, true);
	Test(1000, true);
	Test(1000000, true);
	Test(100000000, false);
	
	return 0;
}
