#!/usr/bin/python

#
#	Polycute - The reference Polycute implementation
#	Copyright (C) 2013  Nicholas Tomlinson
#	
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#	
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#	
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


import re
import subprocess
import sys


def Execute(cmd):
	p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
	stdout,stderr = p.communicate()
	return (stdout.decode("utf-8"), stderr.decode("utf-8"))

def Indent(s, n):
	r = ""
	for i in range (n):
		r += "\t"
	for c in s:
		r += c
		if ('\n' == c):
			for i in range (n):
				r += "\t"
	return r

def Column(a, b, n):
	r = ""
	i = 0
	for c in a:
		r += c
		i += 1
	while (i < n):
		r += ' '
		i += 1
	r += b
	return r

def GetExpandMod():
	additional = set()
	removal = set()
	terminate = 0

	for i in range(3, len(sys.argv)):
		v = sys.argv[i]
		if ("+" == v[0]):
			additional |= {int(v[1:])}
		elif ("-" == v[0]):
			removal |= {int(v[1:])}
		elif ("/" == v[0]):
			terminate = int(v[1:])
		else:
			print ("Invalid argument")
			exit()
	return (additional,removal,terminate)

def GetMainArgs():
	llvm_input = sys.argv[1]
	patern = sys.argv[2]
	regex = re.compile(patern, re.DOTALL)
	return (llvm_input,regex)

def ProcessDumps(dumps, regex, terminate):
	dump_index = 0
	for dump in dumps:
		if (terminate != 0) and (dump_index >= terminate):
			return
		
		if (len(dump)):
			title,body = dump.split("\n", 1)
			title = title.split(" ***")[0]
			dump_index += 1
			
			if (dump_index in additional):
				print ("[+] " + Column(title, "(" + str(dump_index) + ")", 150) + "\n", Indent(body, 1))
			else:
				try:
					body = regex.search(body).group(0)
					if (dump_index in removal):
						print ("[-] " + Column(title, "(" + str(dump_index) + ")", 150))
					else:
						print ("[+] " + Column(title, "(" + str(dump_index) + ")", 150) + "\n", Indent(body, 1))
				except:
					print ("[-] " + Column(title, "(" + str(dump_index) + ")", 150))


llvm_input,regex = GetMainArgs()
additional,removal,terminate = GetExpandMod()
stdout,stderr = Execute(['opt', '-S', '-std-compile-opts', '-print-before-all', '-print-after-all', '-o', '-', llvm_input])
dumps = stderr.split("*** IR Dump ")
ProcessDumps(dumps, regex, terminate)