#!/usr/bin/python

#
#	Polycute - The reference Polycute implementation
#	Copyright (C) 2013  Nicholas Tomlinson
#	
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#	
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#	
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import re
import sys

def Error(string):
	sys.stderr.write("Error: " + string)
	exit()

def Indent(s, n):
	r = ""
	for i in range (n):
		r += "\t"
	for c in s:
		r += c
		if ('\n' == c):
			for i in range (n):
				r += "\t"
	return r

def Column(a, b, n):
	r = ""
	i = 0
	for c in a:
		r += c
		i += 1
	while (i < n):
		r += ' '
		i += 1
	r += b
	return r

def LoadString():
	r = ""
	for line in sys.stdin:
		r += line
	return r

def GetArguments():
	match_pattern = None
	reject_pattern = None
	split_pattern = None
	min_n = None
	max_n = None
	
	previous = None
	for arg in sys.argv[1:]:
		if (None == previous):
			previous = arg
		elif ("-m" == previous):
			previous = None
			if (None != match_pattern):
				Error("Match already specified")
			match_pattern = re.compile(arg, re.DOTALL)
		elif ("-r" == previous):
			previous = None
			if (None != reject_pattern):
				Error("Reject already specified")
			reject_pattern = re.compile(arg, re.DOTALL)
		elif ("-s" == previous):
			previous = None
			if (None != split_pattern):
				Error("Split already specified")
			split_pattern = re.compile(arg, re.DOTALL)
		elif ("-a" == previous):
			previous = None
			if (None != min_n):
				Error("Minimum already specified")
			min_n = int(arg)
		elif ("-b" == previous):
			previous = None
			if (None != max_n):
				Error("Maximum already specified")
			max_n = int(arg)
		else:
			if (None != match_pattern):
				Error("Unknown parameter: " + previous)
	
	if (None != previous):
		Error("Argument expected after: " + previous)
	
	if None == split_pattern:
		split_pattern = re.compile("\n", re.DOTALL)
		
	if None == min_n:
		min_n = 0
		
	if None == max_n:
		max_n = 2**62
	
	return (match_pattern, reject_pattern, split_pattern, min_n, max_n)

def PerformSearch(complete_text, match_pattern, reject_pattern, split_pattern, min_n, max_n):
	texts = split_pattern.split(complete_text)
	i = 0
	for text in texts:
		i += 1
		if ((match_pattern == None) or (match_pattern.match(text))) and ((reject_pattern == None) or (not reject_pattern.match(text))) and (i <= max_n) and (i >= min_n):
			print (Column("[*]Found Match", str(i), 32) + "\n" + Indent(text, 1) + "\n")

print ("[*] Getting Arguments")
match_pattern, reject_pattern, split_pattern, min_n, max_n = GetArguments()
print ("[*] Loading Text")
complete_text = LoadString()
print ("[*] Searching")
PerformSearch(complete_text, match_pattern, reject_pattern, split_pattern, min_n, max_n)
print ("[*] Done")
